/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2023                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  26359   06/14/18  Remove "min" & "max" for PJ45111               */
/*  d26214  01/03/19  Add option for version to fix driver with mq   */
/*                    client (ZMQID)                                 */
/*  d26533  11/20/19  z/TPF WebSphere MQ support for Dynamic Routing */
/*  26653   10/28/20  Add option on LONGRUNNING and MQGET commands to*/
/*                    validate files.                                */
/*  d26841  20230927  Driver updates for 64-bit MQ                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....MQS1                                            */
/* FUNCTION TYPE.....TPF                                             */
/* SEGMENT NAME....  qmqz.cpp                                        */
/*                                                                   */
/* DESCRIPTION:                                                      */
/*   QMQZ is called from "ZTEST MQS1" on the TPF console.            */
/*   It is the driver to test MQ functions; QMQZ determines the      */
/*   command and jumps to the appropriate function in another        */
/*   QMQZXX file.                                                    */
/*                                                                   */
/*********************************************************************/

#include <qmqz.h>
#include <tpf/c_devops.h>                                                       //d26725

int              TraceVar = 0;
char             outputbuff[200] = {0};
char             outputbuff2[MAX_ERR_MSG_BUF + 1] = {0};
MQHCONN          MQueue::connectionHandle = MQHC_UNUSABLE_HCONN;
wtopc_header     QMQZ_Header = {0};
boolean          automatedTestCase = false;                                     //d26725

static void   showEventMsg(void* pMsg);
static MQLONG showMQCFH(MQCFH* pStruc);
static MQLONG showMQCFIN(MQCFIN* pStruc);
static MQLONG showMQCFST(MQCFST* pStruc);

static const sCommandTable CommandTable[] =
{
  { "HELP"        , QMQZ_Help       },   // QMQZ
  { "u"           , QMQZ_Help       },   // QMQZ
  { ""            , QMQZ_Help       },   // QMQZ
  { "MQPUT"       , QMQZ_Put        },   // QMQZ01
  { "MQGET"       , QMQZ_Get        },   // QMQZ01
  { "ZMQSC"       , QMQZ_ZMQSC      },   // QMQZ01
  { "ROLL"        , QMQZ_CommRoll   },   // QMQZ01
  { "COMM"        , QMQZ_CommRoll   },   // QMQZ01
  { "MQCOMM"      , QMQZ_CommRoll   },   // QMQZ01
  { "MQROLL"      , QMQZ_CommRoll   },   // QMQZ01
  { "CONtention"  , QMQZ_Contention },   // QMQZ01
  { "DISplay"     , QMQZ_Display    },   // QMQZ01
  { "MQINQ"       , QMQZ_Inq        },   // QMQZ01
  { "MQSET"       , QMQZ_Set        },   // QMQZ01
  { "MQPUT1"      , QMQZ_Put1       },   // QMQZ01
  { "ERRor"       , QMQZ_ErrorTest  },   // QMQZ02
  { "REQuest"     , QMQZ_Request    },   // QMQZ03
  { "REPly"       , QMQZ_Reply      },   // QMQZ03
  { "CHeckpoint"  , QMQZ_CheckPoint },   // QMQZ05
  { "CASE"        , QMQZ_Suite      },   // QMQZ06
  { "LONGRunning" , QMQZ_LongRunning}    // QMQZ07
};

extern "C" void QMQZ()                   // Entrance Point
{
  QMQZ_Header.wtopc_time = WTOPC_NO_TIME;
  QMQZ_Header.wtopc_number = ((int) ecbp2()->ce2sva / 4096) % 10000;
  int i = 0;                             // start of commands
  CheckSystemState();                    // Should be above 1052
  int     num_params;

  // Switch IStream to balance -- Comment out as part of d26653 as we want to use the
  // Longrunning driver and place instances on specific istreams.  If we need balancing re-enabled we
  // can uncomment in the future.
//  struct swisc_immediate_input swisc_is;                                        //d26533
//  swisc_is.istream = SWISC_IS_BALANCE;                                          //d26533
//  swisc_immediate(&swisc_is);                                                   //d26533

  /* If this ECB is invoked by a longrunning api-SINGLE option, */
  /* bypass the parsing and go to the function right the way.   */
  if (ecbp2()->ce2crsmsg == NULL)
  {
     QMQZ_LongRunning_noparser((NOPARSE*)ECB_CBRW_CA(D0));
     exit(0);
  }

  static sInput Input[INPUT_SIZE];       // input array
  struct IPRSE_output res;               // parsed input list

  num_params = IPRSE_parse(ecbp2()->ce2crsmsg, QMQZGrammar, &res,
                           IPRSE_ALLOC | IPRSE_PRINT, "QMQZ");

  // ztest mqs1 = 2 parameters
  if (num_params == IPRSE_HELP || num_params == 2)
     QMQZ_Help(Input);
  else if (num_params == IPRSE_BAD)
    exit(1);

  BuildInput(res.IPRSE_next->IPRSE_next, Input); // Array of input parameters
  //PrintInput(Input);                           // For debugging purposes

  while (Input[0].Param != CommandTable[i].Command) i++;  // search table
  CommandTable[i].Function(Input);               // Call function
  exit(0);                                       // Free ECB, normal exit
}

void QMQZ_Help(sInput* Input)
{
  String command(Input[1].Param.C());

  if (command == "")
  {
    outc("QMQZ 0.1.5.1 Syntax:                                       ");
    outc("  ZTEST MQS1 (TRACE|NOPRT) function functionArguments      ");
    outc("  Where function is one of the following:                  ");
    outc("    MQPUT         MQPUT1        MQGET         MQSET        ");
    outc("    MQINQ         COMM          ROLL          CONtention   ");
    outc("    REQuest       REPly         DISplay       CASE         ");
    outc("    ERRor         CHeckpoint    LONGRUNNING                ");
    outc("  And functionArguments can be determined by typing        ");
    outc("    ZTEST MQS1 HELP function                               ");
    outc("  Option TRACE will output every MQ API executed           ");
    oute("  Option NOPRT will suppress all output except error output");
  }
  else if (command == "HELP" || command == "FUNCTION")
  {
    outc("ZTEST MQS1 HELP function                                   ");
    outc("  Displays help information for function in QMQZ           ");
    outc("  Examples:                                                ");
    outc("    ZTEST MQS1 HELP MQPUT                                  ");
    outc("    ZTEST MQS1 HELP REP                                    ");
    oute("    ZTEST MQS1 HELP CH                                     ");
  }
  else if (command == "MQPUT")
  {
    outc("ZTEST MQS1 MQPUT qmanager qname (qmgra) Nmsg-w LENgth-x ...");
    outc("        (NO)PER (DATA-abcd) (MSGid-m) (CORrelid-c) ...     ");
    outc("        (EXpiry-e) (SCOPE-Commit|Rollback SCOPESLeep-p) ...");
    outc("        (ECB-y | RIPL-z) (File-u) (Comp-l)                 ");        //d26533
    outc("Places w messages of length x on qname defined in qmanager.");
    outc("PER defines the messages to be persistent.                 ");
    outc("NOPER defines the messages to be not persistent.           ");
    outc("Default is queue's DEFPSIST.                               ");
    outc("DATA overlays first 4 bytes of msg data. Default is QMQZ.  ");
    outc("MSGID and CORRELID can be specified for the msg.           ");
    outc("If e is specified, it is set as the ExpiryInterval value in");
    outc("the Put Message Options and MQPMO_WAIT is set.  e can be   ");
    outc("negative by prefixing it with 'n.' (e.g. n.2 = -2).        ");
    outc("You can also specify Unlimited (minimum abbreviation U)    ");
    outc("to set the wait interval to MQEI_UNLIMITED (-1).           ");
    outc("SCOPE defines commit scope for all MQPUTs in a transaction.");
    outc("SCOPESLeep specifies the elapsed time between the last     ");
    outc("MQPUT and the tx_commit/tx_rollback.                       ");
    outc("If y is specified, y separate ECBs execute the operation,  ");
    outc("each putting w messages onto the queue.                    ");
    outc("If z is specified, the system will re-IPL after            ");
    outc(" the zth message is placed.                                ");
    outc("If z is the total number of messages, the IPL occurs after ");
    outc("the queue is closed.  qmgra specifies the                  ");
    outc("File gives the option to use a file for message content.   ");        //d26533
    oute("Comp allows you to specify compression level (1-10)        ");        //d26533
  }
  else if (command == "MQGET")
  {
    outc("ZTEST MQS1 MQGET qmanager qname Nmsg-w|Sec-w LENgth-x ...  ");
    outc("      ((NO)PRT|PRTN-n) (WAIT-z) (NO)SEQcheck  ...          ");
    outc("      (BRowse-READ|GET|LOCK SLEEP-s WAKEUP-EXIT|SERRC|LOCK|");
    outc("       LOCKN|UNLOCK|BROWSE|BRN|GET) (MSGid-m) (CORrelid-c) ");
    outc("      (SCOPE-Commit|Rollback SCOPESLeep-p) (Ver-v) (ECB-y) ");        //d26214
    outc("      (File-u)                                             ");        //26653
    outc("Removes w messages from qname defined in qmanager.         ");
    outc("PRT outputs the first x characters of the messages         ");
    outc("removed, NOPRT prevents output of message content.         ");
    outc("PRTN outputs the first n messages.  Default is to output   ");
    outc("all messages only if w is not greater than 25.             ");
    outc("If y is specified, y separate ECBs execute the             ");
    outc("operation, each getting w messages from the queue.         ");
    outc("If Sec-w is specified instead of Nmsg-w, messages are      ");
    outc("removed from the queue until w seconds elapse.             ");
    outc("If z is specified, it is set as the WaitInterval value in  ");
    outc("the Get Message Options and MQGMO_WAIT is set.  z can be   ");
    outc("negative by prefixing it with 'n.' (e.g. n.2 = -2).        ");
    outc("You can also specify Unlimited (minimum abbreviation U)    ");
    outc("to set the wait interval to MQWI_UNLIMITED (-1).           ");
    outc("SEQ checks the messages for correct sequence.              ");
    outc("BRowse with READ browses w messages.  GET removes the wth  ");
    outc("message. LOCK locks the last message.  SLEEP specifies the ");
    outc("period between the first BROWSE actopm and WAKEUP action.  ");
    outc("MSGID and CORRELID can be specified to search for match.   ");
    outc("SCOPE defines commit scope for all MQPUTs in a transaction.");
    outc("SCOPESLeep specifies the elapsed time between the last     ");
    outc("MQPUT and the tx_commit/tx_rollback.                       ");        //26653
    outc("Ver can be used to change the MQGMO version. (Default 2)   ");        //26653
    outc("File gives the option to use a file for validating message ");        //26653
    oute("  content.");                                                         //26653
  }
  else if (command == "MQINQ")
  {
    outc("ZTEST MQS1 MQINQ QMGR|qname|qaliasname|processname         ");
    outc("Outputs information about qname, qaliasname, processname,  ");
    oute("or about the running Queue Manager.                        ");
  }
  else if (command == "MQSET")
  {
    outc("ZTEST MQS1 MQSET qname|qaliasname                          ");
    outc(" PUT|GET|TRIGGER|TRIGTYPE|TRIGDATA                         ");
    outc(" ALLOWED|INHIBITED|ON|OFF|FIRST|EVERY|NONE|data            ");
    oute("Set various attributes of a queue or qaliasname            ");
  }
  else if (command == "MQPUT1")
  {
    outc("ZTEST MQS1 MQPUT1 qmanager qname (qmgralias) LENgth-x ...  ");
    outc("        (NO)PER (DATA-abcd) (MSGid-m) (CORrelid-c) ...     ");
    outc("        (EXpiry-e) (SCOPE-Commit|Rollback SCOPESLeep-p) ...");
    outc("        (ECB-y) (File-u) (Comp-l)                          ");        //d26533
    outc("Places one message of length x on qname                    ");
    outc(" defined in qmanager.                                      ");
    outc("PER defines the message to be persistent.                  ");
    outc("NOPER defines the message to be not persistent.            ");
    outc("Default is queue's DEFPSIST.                               ");
    outc("DATA overlays first 4 bytes of msg data. Default is QMQZ.  ");
    outc("MSGID and CORRELID can be specified for the msg.           ");
    outc("If e is specified, it is set as the ExpiryInterval value in");
    outc("the Put Message Options and MQPMO_WAIT is set.  e can be   ");
    outc("negative by prefixing it with 'n.' (e.g. n.2 = -2).        ");
    outc("You can also specify Unlimited (minimum abbreviation U)    ");
    outc("to set the wait interval to MQEI_UNLIMITED (-1).           ");
    outc("SCOPE defines commit scope for all MQPUTs in a transaction.");
    outc("SCOPESLeep specifies the elapsed time between the last     ");
    outc("MQPUT and the tx_commit/tx_rollback.                       ");
    outc("If y is specified, y separate ECBs execute the operation   ");
    outc("(Each putting one message).  qmgralias specifies           ");
    outc(" the queue manager alias.                                  ");        //d26533
    outc("File gives the option to use a file for message content.   ");        //d26533
    oute("Comp allows you to specify compression level (1-10)        ");        //d26533
  }
  else if (command == "CASE")
  {
    if (Input[2].Param != "")                         // User wants code
    {
      outc("I've changed the syntax for HELP CASE...                 ");
      outc("Type ZTEST MQS1 CASE PRINT XXX ,                         ");
      outc("where 'ZTEST MQS1 CASE XXX' will be run                  ");
      outc("Example:                                                 ");
      outc("  ZTEST MQS1 CASE PRINT 1 Q                              ");
      oute("  will output test case 1                                ");
      exit(0);
    }

    outc("ZTEST MQS1 CASE (PRINT) ...                                ");
    outc("  casenumber emptyq (emptyq emptyq) (MQONLY)               ");
    outc("  ALL emptyqueuename (emptyq emptyq)(MQONLY)               ");
    outc("  RANDOM emptyqueuename (emptyq emptyq) (MQONLY)           ");
    outc("  TIME seconds emptyqueuename (emptyq emptyq) (MQONLY)     ");
    outc("  RANGE emptyq (emptyq emptyq) (MQONLY) Low-x Hi-y         ");
    outc("  RIGOR1|RIGOR2|RIGOR3 q1 (q2) (q3)...(q6) (Low-x) Hi-y    ");
    outc("    QM-z may be specified to use QMGR x instead of local QM");
    outc("                                                           ");
    outc("ZTEST MQS1 CASE ... runs cases in a test suite             ");
    outc("Option PRINT will output the code that would be executed   ");
    outc("  instead of executing it.                                 ");
    outc("Option MQONLY prevents any DASD transactions               ");
    outc("The RIGOR test cases are extremely large, look in the docs ");
    oute("  to see what they do.                                     ");
  }
  else if (command == "REPly")
  {
    outc("ZTEST MQS1 REPly qmanager getfromq Nmsg-x LENgth-y ECB-z   ");
    outc("Sends reply messages to MQ request messages by removing    ");
    outc("x messages from getfromq defined in qmanager and           ");
    outc("replying to each request message found..                   ");
    outc("If z is specified, z separate ECBs execute                 ");
    oute("the operation, each replying to x messages.                ");
  }
  else if (command == "ZMQSC")
  {
    outc("ZTEST MQS1 ZMQSC [zmqsc arguments with _ instead of -] ... ");
    outc("                               LOW-x HI-y                  ");
    outc("Runs many ZMQSC commands, each command the same as the     ");
    outc("arguments entered, except numbers from x to y are appended ");
    outc("to the end of the second argument                          ");
    outc("Example:                                                   ");
    outc("  ZTEST MQS1 ZMQSC DEF QL_LOCALQ COMMON_YES LOW-4 HI-7     ");
    outc("will run:                                                  ");
    outc("  ZMQSC DEF QL-LOCALQ004 COMMON-YES                        ");
    outc("  ZMQSC DEF QL-LOCALQ005 COMMON-YES                        ");
    outc("  ZMQSC DEF QL-LOCALQ006 COMMON-YES                        ");
    oute("  ZMQSC DEF QL-LOCALQ007 COMMON-YES                        ");
  }
  else if (command == "COMM" || command == "MQCOMM")
  {
    outc("ZTEST MQS1 COMM qmanager qname (qmgra) Nmsg-w ...          ");
    outc("                 LENgth-x (NO)PER COmmitrate-y (RIPL-z)    ");
    outc("Places w messages of length x and gets < x messages of     ");
    outc("length x on qname defined in qmanager through transactions.");
    outc("For every yth message placed, y-1 messages are removed     ");
    outc("from the queue and a tx_commit commits the transaction.    ");
    outc("PER defines the messages to be persistent (default).       ");
    outc("NOPER defines the messages to be not persistent.           ");
    outc("If z is specified, the system will re-IPL after            ");
    outc("the zth message is placed.  If z is the total number       ");
    outc("of messages, the IPL occurs after the queue is closed.     ");
    outc("If z is the last message before a commit, the IPL          ");
    outc("occurs after the transaction is committed.                 ");
    oute("qmgra specifies the queue manager alias.                   ");
  }
  else if (command == "REQuest")
  {
    outc("ZTEST MQS1 REQuest qmanager puttoq getfromq ...            ");
    outc("                       Nmsg-x LENgth-y ECB-z (NO)PER       ");
    outc("Sends x request type messages of length y                  ");
    outc("to a destination queue manager qmanager, and               ");
    outc("a defined queue puttoq.  The user must also specify        ");
    outc("the reply-to queue getfromq to which the original          ");
    outc("message will be echoed.  If z is specified,                ");
    outc("z separate ECBs execute the operation,                     ");
    oute("each sending x messages.                                   ");
  }
  else if (command == "DISplay")
  {
    outc("ZTEST MQS1 DISplay TABle                                   ");
    outc("  Displays the core addresses of MQ Tables, including:     ");
    outc("   Queue Manager MQT_TO2 Table,                            ");
    outc("   Queue Definition Table,                                 ");
    outc("   and Channel Definition Table.                           ");
    outc("ZTEST MQS1 DISplay CHANnels                                ");
    outc("  Displays the name of all channels that have a definition ");
    outc("  in core.                                                 ");
    outc("ZTEST MQS1 DISplay queuename                               ");
    outc("  Displays the core address of queue queuename             ");
    outc("ZTEST MQS1 DISplay queuename Msg-x                         ");
    outc("  Displays information about the xth message in memory     ");
    outc("  on queue queuename.                                      ");
    outc("ZTEST MQS1 DISplay MEM hexaddress bytes                    ");
    oute("  Same as ZDCOR, but better!                               ");
  }
  else if (command == "ROLL" || command == "MQROLL")
  {
    outc("ZTEST MQS1 ROLL qmanager qname (qmgra) Nmsg-w ...          ");
    outc("            LENgth-x (NO)PER ROllbackrate-y (RIPL-z)       ");
    outc("Places w messages of length x on qname defined in qmanager ");
    outc("through transactions.  For every yth message placed, y-1   ");
    outc("messages are removed from the queue and tx_rollback        ");
    outc("prevents the transaction from going to the queue.          ");
    outc("PER defines the messages to be persistent (default).       ");
    outc("NOPER defines the messages to be not persistent.           ");
    outc("If z is specified, the system will re-IPL                  ");
    outc("after the zth message is placed.  If z is the total        ");
    outc("number of messages, the IPL occurs after the               ");
    outc("queue is closed.  If z is the last message before          ");
    outc("a rollback, the IPL occurs after the transaction           ");
    oute("is rolled back.  Qmgra specifies the queue manager alias.  ");
  }
  else if (command == "ERRor")
  {
    outc("ERRor MQCONN|MQOPEN|MQPUT|MQGET|MQINQ qmanager qname (case)");
    outc("      MQPUT1|MQSET|MQCLOSE|MQDISC qmanager qname (case)    ");
    outc("Runs a set of MQ API calls specifically                    ");
    oute("  testing error return paths.                              ");
  }
  else if (command == "CONtention")
  {
    outc("ZTEST MQS1 CONtention qname ECBs-w (NO)PER                 ");
    outc("  Creates w ECBs putting and getting to the same queue.    ");
    outc("  Each ecb puts x messages and gets w-x messages, where    ");
    oute("  x is the xth ECB created.                                ");
  }
  else if (command == "CHeckpoint")
  {
    outc("ZTEST MQS1 CHeckpoint Messages                             ");
    outc("  Displays total number of messages in the checkpoint.     ");
    outc("  Also shows the number of messages in each queue in       ");
    outc("  the checkpoint, separated into messages prior            ");
    outc("  to sweep and messages after sweep.                       ");
    outc("ZTEST MQS1 CHeckpoint NUMQueues                            ");
    outc("  Displays total number of queues in the checkpoint.       ");
    outc("ZTEST MQS1 CHeckpoint NUMChannels                          ");
    outc("  Displays total number of channels in the checkpoint.     ");
    outc("ZTEST MQS1 CHeckpoint NUMRecords                           ");
    outc("  Displays total number of records used to store the       ");
    outc("  checkpoint.                                              ");
    outc("ZTEST MQS1 CHeckpoint ALLQueuenames                        ");
    outc("  Displays the name of each queue in the checkpoint.       ");
    outc("ZTEST MQS1 CHeckpoint ALLChannelnames                      ");
    outc("  Displays the name of each channel in the checkpoint.     ");
    outc("ZTEST MQS1 CHeckpoint FIND queuename|channelname           ");
    outc("  Searches the checkpoint for any queues or                ");
    oute("  channels by the name queuename or channelname            ");
    }
   else if (command == "LONGRUNNING") {
    outc("ZTEST MQS1 LONGRUNNING qmanager qname SLEEPinterval-x      ");
    outc("        (LOOPCount-y | LOOPTime-t)                         ");
    outc("        (API-PUTGET|PUT|PUT1|GET|GETBY|BROWSE|LOCK|SINGLE) ");        //d26533
    outc("        (LENgth-l) (ECB-z) (Util-u) (Ver-v) (File-f)       ");        //d26533
    outc("        (Comp-c) (VALidate)                                ");        //26653
    outc("  Creates z ECBs putting and getting to the same queue     ");
    outc("  continuously. Between each MQPUT/MQGET, the ECB waits x  ");
    outc("  microseconds before the next MQ operation. If Util is    ");
    outc("  specified then the driver will attempt to use up that    ");
    outc("  percentage of the I-stream it is running on. Loop y      ");        //d26533
    outc("  iterations or for t seconds. Ver-v GMO version, (Def 2)  ");        //d26533
    outc("  Client uses 1. File gives the option to use a file for   ");        //d26533
    outc("  message contents. Comp level c, (1-10). Validate gives   ");        //26653
    oute("  the option to use a file for validating message content. ");        //26653
    }   // end of switch
  else Error("QMQZ HELP: Function name not valid");

  exit(0);
}  // end of help


void PrintInput(sInput *Input)                  // Prints out all input
{
  char outp[100];

  outc("Index:            Input(Index).Param:  Input(Index).Value:");

  for (int i = 0; i < INPUT_SIZE; i++)
  {
    sprintf(outp,"%5d %30s %20s", i, Input[i].Param.C(),
            Input[i].Value.C());
    outc(outp);
  }

  oute("End of Input");
}


void BuildInput(struct IPRSE_output *in, sInput *Input)
{
  int i = 0;

  if (streq(in->IPRSE_parameter, "TRACE"))     // Check for output type
  {
    TraceVar = TRACE;
    in = in->IPRSE_next;
  }

  if (streq(in->IPRSE_parameter, "NOPRT"))
  {
    TraceVar = NOPRT;
    in = in->IPRSE_next;
  }

  while (in)
  {
    Input[i].Param = in->IPRSE_parameter;
    Input[i].Value = in->IPRSE_value;
    in = in->IPRSE_next; i++;
  }

  while (i < INPUT_SIZE)
  {
    Input[i].Param = "";
    Input[i].Value = "";
    i++;
  }
}


void CheckSystemState()
{
  if (*(BYTE *) cinfc(CINFC_READ, CINFC_CMMSTI) & 0xEF)
    out("MQ Drivers should be run in Norm State");
}


void Trace(char *astring)
{
  char temp[100];

  if (TraceVar == TRACE)
  {
    sprintf(temp, "TRACE - %s executed", astring);
    out(temp);
  }
}


int  GetTraceVar()
{ return TraceVar; }


void Error(char *ErrorText)            // General exit on error
{
  QMQZ_Header.wtopc_letter = 'E';
  outu(ErrorText);                     // Output error message
  while (!tx_rollback()) {}            // undo all transactions
  exit(1);                             // Free ECB
}


void outc(char *ToOutPut)              // wtopc(chain)
{
  if (TraceVar != NOPRT && !IsAutomatedTestCase())                              //d26725
  {                                                                             //d26725
    outg(ToOutPut, WTOPC_CHAIN);
  }                                                                             //d26725
  else if (IsAutomatedTestCase())                                               //d26725
  {                                                                             //d26725
      TPF_TC_INFO(ToOutPut);                                                    //d26725
  }                                                                             //d26725
}


void oute(char *ToOutPut)              // wtopc(end chain)
{
  if (TraceVar != NOPRT && !IsAutomatedTestCase())                              //d26725
  {                                                                             //d26725
    outg(ToOutPut, WTOPC_CHAIN_END);
  }                                                                             //d26725
  else if (IsAutomatedTestCase())                                               //d26725
  {                                                                             //d26725
      TPF_TC_INFO(ToOutPut);                                                    //d26725
  }                                                                             //d26725
}


void out(char *ToOutPut)               // wtopc(no chain)
{
  if (TraceVar != NOPRT && !IsAutomatedTestCase())                              //d26725
  {                                                                             //d26725
      outg(ToOutPut, WTOPC_NO_CHAIN);
  }                                                                             //d26725
  else if (IsAutomatedTestCase())                                               //d26725
  {                                                                             //d26725
      TPF_TC_INFO(ToOutPut);                                                    //d26725
  }                                                                             //d26725
}

void outu(char *ToOutPut)              // wtopc, unconditional print
{
  if (!IsAutomatedTestCase())                                                   //d26725
  {
    outg(ToOutPut, WTOPC_NO_CHAIN);
  }                                                                             //d26725
  else                                                                          //d26725
  {                                                                             //d26725
    TPF_TC_ERROR(ToOutPut);                                                     //d26725
  }                                                                             //d26725
}


void outg(char *ToOutPut, t_wtopc_chain CHAIN_COMMAND)
{
  if (strlen(ToOutPut) > MAX_ERR_MSG_BUF)
  {
    memcpy(outputbuff2, ToOutPut, MAX_ERR_MSG_BUF);
    outputbuff2[MAX_ERR_MSG_BUF] = '\0';
    wtopc(outputbuff2, WTOPC_PRC, CHAIN_COMMAND, &QMQZ_Header);
  }
  else
    wtopc(ToOutPut, WTOPC_PRC, CHAIN_COMMAND, &QMQZ_Header);
}


// Have to be careful, because ToOutPut and output may be sharing the
// same space (if sf is called twice, as with a trace message)
// can't do a sprintf from and to the same place...

char *sf(const char* ToOutPut, int i)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i);
  free(buffer);
  return outputbuff;
}

char *sf(const char* ToOutPut, void* i)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i);
  free(buffer);
  return outputbuff;
}

char *sf(const char *ToOutPut, int i, int j)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i, j);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, int i, int j, int k)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i, j, k);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, int i, int j, const char *s)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i, j, s);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, int i, const char *s, int j, const char *t)
{
  sprintf(outputbuff, ToOutPut, i, s, j, t);
  return outputbuff;
}


char *sf(const char *ToOutPut, int i, const char *s)
{
  sprintf(outputbuff, ToOutPut, i, s);
  return outputbuff;
}

char *sf(const char *ToOutPut, int i, const char *s, const char *t)
{
  sprintf(outputbuff, ToOutPut, i, s, t);
  return outputbuff;
}

char *sf(const char *ToOutPut, const char *s, char *buf)
{
  if (buf == NULL) buf = outputbuff;
  sprintf(buf, ToOutPut, s);
  return buf;
}

char *sf(const char *ToOutPut, const char *s, const char *buf)
{
  sprintf(outputbuff, ToOutPut, s);
  return outputbuff;
}

char *sf(const char *ToOutPut, const char *s, int i)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, s, i);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, const char *s, int i, int j)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, s, i, j);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, const char *s, int i, const char *t)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, s, i, t);
  free(buffer);
  return outputbuff;
}


char *sf(const char *ToOutPut, const char *r, const char *s, const char *t)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);

  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, r, s, t);
  free(buffer);
  return outputbuff;
}

char *sf(const char *ToOutPut, MQBYTE24 i)
{
  char* buffer = (char*) malloc(strlen(ToOutPut) + 1);
  memset(outputbuff, ' ', MAX_ERR_MSG_BUF + 1);
  memcpy(buffer, ToOutPut, strlen(ToOutPut) + 1);
  sprintf(outputbuff, buffer, i);
  free(buffer);
  return outputbuff;
}


// _________________________________________________________
//
//
//      THESE ARE OPERATORS FOR A MODIFIED CLASS STRING
//
// _________________________________________________________
//
String::String()
{
  Buffer = new char[1];
  Buffer[0] = '\0';
}

String::String(char *cString)
{
  if (cString == NULL) cString = "";
  int Len = strlen(cString);
  Buffer = new char[Len + 1];
  memcpy(Buffer, cString, Len);
  Buffer[Len] = '\0';
}

String::String(int NumChars)
{
  Buffer = new char[NumChars + 1];
  memset(Buffer, '\0', NumChars + 1);
}

String::String(char Fill, int NumChars)
{
  Buffer = new char[NumChars + 1];
  memset(Buffer, Fill, NumChars);
  Buffer[NumChars] = '\0';
}

String::String(String &str)
{
  Buffer = new char[str.Length() + 1];
  memcpy(Buffer, str.C(), str.Length());
}

String& String::operator=(String &value)
{
  int Len = value.Length();
  char *NewBuffer = new char[Len + 1];

  memcpy(NewBuffer, value.C(), Len);
  NewBuffer[Len] = '\0';
  delete[] Buffer;
  Buffer = NewBuffer;
  return *this;
}

String& String::operator=(char *value)
{
  int Len = strlen(value);

  delete[] Buffer;
  Buffer = new char[Len + 1];
  memcpy(Buffer, value, Len);
  Buffer[Len] = '\0';
  return *this;
}

String& String::operator+=(char* rhs)
{
  int Len = strlen(Buffer);
  char *NewBuffer = new char[Len + strlen(rhs) + 1];

  memcpy(NewBuffer, Buffer, Len);
  memcpy(NewBuffer + Len, rhs, strlen(rhs));
  Buffer[Len + strlen(rhs)] = '\0';
  delete[] Buffer;
  Buffer = NewBuffer;
  return *this;
}

char* String::Range(int start, int bytes)
{
  char *Data = new char[bytes + 1];

  memcpy(Data, (char *) Buffer + start, bytes);
  Data[bytes] = '\0';
  return Data;
}

void String::Convert(char A, char B)
{
  unsigned int i;

  for (i = 0; i < strlen(Buffer); i++)
  {
    if (Buffer[i] == A)
      Buffer[i] = B;
  }
}

void String::CopyTo(char *S)
{
  strcpy(S, Buffer);
}


// _________________________________________________________
//
//
//      MQueue is a queue class, to handle MQ API
//                  calls to one queue.
//
// _________________________________________________________
//

MQueue::MQueue()
{
  objectHandle = MQHO_UNUSABLE_HOBJ;
  reason = MQRC_NONE;
  completionCode = MQCC_OK;

  MQGMO tempGetMessageOptions = {MQGMO_DEFAULT};
  GmoVersion = MQGMO_VERSION_2; // default to GMO version 2                     //d26214
  tempGetMessageOptions.Version  = GmoVersion;                                  //d26214
  memcpy(&getMessageOptions, &tempGetMessageOptions, sizeof(MQGMO));

  MQPMO tempPutMessageOptions = {MQPMO_DEFAULT};
  memcpy(&putMessageOptions, &tempPutMessageOptions, sizeof(MQOD));

  MQOD tempObjectDescriptor = {MQOD_DEFAULT};
  memcpy(&objectDescriptor, &tempObjectDescriptor, sizeof(MQOD));

  MQMD tempMessageDescriptor = {MQMD_DEFAULT};
  memcpy(&messageDescriptor, &tempMessageDescriptor, sizeof(MQMD));
  memcpy(&messageDescriptor.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);

  openOptions = MQOO_OUTPUT + MQOO_INPUT_AS_Q_DEF;
  GetError = MQRC_NONE;

  messageDescriptor.Persistence = MQPER_PERSISTENCE_AS_Q_DEF;
  messageDescriptor.CodedCharSetId = CCSID_EBCDIC;
  objectDescriptor.ObjectQMgrName[0] = '\0';
  printGetMessages = FALSE;
  printWaitMessages = TRUE;
  CheckMsgSeq = FALSE;
}

void MQueue::Conn(char* queueManagerName)
{
  MQCONN(queueManagerName, &connectionHandle,
         &completionCode, &reason);
  strcpy(queueManager, queueManagerName);
  CheckError("MQCONN", reason);
}

void MQueue::Open()
{
  MQOPEN(connectionHandle,
         &objectDescriptor,
         openOptions,
         &objectHandle,
         &completionCode,
         &reason);
  CheckError("MQOPEN", reason);
}

void MQueue::Put1(char* message, MQLONG messageLength)
{
  MQPUT1(connectionHandle,
         &objectDescriptor,
         &messageDescriptor,
         &putMessageOptions,
         messageLength,
         message,
         &completionCode,
         &reason);
  CheckError("MQPUT1", reason);
}

void MQueue::Put(char* message, MQLONG messageLength)
{
  MQPUT(connectionHandle,
        objectHandle,
        &messageDescriptor,
        &putMessageOptions,
        messageLength,
        message,
        &completionCode,
        &reason);
  CheckError("MQPUT", reason);
}

MQLONG MQueue::Get(char* message, MQLONG MessageLength)
{
  timeval startTime, stopTime;

  gettimeofday(&startTime, NULL);

  getMessageOptions.Version = GmoVersion;                                       //d26214
  memcpy(messageDescriptor.MsgId, MQMI_NONE, sizeof(messageDescriptor.MsgId));  //d26841
  memcpy(messageDescriptor.CorrelId, MQCI_NONE, sizeof(messageDescriptor.CorrelId));

  MQGET(connectionHandle,
        objectHandle,
        &messageDescriptor,
        &getMessageOptions,
        MessageLength,
        message,
        &returnLength,
        &completionCode,
        &reason);

  gettimeofday(&stopTime, NULL);

  if (reason == GetError ||
      (reason == MQRC_TRUNCATED_MSG_ACCEPTED &&
       (getMessageOptions.Options & MQGMO_ACCEPT_TRUNCATED_MSG)))
  {
    Trace("MQGET");
  }
  else
    CheckError("MQGET", reason);

  if (printGetMessages &&
      (reason == MQRC_TRUNCATED_MSG_ACCEPTED ||
       reason == MQRC_NONE))
  {
    boolean EventMsg = memcmp(messageDescriptor.Format, MQFMT_EVENT,
                              sizeof messageDescriptor.Format) == 0;
    boolean DLQMsg   = memcmp(message, MQDLH_STRUC_ID,                          //d26214
                              sizeof MQDLH_STRUC_ID) == 0;                      //d26214

    if (!EventMsg)
    {
      message[MIN(MessageLength, 50)] = '\0';                                   //26359

      if (messageDescriptor.CodedCharSetId != CCSID_EBCDIC)
      {
        out("Converting message to EBCDIC");
        message = CodepageConvert(CCSID_ASCII, CCSID_EBCDIC, message);
      }
    }

    if (messageDescriptor.Expiry != MQEI_UNLIMITED)
       outc(sf("MQGET: Persistence = %d, DataLength = %d, Expiry = %-10lu",
            messageDescriptor.Persistence, returnLength,
            messageDescriptor.Expiry));
    else
       outc(sf("MQGET: Persistence = %d, DataLength = %d,",
            messageDescriptor.Persistence, returnLength));

    if (isBrowse() || isLock())
    {
      outc(sf("       Browse Option : %s, Lock Option : %s %s",
               getBrowseOptString(), getLockOptString(), ""));
    }

    outc(sf("       PutDate            = %.8s",  messageDescriptor.PutDate));
    outc(sf("       PutTime            = %.8s",  messageDescriptor.PutTime));
    outc(sf("       MsgID    (in text) = %.24s", messageDescriptor.MsgId));
    outc(sf("       CorrelID (in text) = %.24s", messageDescriptor.CorrelId));
    outc(sf("       CCSID              = %d", messageDescriptor.CodedCharSetId));

    // If message has a dead letter header, display the reason
    if (DLQMsg)                                                                 //d26214
    {                                                                           //d26214
       PMQDLH      pMqdlh      = (PMQDLH) message;                              //d26214
       outc(sf("       DLH Reason         = %s (%d)",                           //d26214
            MQRC_String(pMqdlh->Reason), pMqdlh->Reason));                      //d26214
    }                                                                           //d26214

    // If this is an event message, display its contents.
    if (EventMsg)
    {
      // Warn if entire event message wasn't read. Can't display
      // it unless we have all of it. CTL-4 from decoding what is not there
      if (reason == MQRC_TRUNCATED_MSG_ACCEPTED)
        outc("*** WARNING: event message was truncated could not decode ***");  //17629
      else
        showEventMsg(message);

      oute("END OF MESSAGE DISPLAY");
    }
    else /* If msg has a MQDLH then we need to skip that to get to the data */  //d26214
    {                                                                           //d26533
      // Warn if entire DLQ message wasn't read. Can't display
      // it unless we have all of it. CTL-4 from decoding what is not there
      if (DLQMsg && reason == MQRC_TRUNCATED_MSG_ACCEPTED)                      //d26533
      {                                                                         //d26533
        outc("*** WARNING: message was truncated could not be decoded ***");    //d26533
        oute(sf("       Message Data = %.40s", message));                       //d26533
      }                                                                         //d26533
      else                                                                      //d26533
        oute(sf("       Message Data = %.40s", DLQMsg ? message + sizeof(MQDLH): message));
    }                                                                           //d26533
  }

  int secs  = stopTime.tv_sec - startTime.tv_sec;
  int usecs = stopTime.tv_usec - startTime.tv_usec;

  if (usecs < 0)
  {
    secs--;
    usecs += 1000000;
  }

  if (printWaitMessages && (getMessageOptions.Options & MQGMO_WAIT))
    oute(sf("       Wait Time = %d.%d secs", secs, usecs / 1000));

  return reason;
}

MQLONG MQueue::Get(char* message, MQLONG MessageLength, char * fileName,        //26653
                   char * validationFile)                                       //26653
{                                                                               //26653
  timeval startTime, stopTime;                                                  //26653

  gettimeofday(&startTime, NULL);                                               //26653

  getMessageOptions.Version = GmoVersion;                                       //26653
  memcpy(messageDescriptor.MsgId, MQMI_NONE, sizeof(messageDescriptor.MsgId));  //26841
  memcpy(messageDescriptor.CorrelId, MQCI_NONE, sizeof(messageDescriptor.CorrelId));

  MQGET(connectionHandle,                                                       //26653
        objectHandle,                                                           //26653
        &messageDescriptor,                                                     //26653
        &getMessageOptions,                                                     //26653
        MessageLength,                                                          //26653
        message,                                                                //26653
        &returnLength,                                                          //26653
        &completionCode,                                                        //26653
        &reason);                                                               //26653

  gettimeofday(&stopTime, NULL);                                                //26653

  if (reason == GetError ||                                                     //26653
      (reason == MQRC_TRUNCATED_MSG_ACCEPTED &&                                 //26653
       (getMessageOptions.Options & MQGMO_ACCEPT_TRUNCATED_MSG)))               //26653
  {                                                                             //26653
    Trace("MQGET");                                                             //26653
  }                                                                             //26653
  else                                                                          //26653
    CheckError("MQGET", reason);                                                //26653

  // if file was specified, validate that the size and contents of the file
  // match the size and contents of the message that was retrieved
  size_t fileLen = 0;                                                           //26653
  size_t maxBufLen = MessageLength;                                             //26653
  char file[maxBufLen + 1];                                                     //26653

  if (fileName &&                                                               //26653
     (reason == MQRC_TRUNCATED_MSG_ACCEPTED || reason == MQRC_NONE))            //26653
  {                                                                             //26653
    // if we already opened the file on the put, use the pointer to that file
    // for validation
    if (validationFile) {                                                       //26653
      if (strncmp(validationFile,message,maxBufLen) == 0) {                     //26653
        out("The contents and size of the message have been "                   //26653
             "successfully validated.");                                        //26653
      }                                                                         //26653
      else                                                                      //26653
      {                                                                         //26653
        Error("Message contents or size does not match validation file.");      //26653
      }                                                                         //26653
    }                                                                           //26653
    // The validation file hasn't been opened yet, so open and read it
    else                                                                        //26653
    {                                                                           //26653
      FILE *fp = fopen(fileName, "r");                                          //26653
      if (fp != NULL) {                                                         //26653
          fileLen = fread(file, sizeof(char), maxBufLen, fp);                   //26653
          if (ferror(fp) != 0) {                                                //26653
            Error("Validation file could not be read.");                        //26653
          }                                                                     //26653
          fclose(fp);                                                           //26653

          if (reason == MQRC_NONE) {                                            //26653
            if((strncmp(file,message,maxBufLen) == 0)                           //26653
                    && (fileLen == maxBufLen)) {                                //26653
              out("The contents and size of the message have been "             //26653
                   "successfully validated.");                                  //26653
            }                                                                   //26653
            else                                                                //26653
            {                                                                   //26653
              Error("Message contents or size does not match validation file.");//26653
            }                                                                   //26653
          }                                                                     //26653
      }                                                                         //26653
      else                                                                      //26653
      {                                                                         //26653
        Error("Validation file could not be opened.");                          //26653
      }                                                                         //26653
    }                                                                           //26653
  }                                                                             //26653

  if (printGetMessages &&                                                       //26653
      (reason == MQRC_TRUNCATED_MSG_ACCEPTED ||                                 //26653
       reason == MQRC_NONE))                                                    //26653
  {                                                                             //26653
    boolean EventMsg = memcmp(messageDescriptor.Format, MQFMT_EVENT,            //26653
                              sizeof messageDescriptor.Format) == 0;            //26653
    boolean DLQMsg   = memcmp(message, MQDLH_STRUC_ID,                          //26653
                              sizeof MQDLH_STRUC_ID) == 0;                      //26653

    if (!EventMsg)                                                              //26653
    {                                                                           //26653
      message[MIN(MessageLength, 50)] = '\0';                                   //26359

      if (messageDescriptor.CodedCharSetId != CCSID_EBCDIC)                     //26653
      {                                                                         //26653
        out("Converting message to EBCDIC");                                    //26653
        message = CodepageConvert(CCSID_ASCII, CCSID_EBCDIC, message);          //26653
      }                                                                         //26653
    }                                                                           //26653

    if (messageDescriptor.Expiry != MQEI_UNLIMITED)                             //26653
       outc(sf("MQGET: Persistence = %d, DataLength = %d, Expiry = %-10lu",     //26653
            messageDescriptor.Persistence, returnLength,                        //26653
            messageDescriptor.Expiry));                                         //26653
    else                                                                        //26653
       outc(sf("MQGET: Persistence = %d, DataLength = %d,",                     //26653
            messageDescriptor.Persistence, returnLength));                      //26653

    if (isBrowse() || isLock())                                                 //26653
    {                                                                           //26653
      outc(sf("       Browse Option : %s, Lock Option : %s %s",                 //26653
               getBrowseOptString(), getLockOptString(), ""));                  //26653
    }

    outc(sf("       PutDate            = %.8s",  messageDescriptor.PutDate));   //26653
    outc(sf("       PutTime            = %.8s",  messageDescriptor.PutTime));   //26653
    outc(sf("       MsgID    (in text) = %.24s", messageDescriptor.MsgId));     //26653
    outc(sf("       CorrelID (in text) = %.24s", messageDescriptor.CorrelId));  //26653
    outc(sf("       CCSID              = %d",messageDescriptor.CodedCharSetId));//26653

    // If message has a dead letter header, display the reason
    if (DLQMsg)                                                                 //26653
    {                                                                           //26653
       PMQDLH      pMqdlh      = (PMQDLH) message;                              //26653
       outc(sf("       DLH Reason         = %s (%d)",                           //26653
            MQRC_String(pMqdlh->Reason), pMqdlh->Reason));                      //26653
    }                                                                           //26653

    // If this is an event message, display its contents.
    if (EventMsg)                                                               //26653
    {
      // Warn if entire event message wasn't read. Can't display
      // it unless we have all of it. CTL-4 from decoding what is not there
      if (reason == MQRC_TRUNCATED_MSG_ACCEPTED)                                //26653
        outc("*** WARNING: event message was truncated could not decode ***");  //26653
      else                                                                      //26653
        showEventMsg(message);                                                  //26653

      oute("END OF MESSAGE DISPLAY");                                           //26653
    }                                                                           //26653
    else /* If msg has a MQDLH then we need to skip that to get to the data */  //26653
    {                                                                           //26653
      // Warn if entire DLQ message wasn't read. Can't display
      // it unless we have all of it. CTL-4 from decoding what is not there
      if (DLQMsg && reason == MQRC_TRUNCATED_MSG_ACCEPTED)                      //26653
      {                                                                         //26653
        outc("*** WARNING: message was truncated could not be decoded ***");    //26653
        oute(sf("       Message Data = %.40s", message));                       //26653
      }                                                                         //26653
      else                                                                      //26653
        oute(sf("       Message Data = %.40s", DLQMsg ? message + sizeof(MQDLH): message));
    }                                                                           //26653
  }                                                                             //26653

  int secs  = stopTime.tv_sec - startTime.tv_sec;                               //26653
  int usecs = stopTime.tv_usec - startTime.tv_usec;                             //26653

  if (usecs < 0)                                                                //26653
  {                                                                             //26653
    secs--;                                                                     //26653
    usecs += 1000000;                                                           //26653
  }                                                                             //26653

  if (printWaitMessages && (getMessageOptions.Options & MQGMO_WAIT))            //26653
    oute(sf("       Wait Time = %d.%d secs", secs, usecs / 1000));              //26653

  return reason;                                                                //26653
}                                                                               //26653

void MQueue::Inq(MQLONG attr, long *pValue)
{
  MQLONG selector[1] = {attr};
  MQLONG intRet[1];

  MQINQ(connectionHandle, objectHandle,
        1, selector,
        1, intRet,
        0, NULL,
        &completionCode, &reason);

  CheckError("MQINQ", reason);

  *pValue = intRet[0];
}

void MQueue::Close()
{
  MQCLOSE(connectionHandle, &objectHandle,
          0,
          &completionCode, &reason);
  CheckError("MQCLOSE", reason);
}

void MQueue::Disc()
{
  MQDISC(&connectionHandle,
         &completionCode, &reason);
  CheckError("MQDISC", reason);
}

/****************************/
/* Event Message Formatting */
/****************************/

// Macro to create code conversion table.

#define codestruc(name) struct {MQLONG val; char* str;} name[] =
#define code(c) {c, #c}

#define convert(struc, src, tgt)                                       \
  for (int struc##i = 0; struc[struc##i].val != -1; struc##i++)        \
  {                                                                    \
    if (struc[struc##i].val == src)                                    \
    {                                                                  \
      tgt = struc[struc##i].str;                                       \
      break;                                                           \
    }                                                                  \
  }


// Display formatted event message.

static void showEventMsg(void* pMsg)
{
  MQLONG ParameterCount = ((MQCFH*) pMsg)->ParameterCount + 1;
  char*  pChar = (char*) pMsg;

  outc("EVENT MESSAGE");

  while (ParameterCount > 0)
  {
    MQLONG Type = *((MQLONG*) pMsg);

    switch (Type)
    {
      case MQCFT_EVENT:
        pChar += showMQCFH((MQCFH*) pMsg);
        ParameterCount--;
        break;

      case MQCFT_INTEGER:
        pChar += showMQCFIN((MQCFIN*) pMsg);
        ParameterCount--;
        break;

      case MQCFT_STRING:
        pChar += showMQCFST((MQCFST*) pMsg);
        ParameterCount--;
        break;

      default:
        outc(sf("unknown structure type %d", Type));
        ParameterCount = 0;
        break;
    }

    pMsg = pChar;
  }
}

// Display MQCFT structure.

static MQLONG showMQCFH(MQCFH* pStruc)
{
  // Command code conversion.

  codestruc(Commands)
  {
    code(MQCMD_Q_MGR_EVENT),
    code(MQCMD_PERFM_EVENT),
    code(MQCMD_CHANNEL_EVENT),
    { -1, "" }
  };

  char* Command = "";

  convert(Commands, pStruc->Command, Command);

  // Reason code conversion.

  codestruc(Reasons)
  {
    code(MQRC_ALIAS_BASE_Q_TYPE_ERROR),
    code(MQRC_GET_INHIBITED),
    code(MQRC_PUT_INHIBITED),
    code(MQRC_UNKNOWN_OBJECT_NAME),
    code(MQRC_UNKNOWN_ALIAS_BASE_Q),
    code(MQRC_REMOTE_Q_NAME_ERROR),
    code(MQRC_XMIT_Q_TYPE_ERROR),
    code(MQRC_XMIT_Q_USAGE_ERROR),
    code(MQRC_UNKNOWN_XMIT_Q),
    code(MQRC_UNKNOWN_REMOTE_Q_MGR),
    code(MQRC_Q_MGR_ACTIVE),
    code(MQRC_Q_MGR_NOT_ACTIVE),
    code(MQRC_CHANNEL_STARTED),
    code(MQRC_CHANNEL_STOPPED),
    code(MQRC_CHANNEL_STOPPED_BY_USER),
    code(MQRC_Q_DEPTH_HIGH),
    code(MQRC_Q_DEPTH_LOW),
    code(MQRC_Q_FULL),
    code(MQRC_Q_SERVICE_INTERVAL_HIGH),
    code(MQRC_Q_SERVICE_INTERVAL_OK),
    code(MQRC_CHANNEL_SSL_ERROR),
    { -1, "" }
  };

  char* Reason = "";

  convert(Reasons, pStruc->Reason, Reason);

  // Display the structure.

  outc("  MQCFH");
  outc(sf("    Length          - %d", pStruc->StrucLength));
  outc(sf("    Version         - %d", pStruc->Version));
  outc(sf("    Command         - %d (%s)", pStruc->Command, Command));
  outc(sf("    Reason          - %d (%s)", pStruc->Reason, Reason));
  outc(sf("    Parameter Count - %d", pStruc->ParameterCount));
  return pStruc->StrucLength;
}

// Display MQCFIN structure.

static MQLONG showMQCFIN(MQCFIN* pStruc)
{
  // Parameter type conversion.

  codestruc(Parameters)
  {
    code(MQIA_Q_TYPE),
    code(MQIA_APPL_TYPE),
    code(MQIACF_REASON_QUALIFIER),
    code(MQIACF_ERROR_IDENTIFIER),
    code(MQIA_TIME_SINCE_RESET),
    code(MQIA_HIGH_Q_DEPTH),
    code(MQIA_MSG_ENQ_COUNT),
    code(MQIA_MSG_DEQ_COUNT),
    code(MQIACH_SSL_RETURN_CODE),
    code(MQIACH_SSL_CLIENT_AUTH),
    { -1, "" }
  };

  char* Parameter = "";

  convert(Parameters, pStruc->Parameter, Parameter);

  // Display the structure.

  outc("  MQCFIN");
  outc(sf("    Length          - %d", pStruc->StrucLength));
  outc(sf("    Parameter       - %d (%s)", pStruc->Parameter,
                                           Parameter));

  // The translation of the parameter value
  // depends on the parameter type.

  char*   ParmName = "";
  boolean ShowName = TRUE;

  switch (pStruc->Parameter)
  {
    case MQIA_Q_TYPE:
    {
      codestruc(QType)
      {
        code(MQQT_LOCAL),
        code(MQQT_MODEL),
        code(MQQT_ALIAS),
        code(MQQT_REMOTE),
        { -1, "" }
      };

      convert(QType, pStruc->Value, ParmName);
      break;
    }
    case MQIA_APPL_TYPE:
    {
      codestruc(ApplType)
      {
        code(MQAT_TPF),
        { -1, "" }
      };

      convert(ApplType, pStruc->Value, ParmName);
      break;
    }
    case MQIACF_REASON_QUALIFIER:
    {
      codestruc(ReasonQualifier)
      {
        code(MQRQ_Q_MGR_STOPPING),
        code(MQRQ_Q_MGR_QUIESCING),
        code(MQRQ_CHANNEL_STOPPED_OK),
        code(MQRQ_CHANNEL_STOPPED_ERROR),
        code(MQRQ_CHANNEL_STOPPED_RETRY),
        code(MQRQ_CHANNEL_STOPPED_DISABLED),
        code(MQRQ_SSL_HANDSHAKE_ERROR),
        code(MQRQ_SSL_CIPHER_SPEC_ERROR),
        code(MQRQ_SSL_CLIENT_AUTH_ERROR),
        code(MQRQ_SSL_PEER_NAME_ERROR),
        { -1, "" }
      };

      convert(ReasonQualifier, pStruc->Value, ParmName);
      break;
    }
    case MQIACF_ERROR_IDENTIFIER:
    {
      codestruc(ErrorIdentifier)
      {
        code(MQRQ_CHANNEL_STOPPED_DISABLED),
        { -1, "" }
      };

      convert(ErrorIdentifier, pStruc->Value, ParmName);
      break;
    }
    default:
      ShowName = FALSE;
      break;
  }

  if (ShowName)
    outc(sf("    Value           - %d (%s)", pStruc->Value, ParmName));
  else
    outc(sf("    Value           - %d", pStruc->Value));

  return pStruc->StrucLength;
}

// Display MQCFST structure.

static MQLONG showMQCFST(MQCFST* pStruc)
{
  // Parameter type conversion.

  codestruc(Parameters)
  {
    code(MQCA_Q_MGR_NAME),
    code(MQCA_Q_NAME),
    code(MQCA_BASE_Q_NAME),
    code(MQCA_XMIT_Q_NAME),
    code(MQCACF_APPL_NAME),
    code(MQCACF_OBJECT_Q_MGR_NAME),
    code(MQCACH_CHANNEL_NAME),
    code(MQCACH_XMIT_Q_NAME),
    code(MQCACH_CONNECTION_NAME),
    code(MQCACH_SSL_CIPHER_SPEC),
    code(MQCACH_SSL_PEER_NAME),
    code(MQCACH_SSL_HANDSHAKE_STAGE),
    { -1, "" }
  };

  char* Parameter = "";

  convert(Parameters, pStruc->Parameter, Parameter);

  // Display the structure.

  outc("  MQCFST");
  outc(sf("    Length          - %d", pStruc->StrucLength));
  outc(sf("    Parameter       - %d (%s)", pStruc->Parameter,
                                           Parameter));
  outc(sf("    CCSI            - %d", pStruc->CodedCharSetId));
  outc(sf("    String Length   - %d", pStruc->StringLength));
  outc("    String:");
  PrintData(pStruc->String, pStruc->StringLength, FALSE);
  return pStruc->StrucLength;
}

void SetAutomatedTestCase(boolean automatedTest)                                //d26725
{                                                                               //d26725
  automatedTestCase = automatedTest;                                            //d26725
}                                                                               //d26725

boolean IsAutomatedTestCase()                                                   //d26725
{                                                                               //d26725
  return automatedTestCase;                                                     //d26725
}                                                                               //d26725
