/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2023                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  d26214  06/15/18  Add option for version to fix driver with mq   */
/*                    client (ZMQID)                                 */
/*  d26533  11/06/19  z/TPF WebSphere MQ Support for Dynamic Routing */
/*  26653   10/28/20  Add option on LONGRUNNING and MQGET commands to*/
/*                    validate files.                                */
/*  d26725  09/29/21  Create z/TPF MQ DevOps Test Cases              */
/*  d26841  20230927  MQS1 Driver changes for 64-bit MQ support      */
/*                                                                   */
/*********************************************************************/

//  ________________________________________________________________
//
//      QMQZ.H CONTAINS
//        - Include Files
//        - Constant and Structure definitions
//        - Function Prototypes
//      For
//        - QMQZ.CPP
//        - QMQZ01.CPP
//        - QMQZ02.CPP
//        - QMQZ03.CPP
//        - QMQZ04.CPP
//        - QMQZ05.CPP
//        - QMQZ06.CPP
//        - QMQZ07.CPP
//  ________________________________________________________________
//
//#include <c_mqscd.h>              // Character conversion

#include <tpf/c_icnv.h>           // Character conversion
#include <tpf/c_mqcp.h>           // Checkpoint routines
#include <tpf/c_mqmg.h>           // Queue Manager Definitions
#include <tpf/c_mqs.h>            // MQ Header File
#include <tpf/c_cmqcfc.h>         // MQ Instrumentation types
#include <tpf/c_mqpc.h>           //
#include <iconv.h>                // Character conversion
#include <stddef.h>               // Standard C definitions
#include <stdio.h>                // sprintf
#include <stdarg.h>               // More C stuff
#include <stdlib.h>               // More Character Conversion
#include <string.h>               // String stuff
#include <sys/time.h>             // gettimeofday
#include <tpf/tpfio.h>            // DASD I/O Functions
#include <tpf/tpfparse.h>         // IPRSE_parse
#include <sys/unistd.h>           // POSIX Functions (sleep)
#include <tpf/zimageq.h>          // check system state
#include <tpf/tpfapi.h>

typedef struct NOPARSEt {
 int count;
 int time;
 int length;
 char qmgr[40];
 char queue[40];
} NOPARSE;

#define MAX_ERR_MSG_BUF 71

#define c74EMPTY        0         // No command
#define c74BEGIN        1         // tx_begin
#define c74COMMIT       2         // tx_commit
#define c74ROLLBACK     3         // tx_rollback
#define c74PUT          5         // MQPUT
#define c74GET          6         // MQGET
#define CCSID_ASCII     850       // CCSID for ASCII codepage
#define CCSID_EBCDIC    500       // CCSID for EBCDIC codepage
#define INPUT_SIZE      14        // Size of input array
#define MAXMSG_COUNT    100000    // Maximum messages to handle
#define MAXMSG_LENGTH   5000000   // malloc max
#define MAX_GMO_VERSION 2         // Maximum GMO version                        //d26214
#define MAXDELAY        30        // MQGET delay limit
#define MAXECB          10000000  // Maximum ECBs to launch
#define NOPRT           2         // No output
#define PRINT_INTERVAL  20        // Output every PRINT_INTERVAL secs
#define QN              "a*"      // Queue name parameters
#define QX              "(a*)*:48"
// #define string          String    // case insensitive
#define SIZEOFA         31        // size of rigor case array
#define TRACE           1         // Output MQAPI calls
#define NOMSG           MQRC_NO_MSG_AVAILABLE

#define PARM_WAIT_U     "Wait-Unlimited"
#define PARM_WAIT_L     "Wait-d*"
#define PARM_WAIT_N     "Wait-N.d*"
#define PARM_WAIT       " {" PARM_WAIT_U "|" PARM_WAIT_L "|"           \
                             PARM_WAIT_N "} "
#define PARM_SEQCHK     "(NO)SEQcheck"
#define PARM_BR_R       "BRowse-Read"
#define PARM_BR_G       "BRowse-Get"
#define PARM_BR_L       "BRowse-Lock"
#define PARM_BR         "{" PARM_BR_R "|" PARM_BR_G  "|" PARM_BR_L "}"
#define PARM_SLEEP      "SLEEP-d*"
#define PARM_WAKEUP_EXIT    "WAKEup-Exit"
#define PARM_WAKEUP_SERRC   "WAKEup-Serrcexit"
#define PARM_WAKEUP_LOCK    "WAKEup-Lock"
#define PARM_WAKEUP_LOCKN   "WAKEup-LOCKNext"
#define PARM_WAKEUP_UNLOCK  "WAKEup-Unlock"
#define PARM_WAKEUP_BROWSE  "WAKEup-Browse"
#define PARM_WAKEUP_BRN     "WAKEup-BRNext"
#define PARM_WAKEUP_GET     "WAKEup-Get"
#define PARM_WAKEUP     "{" PARM_WAKEUP_EXIT   "|" PARM_WAKEUP_SERRC   \
                        "|" PARM_WAKEUP_LOCK   "|" PARM_WAKEUP_LOCKN   \
                        "|" PARM_WAKEUP_UNLOCK "|" PARM_WAKEUP_BROWSE  \
                        "|" PARM_WAKEUP_BRN    "|" PARM_WAKEUP_GET "} "
#define PARM_BROWSE     PARM_BR " [" PARM_SLEEP PARM_WAKEUP "]"
#define PARM_NOPRT      "(NO)PRT"
#define PARM_PRTN       "PRTN-d*"
#define PARM_PRT        "{" PARM_NOPRT "|" PARM_PRTN "} "
#define PARM_DATA       "DATA-(u*)*:4"
#define PARM_MSGID      "MSGid-(u*)*:24"
#define PARM_CORRELID   "CORrelid-(u*)*:24"
#define PARM_EXPIRY_U   "EXpiry-Unlimited"
#define PARM_EXPIRY_L   "EXpiry-d*"
#define PARM_EXPIRY_N   "EXpiry-N.d*"
#define PARM_EXPIRY     " {" PARM_EXPIRY_U "|" PARM_EXPIRY_L "|"       \
                             PARM_EXPIRY_N "} "
#define PARM_SCOPE_C    "SCOPEaction-Commit"
#define PARM_SCOPE_R    "SCOPEaction-Rollback"
#define PARM_SCOPE      " {" PARM_SCOPE_C "|" PARM_SCOPE_R "} "
#define PARM_SCOPE_SLP  "SCOPESleep-d*"
#define PARM_CSCOPE     PARM_SCOPE PARM_SCOPE_SLP
#define PARM_LENGTH     "LENgth-d*"
#define PARM_UTIL       "UTil-d*"
#define PARM_VER        "Ver-d*"                                                //d26214
#define PARM_FILE       "File-u*"
#define PARM_COMPRESS   "Comp-d*"
#define PARM_VALIDATE   "(NO)VALidate"                                          //26653

#define PARM_API_PUTGET     "API-PUTGET"
#define PARM_API_PUT        "API-PUT"
#define PARM_API_PUT1       "API-PUT1"                                          //d26533
#define PARM_API_GET        "API-GET"
#define PARM_API_GETBY      "API-GETBYid"
#define PARM_API_GETUNDER   "API-GETUNDERcursor"
#define PARM_API_BROWSE     "API-BRowse"
#define PARM_API_SINGLE     "API-SINGLE"
#define PARM_API_LOCK       "API-Lock"

#define PARM_API        "{" PARM_API_PUTGET    "|" PARM_API_PUT1       \
                        "|" PARM_API_PUT       "|" PARM_API_GET        \
                        "|" PARM_API_BROWSE    "|" PARM_API_LOCK       \
                        "|" PARM_API_GETBY     "|" PARM_API_GETUNDER   \
                        "|" PARM_API_SINGLE " } "                               //d26533

#define QMQZGrammar IPRSE_QUOTE_GRAMMAR IPRSE_STRICT_GRAMMAR IPRSE_MIXED_CASE_GRAMMAR \
"ZTEST MQS1 [{TRACE | NOPRT}] [{"                                      \
"  {u | HELP} [{MQCOMM | MQROLL | DISplay | MQPUT | MQPUT1 | "         \
   "MQGET | REQuest | REPly | ERRor | CONtention | MQINQ | CASE |"     \
   "CHeckpoint | MQSET | HELP | COMM | ROLL | FUNCTION | LONGRUNNING} "\
   "["QX"] ["QX"] ["QX"] ["QX"] ["QX"]]"                               \
                                                                       \
"| {MQCOMM | MQROLL | COMM | ROLL} "QX" "QX" ["QX"] "                  \
   "Nmsg-d* " PARM_LENGTH " [(NO)PERsis]"                              \
   "{COmmitrate-d* | ROllbackrate-d*} [ECB-d*] [RIPL-d*]"              \
                                                                       \
"| DISplay {TABle | CHANnels | MEMory "QX" ["QX"] | "QX" [Msg-d*] }"   \
                                                                       \
"| MQPUT "QX" "QX" ["QX"] "                                            \
   "Nmsg-d* " PARM_LENGTH "[(NO)PERsis]"                               \
   "[" PARM_DATA  "] [" PARM_MSGID "] [" PARM_CORRELID "]"             \
   "[" PARM_EXPIRY "] [" PARM_CSCOPE "]" "[ECB-d*] [RIPL-d*] "         \
   "[" PARM_FILE "] [" PARM_COMPRESS "] "                              \
                                                                       \
"| MQPUT1 "QX" "QX" ["QX"]" PARM_LENGTH "[(NO)PERsis]"                 \
   "[" PARM_DATA "] [" PARM_MSGID "] [" PARM_CORRELID "] ["            \
   PARM_EXPIRY "] [" PARM_CSCOPE "]" "[ECB-d*] "                       \
   "[" PARM_FILE "] [" PARM_COMPRESS "] "                              \
                                                                       \
"| MQGET "QX" "QX" {Nmsg-d* | Sec-d*}" PARM_LENGTH                     \
   "[" PARM_SEQCHK "][" PARM_PRT "][" PARM_WAIT "][" PARM_BROWSE       \
   "] [" PARM_MSGID "][" PARM_CORRELID "][" PARM_CSCOPE "]"            \
   "[" PARM_VER "] [" PARM_FILE "]" "[ECB-d*]"                         \
                                                                       \
"| REQuest "QX" "QX" "QX" Nmsg-d* " PARM_LENGTH "[ECB-d*]"             \
   "[(NO)PERsis]"                                                      \
                                                                       \
"| REPly "QX" "QX" Nmsg-d* " PARM_LENGTH "[ECB-d*]"                    \
                                                                       \
"| ERRor {MQCONN | MQOPEN | MQPUT | MQGET | MQPUT1 | MQCLOSE | "       \
   "MQINQ | MQSET | MQDISC} "QX" "QX" [d*]"                            \
                                                                       \
"| CONtention {"QX" ECBs-d* [(NO)PERsis] | "                           \
   "INCOMING "QX" d* d* (NO)PERsis}"                                   \
                                                                       \
"| MQINQ {QMGR | "QX" } "                                              \
                                                                       \
"| MQSET "QX" {PUT | GET} {ALLOWED | INHIBITED}"                       \
                                                                       \
"| MQSET "QX" {TRIGGER | TRIGTYPE | TRIGDATA} "                        \
   "{ON | OFF | FIRST | NONE | EVERY | "QX"}"                          \
                                                                       \
"| CASE [PRINT] {d* | SPECial | RANGE | RIGOR1 | RIGOR2 | RIGOR3"      \
   " | RIGOR4 | ALL"                                                   \
   " | RIGORBACKUP | RANDOM | TIME d*} "QX" ["QX"] ["QX"] ["QX"] ["QX  \
   "] ["QX"] [MQONLY] [Low-d*] [Hi-d*] [QM-(a*)*:48]"                  \
                                                                       \
"| ZMQSC ["QX"] ["QX"] ["QX"] ["QX"] ["QX"] ["QX"] ["QX"] Low-d* Hi-d*"\
                                                                       \
"| CHeckpoint {Messages | NUMQueues | NUMChannels | NUMRecords "       \
   "| ALLQueuenames | ALLChannelnames | FIND "QX"}"                    \
                                                                       \
"| LONGRunning "QX" "QX" SLEEPinterval-d* {LOOPCount-d* | LOOPTime-d*}"\
  "["PARM_VALIDATE"] ["PARM_API"] [ECB-d*] [LENgth-d*] [UTil-d*] "     \
  "[" PARM_FILE "] [" PARM_COMPRESS "] "                               \
"}]"                                                                            //26653

class String                                          // String class
{
  private:
    char *Buffer;                                     // String data

  public:
    String();                                         // Generic Constructor
    String(char *cString);                            // Make from array
    String(String& str);                              // Make from another String
    String(int NumChars);                             // Make with blank space
    String(char Fill, int NumChars);                  // Fill an array
    ~String() { delete[] Buffer; }                    // Destructor

    String& operator= (String& rhs);                  // Assignment to a String
    String& operator= (char* rhs);                    // Assignment to a character array
    String& operator+=(char* rhs);                    // Append with Array

    int operator==(char* rhs)
    { return (strcmp(Buffer, rhs) == 0); }            // == comparing to array

    int operator!=(char* rhs)
    { return (strcmp(Buffer, rhs) != 0); }            // != comparing to array

    int operator==(String& rhs)
    { return (strcmp(Buffer, rhs.C()) == 0); }        // == comparing to string

    int operator!=(String& rhs)
    { return (strcmp(Buffer, rhs.C()) != 0); }        // != comparing to string

    char operator[](int k)
    { return Buffer[k]; }                             // Character in array

    void MakeUpperCase()
    {
      for (int n = 0; Buffer[n]; n++)
        Buffer[n] = toupper(Buffer[n]);
    }

    char *C() const { return Buffer; }                // C language character Data array
    char *Range(int start, int bytestoget);           // Get a substring
    void  Convert(char A, char B);                    // Convert all of one character
    void  CopyTo(char *S);                            // Copy to array
    int   Length() { return strlen(Buffer); }         // String length
    int   ToInt() { return atol(Buffer); }            // Integer value of the string
    long  ToPtr() { return atoll(Buffer); }           // String value to long long
    int   ToNegInt() { return -atol(Buffer + 2); }    // negative value parm (n.1)
};

typedef void (*CFunction)();
typedef void (*CommandFunction)(struct sInput *Input);
typedef void (*ErrorFunction)(char *QMgr, char *QXame,
                              unsigned int CaseNum);

struct sTest {String Param; void* Value; };

struct sA     {int numCommands; BYTE Command[SIZEOFA]; };
struct sABACK {int testCase; int commandNum; sA A; };
struct sInput {String Param; String Value; };
struct sCommandTable {char *Command; CommandFunction Function; };
struct sErrorTable {char *Command; ErrorFunction Function; };
struct sETable {char *Text; MQLONG EReason; };

extern "C" void QMQZ();                                              // QMQZ

void ArrayBackup(sA* pA);                                            // QMQZ06
void ArrayUpdate(sA* pA);                                            // QMQZ06
void BuildInput(struct IPRSE_output *in, sInput *Input);             // QMQZ
void CaseError(char *msg);                                           // QMQZ06
void CheckError(char *API, MQLONG Reason);                           // QMQZ06
void CheckMsgLength(MQLONG Length);                                  // QMQZ01
void CheckGmoVersion(MQLONG ver);                                               //d26214
void CheckNumCases();                                                // QMQZ06
void CheckNumCases(int casenum);                                     // QMQZ06
void CheckNumMsgs(int num);                                          // QMQZ01
void CheckNumEcbs(int numEcbs);                                      // QMQZ05
void CheckRate(int commRate, int numMsgs);                           // QMQZ01
void CheckRiplPoint(int point, int msgs);                            // QMQZ01
void CheckSystemState();                                             // QMQZ
char *CodepageConvert(u_short c_from, u_short c_to, char *data);     // QMQZ03
void CommitOrRollback(sInput* Input);                                // QMQZ01
void CountMsgList(unsigned int FileAddr, int *total);                // QMQZ05
void CountMessages(MQChkptQEntry, int *total);                       // QMQZ05
void CPNumMessages();                                                // QMQZ05
void CPNumQueues();                                                  // QMQZ05
void CPNumChannels();                                                // QMQZ05
void CPNumRecords();                                                 // QMQZ05
void CPFindAllQueues();                                              // QMQZ05
void CPFindAllChannels();                                            // QMQZ05
void CPFindName(char Name[MQ_Q_NAME_LENGTH]);                        // QMQZ05
void CreateECBs(int ecb_count, char *zmsg, boolean wait=FALSE);      // QMQZ01
void DisplayMemoryMessage(void* pMessage);                           // QMQZ01 //d26841
void EndArrayBackup();                                               // QMQZ06
void Error(char *ErrorMessage);                                      // QMQZ06
void error_test_conn(char *QMgr, char *QXame, unsigned int CaseNum); // QMQZ02
void error_test_close(char *QMgr, char *QXame, unsigned int CaseNum);// QMQZ02
void error_test_disc(char *QMgr, char *QXame, unsigned int CaseNum); // QMQZ02
void error_test_get(char *QMgr, char *QXame, unsigned int CaseNum);  // QMQZ02
void error_test_inq(char *QMgr, char *QXame, unsigned int CaseNum);  // QMQZ02
void error_test_open(char *QMgr, char *QXame, unsigned int CaseNum); // QMQZ02
void error_test_put(char *QMgr, char *QXame, unsigned int CaseNum);  // QMQZ02
void error_test_put1(char *QMgr, char *QXame, unsigned int CaseNum); // QMQZ02
void error_test_set(char *QMgr, char *QXame, unsigned int CaseNum);  // QMQZ02
void FileSomething(char toput);                                      // QMQZ06
void Get(int BytesToGet, char MsgToGet, int Handle=1);               // QMQZ06
void Get(char *StringIn, int Handle=1);                              // QMQZ06
void Get(MQLONG CodeIn, int Handle=1, int BytesToGet=1);             // QMQZ06
void Get(MQLONG CodeIn, int BytesToGet, char CharToGet, int Handle=1);//QMQZ06
void GetOneFromEach(int MessageStart);                               // QMQZ06
int  GetTraceVar();                                                  // QMQZ
int  HelpCode(char *OutString);                                      // QMQZ06
char *MessagetoPut(char *QMsg, int i, int msgLength);                // QMQZ01
char *MQRC_String(int Reason);                                       // QMQZ04
void NextControlRec(MQCntrlRecord **CurrRec);                        // QMQZ05
int  OKChar(char charIn);                                            // QMQZ01
void out(char *ToOutPut);      // Make inline in the header???       // QMQZ
void outc(char *ToOutPut);                                           // QMQZ
void oute(char *ToOutPut);                                           // QMQZ
void outg(char *ToOutPut, t_wtopc_chain CHAIN_COMMAND);              // QMQZ
void outu(char *ToOutPut);                                           // QMQZ
void padWithSpaces(MQCHAR *buffer, int length);                      // QMQZ01
void PrintCodes(char *api, MQLONG Reason, MQLONG expected);          // QMQZ02
void PrintCaseNumbers();                                             // QMQZ06
void PrintData(void* pData, int bytesToPrint, boolean ShowAddr=TRUE);// QMQZ01
void PrintInput(sInput *Input);                                      // QMQZ
void Put(char *StringIn, int Handle=1);                              // QMQZ06
void Put(int BytesToPut, char MsgToPut, int Handle=1);               // QMQZ06
void PutOneOnEach(int MessageStart);                                 // QMQZ06
void QMQZ_CheckPoint(sInput *Input);                                 // QMQZ05
void QMQZ_Contention(sInput *Input);                                 // QMQZ01
void QMQZ_CommRoll(sInput *Input);                                   // QMQZ01
void QMQZ_Display(sInput *Input);                                    // QMQZ01
void QMQZ_ErrorTest(sInput *Input);                                  // QMQZ02
void QMQZ_Get(sInput *Input);                                        // QMQZ01
void QMQZ_Help(sInput* Input);                                       // QMQZ
void QMQZ_Inq(sInput *Input);                                        // QMQZ01
void QMQZ_Put(sInput *Input);                                        // QMQZ01
void QMQZ_Put1(sInput *Input);                                       // QMQZ01
void QMQZ_Request(sInput *Input);                                    // QMQZ03
void QMQZ_Reply(sInput *Input);                                      // QMQZ03
void QMQZ_Set(sInput *Input);                                        // QMQZ01
void QMQZ_Suite(sInput *Input);                                      // QMQZ06
void QMQZ_ZMQSC(sInput *Input);                                      // QMQZ01
void QMQZ_LongRunning(sInput *Input);                                // QMQZ07
void QMQZ_LongRunning_noparser(NOPARSE * input);                     // QMQZ07
void Ripl();                                                         // QMQZ01
void Rigor1(int min, int max);                                       // QMQZ06
void Rigor2(int min, int max);                                       // QMQZ06
void Rigor3(int max);                                                // QMQZ06
void RigorRun(sA* pA);                                               // QMQZ06
int  RigorFindFree(sA* pA, int place);                               // QMQZ06
void RigorWatchA();                                                  // QMQZ06
void RunAllCases(int low, int hi);                                   // QMQZ06
void RunCase(int PrintSuccess);                                      // QMQZ06
void RunRandomized();                                                // QMQZ06
void RunSpecial();                                                   // QMQZ06
void RunTimed(int seconds);                                          // QMQZ06
char *sf(const char *ToOutPut, int i);                               // QMQZ
char *sf(const char *ToOutPut, void* i);                             // QMQZ
char *sf(const char *ToOutPut, int i, const char *s);                // QMQZ
char *sf(const char *ToOutPut, int i, const char *s, const char* t); // QMQZ
char *sf(const char *ToOutPut, int i, const char *s, int j, const char *t);  // QMQZ
char *sf(const char *ToOutPut, int i, int j, int k);                 // QMQZ
char *sf(const char *ToOutPut, int i, int j);                        // QMQZ
char *sf(const char *ToOutPut, int i, int j, const char *s);         // QMQZ
char *sf(const char *ToOutPut, const char *s, char *buf = NULL);       // QMQZ
char *sf(const char *ToOutPut, const char *s, int i);                // QMQZ
char *sf(const char *ToOutPut, const char *s, int i, int j);         // QMQZ
char *sf(const char *ToOutPut, const char *s, int i, const char *t); // QMQZ
char *sf(const char *ToOutPut, const char *r, const char *s, const char *t); // QMQZ
void StartArrayBackup();                                             // QMQZ06
void Trace(char *astring);                                           // QMQZ
void TX_Begin();                                                     // QMQZ06
void TX_Rollback();                                                  // QMQZ06
void TX_Commit();                                                    // QMQZ06
long HexToInt(char* in);
MQ2RC SetUpQueues();                                                 // QMQZ06  //d26725
void ExecuteCommand(char * command);                                 // QMQZ06  //d26725
void ZdevoError(char * qName);                                       // QMQZ06  //d26725
boolean IsTestCaseValid();                                           // QMQZ06  //d26725
MQ2RC QueueExists(char * qName);                                     // QMQZ06  //d26725
unsigned char GetProtectKey();                                       // QMQZ06  //d26725
unsigned long GetMemoryInUse();                                      // QMQZ06  //d26725
void SetAutomatedTestCase(boolean automatedTest);                    // QMQZ    //d26725
boolean IsAutomatedTestCase();                                       // QMQZ    //d26725


// I made this class to facilitate future coding to test or use MQ APIs,
// its not worth the effort to convert everything to use this...

class MQueue
{
  private:
    MQHOBJ objectHandle;
    MQLONG completionCode;
    MQLONG reason;
    MQLONG GetError;
    MQLONG openOptions;
    MQLONG returnLength;
    char   queueManager[MQ_Q_MGR_NAME_LENGTH + 1];
    MQGMO  getMessageOptions;
    MQPMO  putMessageOptions;
    MQMD   messageDescriptor;
    MQOD   objectDescriptor;
    BOOL   printGetMessages;
    BOOL   printWaitMessages;
    BOOL   CheckMsgSeq;
    MQLONG GmoVersion;                                                          //d26214
    static MQHCONN connectionHandle;

  public:
    MQueue();
   ~MQueue() {}

    void SetPersistence(MQLONG persistence)
    { messageDescriptor.Persistence = persistence; }

    void SetGetOptions(MQLONG newOptions)
    { getMessageOptions.Options = newOptions; }

    void AddGetOptions(MQLONG newOptions, MQLONG removeOptions = 0)
    {
      getMessageOptions.Options &= ~removeOptions;
      getMessageOptions.Options |= newOptions;
    }
    void ClearMessageDescriptor() 
    {
       MQMD tempMessageDescriptor = {MQMD_DEFAULT};
       memcpy(&messageDescriptor, &tempMessageDescriptor, sizeof(MQMD));
       memcpy(&messageDescriptor.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);
    }
    void ClearMatchOptions()
    { getMessageOptions.MatchOptions = MQMO_NONE; }

    void SetMsgID(String MsgID)
    {
      memset(messageDescriptor.MsgId, ' ',
             sizeof(messageDescriptor.MsgId));
      memcpy(messageDescriptor.MsgId, MsgID.C(), MsgID.Length());
    }

    void SetMsgID(char* MsgID)
    {
      memset(messageDescriptor.MsgId, ' ',
             sizeof(messageDescriptor.MsgId));
      memcpy(messageDescriptor.MsgId, MsgID, strlen(MsgID));
    }

    void SetCorrelID(String CorrelID)
    {
      memset(messageDescriptor.CorrelId, ' ',
             sizeof(messageDescriptor.CorrelId));
      memcpy(messageDescriptor.CorrelId, CorrelID.C(),
              CorrelID.Length());
    }

    void SetCorrelID(char* CorrelID)
    {
      memset(messageDescriptor.CorrelId, ' ',
             sizeof(messageDescriptor.CorrelId));
      memcpy(messageDescriptor.CorrelId, CorrelID, strlen(CorrelID));
    }

    void AddMatchMsgID(String MsgID)
    {
      getMessageOptions.MatchOptions |= MQMO_MATCH_MSG_ID;
      memset(messageDescriptor.MsgId, ' ',
             sizeof(messageDescriptor.MsgId));
      memcpy(messageDescriptor.MsgId, MsgID.C(), MsgID.Length());
    }

    void AddMatchMsgID(char* MsgID)
    {
      getMessageOptions.MatchOptions |= MQMO_MATCH_MSG_ID;
      if (MsgID != NULL)
      {
         memset(messageDescriptor.MsgId, ' ',
              sizeof(messageDescriptor.MsgId));
         memcpy(messageDescriptor.MsgId, MsgID, strlen(MsgID));
      }
    }

    void AddMatchCorrelID(String CorrelID)
    {
      getMessageOptions.MatchOptions |= MQMO_MATCH_CORREL_ID;
      memset(messageDescriptor.CorrelId, ' ',
             sizeof(messageDescriptor.CorrelId));
      memcpy(messageDescriptor.CorrelId, CorrelID.C(),
              CorrelID.Length());
    }

    void AddMatchCorrelID(char* CorrelID)
    {
      getMessageOptions.MatchOptions |= MQMO_MATCH_CORREL_ID;
      memset(messageDescriptor.CorrelId, ' ',
             sizeof(messageDescriptor.CorrelId));
      memcpy(messageDescriptor.CorrelId, CorrelID, strlen(CorrelID));
    }

    void SetGetError(MQLONG newCode)
    { GetError = newCode; }

    void SetOpenOptions(MQLONG options)
    { openOptions = options; }

    void SetObjectName(char* oName)
    { strcpy(objectDescriptor.ObjectName, oName); }

    void SetQueueManagerAlias(char* managerAlias)
    { strcpy(objectDescriptor.ObjectQMgrName, managerAlias); }

    void SetGetPrint(BOOL value)
    { printGetMessages = value; }

    void SetWaitPrint(BOOL value)
    { printWaitMessages = value; }

    void SetWaitInterval(MQLONG WaitInterval)
    { getMessageOptions.WaitInterval = WaitInterval; }

    void SetExpiry(MQLONG ExpiryInterval)
    { messageDescriptor.Expiry = ExpiryInterval; }

    void SetMatchOptions(MQLONG MatchOptions)
    { getMessageOptions.MatchOptions = MatchOptions; }

    void SetSeqChk(BOOL SeqChk)
    { CheckMsgSeq = SeqChk; }

    void SetGmoVersion(int newGmoVersion)                                       //d26214
    { GmoVersion = newGmoVersion; }                                             //d26214

    MQLONG GetGmoVersion()                                                      //d26214
    { return GmoVersion; }                                                      //d26214

    MQLONG GetWaitInterval()
    { return getMessageOptions.WaitInterval; }

    MQLONG GetExpiry()
    { return messageDescriptor.Expiry; }

    BOOL GetSeqChk(void)
    { return CheckMsgSeq; }

    BOOL GetWaitPrint(void)
    { return printWaitMessages; }

    MQLONG GetPersistence()
    { return messageDescriptor.Persistence; }

    char*  GetPersistenceString()
    {
      switch (messageDescriptor.Persistence)
      {
        case MQPER_NOT_PERSISTENT:
          return "NOPER";

        case MQPER_PERSISTENT:
          return "PER";

        default:
          return "";
      }
    }

    const char*  GetPrintString()
    { return printGetMessages ? "PRT" : "NOPRT"; }

    char*  GetManagerName()
    { return queueManager; }

    char*  GetManagerAlias()
    { return objectDescriptor.ObjectQMgrName; }

    char*  GetObjectName()
    { return objectDescriptor.ObjectName; }

    boolean isBrowse()
    {
       MQLONG BrowseOptions     = MQGMO_BROWSE_FIRST |
                                  MQGMO_BROWSE_MSG_UNDER_CURSOR |
                                  MQGMO_BROWSE_NEXT;


       return (getMessageOptions.Options & BrowseOptions);
    }

    boolean isLock()
    {
       MQLONG LockOptions     = MQGMO_LOCK |
                                MQGMO_UNLOCK;

       return (getMessageOptions.Options & LockOptions);
    }

    char* getBrowseOptString()
    {
       if (getMessageOptions.Options & MQGMO_BROWSE_FIRST)
          return ("BROWSE_FIRST");
       else if (getMessageOptions.Options & MQGMO_BROWSE_NEXT)
          return ("BROWSE_NEXT");
       else if (getMessageOptions.Options & MQGMO_BROWSE_MSG_UNDER_CURSOR)
          return ("BROWSE_MSG_UNDER_CURSOR");
       else
          return ("NONE");
    }

    char* getLockOptString()
    {
       if (getMessageOptions.Options & MQGMO_LOCK)
          return ("LOCK");
       else if (getMessageOptions.Options & MQGMO_UNLOCK)
          return ("UNLOCK");
       else
          return ("NONE");
    }

    MQLONG getCompletionCode() 
    {
       return completionCode;
    }

    MQLONG getReturnLength() 
    {
       return returnLength;
    }

    void Conn(char* queueManagerName);                   // QMQZ
    void Open();                                         // QMQZ
    void Put1(char* message, MQLONG messageLength);      // QMQZ
    void Put(char* message, MQLONG messageLength);       // QMQZ
    MQLONG Get(char* message, MQLONG messageLength);     // QMQZ
    MQLONG Get(char* message, MQLONG messageLength,      // QMQZ                //26653
               char * fileName, char * validationFile);  // QMQZ                //26653
    void Inq(MQLONG attr, long *pValue);                 // QMQZ
    void Close();                                        // QMQZ
    void Disc();                                         // QMQZ
};
