/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2008, 2023                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  TD00814 06/10/98  MQPUT MQPUT1 MQGET                             */
/*  17629   02/12/08  Added SSL Event status for MQINQ QMGR          */
/*  26359   06/14/18  Remove "min" & "max" for PJ45111               */
/*  d26214  01/03/19  Add option for version to fix driver with      */
/*                    mq client (ZMQID)                              */
/*  d26533  11/06/19  z/TPF support for Dynamic Routing              */
/*  26653   10/28/20  Add option on LONGRUNNING and MQGET commands to*/
/*                    validate files.                                */
/*  d26841  20230927  MQS1 Driver changes for 64-bit MQ support      */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....QMQZ01                                          */
/* FUNCTION TYPE.....TPF                                             */
/*                                                                   */
/* SEGMENT NAME......qmqz01.cpp                                      */
/* #PRAGMA MAP NAME..mqs_pgp1                                        */
/*                                                                   */
/* DESCRIPTION: part of SO  QMQZ                                     */
/*                                                                   */
/*********************************************************************/

#include <qmqz.h>
#include <fstream>
#include <zlib.h>
//#include <tpftime.h>

#define MSG_DATA_PREFIX  "QMQZ.Message."
#define MSG_SEQ_PFX ".Message."

/**********************************************************************/
/* static function declarations                                       */
/**********************************************************************/

static void checkMsgSeq(char *Message);
static void QMQZ_Putx(sInput *Input, boolean Put1);

void QMQZ_CommRoll(sInput* Input)
{
  MQueue    myQueue;
  char      zMessage[250] = "";
  char*     command;
  int       i;
  int       numECBs;
  int       numMsgs;
  int       msgLength;
  int       rate;
  int       riplPoint = 0;
  int       msgsInScope;

  command = Input[0].Param.C();
  myQueue.Conn(Input[1].Value.C());
  myQueue.SetObjectName(Input[2].Value.C());

  if (Input[3].Param == QN)
  {
    myQueue.SetQueueManagerAlias(Input[3].Value.C());
    Input++;
  }

  CheckNumMsgs(numMsgs = Input[3].Value.ToInt());
  CheckMsgLength(msgLength = Input[4].Value.ToInt());

  if (Input[5].Param == "(NO)PERsis")
  {
    if (Input[5].Value[0] == 'N' || Input[5].Value[0] == 'n')
      myQueue.SetPersistence(MQPER_NOT_PERSISTENT);
    else if (Input[5].Value[0] == 'Y' || Input[5].Value[0] == 'y')
      myQueue.SetPersistence(MQPER_PERSISTENT);

    Input++;
  }

  CheckRate(rate = Input[5].Value.ToInt(), numMsgs);

  if (Input[6].Param == "ECB-d*")
  {
    CheckNumEcbs(numECBs = Input[6].Value.ToInt());

    sprintf(zMessage,
            "ZTEST MQS1 %s %s %s %s Nmsg-%d LENgth-%d %s Co-%d",
            command,
            myQueue.GetManagerName(),
            myQueue.GetObjectName(),
            myQueue.GetManagerAlias(),
            numMsgs,
            msgLength,
            myQueue.GetPersistenceString(),
            rate);

    CreateECBs(numECBs-1, zMessage);
    Input++;
  }

  if (Input[6].Param == "RIPL-d*")
    CheckRiplPoint(riplPoint = Input[6].Value.ToInt(), numMsgs);

  myQueue.Open();
  myQueue.SetGetOptions(MQGMO_ACCEPT_TRUNCATED_MSG);
  myQueue.ClearMatchOptions();                                                  //d26841

  String buffer('Z', msgLength);

  for (int j = 0; j < numMsgs; j += rate)
  {
    TX_Begin();                        // Begin Transaction
    msgsInScope = MIN(rate, numMsgs - j);                                       //26359

    for (i = 1; i <= msgsInScope; i++) // put rate messages
    {
      myQueue.Put(MessagetoPut(buffer.C(), i, msgLength), msgLength);

      if ((i + j == riplPoint) && (i != msgsInScope))
        Ripl();
    }

    outc(sf("%d MQPUTs added to transaction scope.", msgsInScope));

    for (i = 1; i < msgsInScope; i++)  // get rate-1 messages
      myQueue.Get(buffer.C(), msgLength);

    outc(sf("%d MQGETs added to transaction scope.", msgsInScope-1));

    if ((Input[0].Value == "MQROLL") || (Input[0].Value == "ROLL"))
    {
      TX_Rollback();
      outc("Transaction Rolled back");
    }
    else
    {
      TX_Commit();
      outc("Transaction Commited");
    }

    if (j + msgsInScope == riplPoint)
      Ripl();
  }

  myQueue.Close();
  myQueue.Disc();

  if ((Input[0].Value == "MQROLL") || (Input[0].Value == "ROLL"))
    oute("Transactions Complete");
  else
  {
    outc("Transactions Complete");
    oute(sf("%d Messages have been added to the queue",
            (numMsgs - 1)/rate + 1));
  }
}


void QMQZ_Contention(sInput *Input)
{
  String  TrgtQName;                             // Target queue name
  String  QMngrName;                             // Queue Manager name
  char    msg[100] = "";                         // Message to spawn get ecbs
  String  Persistence;                           // Persistence or not
  String  TraceMsg;                              // User might want a trace
  int     numEcbs;                               // Number of ECBs
  int     numtoput, numtoget, i, TX_ErrorCode;
  MQLONG  Reason;
  MQLONG  CompCode;                              // completion code
  MQLONG  msgLen;                                // Msg length in bytes
  MQPMO   pmo = {MQPMO_DEFAULT};                 // Put Msg Options
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;           // Connection Handle
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;             // Object Handle
  MQOD    ObjDesc = {MQOD_DEFAULT};              // Object Descriptor
  MQMD    md = {MQMD_DEFAULT};                   // Message Descriptor
  MQGMO   gmo = {MQGMO_DEFAULT};                 // Get Msg Options
  gmo.Version  = MQGMO_VERSION_2;
  gmo.MatchOptions = MQMO_NONE;                                                 //d26841

  if (Input[1].Param != "INCOMING")    // ECBs spawned are "INCOMING"
  {
    TrgtQName = Input[1].Value;
    CheckNumEcbs(numEcbs = Input[2].Value.ToInt());

    if ((Input[3].Param == "(NO)PERsis") &&
        (Input[3].Value[0] == 'N'))
    {
      Persistence = "NOPER";
    }
    else
      Persistence = "PER";

    switch (GetTraceVar())
    {
      case TRACE: TraceMsg = "TRACE";  break;
      case NOPRT: TraceMsg = "NOPRT";  break;
      default:    TraceMsg = "";
    }

    out(sf("Launching %d Contention ECBs, %d messages "
           "will be left on the queue", numEcbs, numEcbs));

    for (int i = 1; i <= numEcbs; i++)
    {
      sprintf(msg, "ZTEST MQS1 %s CON INCOMING %s %d %d %s",
                   TraceMsg.C(), TrgtQName.C(), i, numEcbs - i, Persistence.C());
      CreateECBs(1, msg);
    }
  }

  else
  {
    Input[2].Value.CopyTo(ObjDesc.ObjectName);
    numtoput = Input[3].Value.ToInt();
    numtoget = Input[4].Value.ToInt();
    gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;           // prep for gets
    memcpy(md.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);  // prep for puts or gets

    if (Input[5].Value[0] == 'N')
      md.Persistence = MQPER_NOT_PERSISTENT;

    md.CodedCharSetId = CCSID_EBCDIC;

    out(sf("ECB %d started to put %d and get %d",
           numtoput, numtoput, numtoget));

    MQCONN("", &Hconn, &CompCode, &Reason);            // Connect to locally defined Queue Manager
    sprintf(msg, "ECB %d MQCONN", numtoput);
    CheckError(msg, Reason);

    MQOPEN(Hconn, &ObjDesc, MQOO_INPUT_AS_Q_DEF + MQOO_OUTPUT,
           &Hobj, &CompCode, &Reason);                 // Open queue
    sprintf(msg, "ECB %d MQOPEN", numtoput);
    CheckError(msg, Reason);

    for (int loop = 1; loop <= 2; loop++)
    {
      TX_ErrorCode = tx_begin();
      CheckError(sf("ECB %d tx_begin", numtoput), TX_ErrorCode);
      FileSomething('A');

      for (i = 1; i <= numtoput; i++)
      {
        sprintf(msg, "ECB %d message %d", numtoput, i);
        MQPUT(Hconn, Hobj, &md, &pmo, strlen(msg) + 1, msg,
              &CompCode, &Reason);
        CheckError(sf("ECB %d MQPUT", numtoput), Reason);
      }

      if (loop == 1)
      {
        TX_ErrorCode = tx_rollback();
        sprintf(msg, "ECB %d tx_rollback", numtoput);
      }
      else
      {
        TX_ErrorCode = tx_commit();
        sprintf(msg, "ECB %d tx_commit", numtoput);
      }

      CheckError(msg, TX_ErrorCode);
    }

    for (int loop = 1; loop <= 2; loop++)
    {
      TX_ErrorCode = tx_begin();
      CheckError(sf("ECB %d tx_begin", numtoput), TX_ErrorCode);
      FileSomething('C');

      for (i = 1; i <= numtoget;)
      {
        MQGET(Hconn, Hobj, &md, &gmo, 100, (MQBYTE *) msg,
              &msgLen, &CompCode, &Reason);

        if (Reason == MQRC_NO_MSG_AVAILABLE)
        {
          Trace(sf("ECB %d MQGET - No message available, sleep(5)",
                   numtoput));
          sleep(5);
          tpf_yieldc(YIELDC_READY_LIST);
        }
        else if ((Reason == MQRC_NONE) ||
                 (Reason == MQRC_TRUNCATED_MSG_ACCEPTED))
        {
          i++;
          Trace(sf("ECB %d MQGET", numtoput));
        }
        else
        {
          CheckError(sf("ECB %d MQGET", numtoput), Reason);
          break;
        }
      }

      if (loop == 1)
      {
        TX_ErrorCode = tx_rollback();
        sprintf(msg, "ECB %d tx_rollback", numtoput);
      }
      else
      {
        TX_ErrorCode = tx_commit();
        sprintf(msg, "ECB %d tx_commit", numtoput);
      }

      CheckError(msg, TX_ErrorCode);
    }

    MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
    CheckError(sf("ECB %d MQCLOSE", numtoput), Reason);

    MQDISC(&Hconn, &CompCode, &Reason);
    CheckError(sf("ECB %d MQDISC", numtoput), Reason);

    out(sf("ECB %d completed", numtoput));
  }
}


void QMQZ_Put(sInput *Input)
{ QMQZ_Putx(Input, FALSE); }

void QMQZ_Put1(sInput *Input)
{ QMQZ_Putx(Input, TRUE); }

static void QMQZ_Putx(sInput *Input, boolean Put1)
{
  MQueue  myQueue;                // Contains data for the MQ APIs
  char*   message;                // message to put to queue
  char    zmsg[250] = "";         // buffer for message to other ECBs
  int     ecb_count = 1;          // Number of ECBs to start
  int     numMsgs = 1;            // Number of msgs to MQPUT
  int     msgLength = 0;          // message length
  int     putLength = 0;          // data length
  int     RiplPoint = 0;          // Messages to put before ZRIPLing
  int     comp = 0;
  time_t  ltime;
  String  eyecatcher;
  String  MsgID;                  // message ID for match
  String  CorrelID;               // correlation ID for match
  String  ExpiryParm;             // expiry parm for other ECBs
  String  file;
  enum {NOCS, COMMIT, ROLLBACK} commitScope = NOCS;
  int     csSleepTime = 0;

  myQueue.Conn(Input[1].Value.C());
  myQueue.SetObjectName(Input[2].Value.C());

  if (Input[3].Param == QN)
  {
    myQueue.SetQueueManagerAlias(Input[3].Value.C());
    Input++;
  }

  // Set defaults.

  myQueue.SetPersistence(MQPER_PERSISTENCE_AS_Q_DEF);

  // Keyword Parameters

  for (int i = 3; Input[i].Param != ""; i++)
  {
    if (Input[i].Param == "Nmsg-d*")
      CheckNumMsgs(numMsgs = Input[i].Value.ToInt());

    else if (Input[i].Param == PARM_LENGTH)
      CheckMsgLength(msgLength = Input[i].Value.ToInt());

    else if (Input[i].Param == "(NO)PERsis")
    {
      if (Input[i].Value[0] == 'N' || Input[i].Value[0] == 'n')
        myQueue.SetPersistence(MQPER_NOT_PERSISTENT);
      else if (Input[i].Value[0] == 'Y' || Input[i].Value[0] == 'y')
        myQueue.SetPersistence(MQPER_PERSISTENT);
    }

    else if (Input[i].Param == "ECB-d*")         // Spawn ECBs
      CheckNumEcbs(ecb_count = Input[i].Value.ToInt());

    else if (Input[i].Param == "RIPL-d*")
      CheckRiplPoint(RiplPoint = Input[i].Value.ToInt(), numMsgs);

    else if (Input[i].Param == PARM_DATA)
      eyecatcher = Input[i].Value.C();

    else if (Input[i].Param == PARM_MSGID)
    {
      MsgID = Input[i].Value;
      myQueue.SetMsgID(MsgID.C());
    }

    else if (Input[i].Param == PARM_CORRELID)
    {
      CorrelID = Input[i].Value;
      myQueue.SetCorrelID(CorrelID.C());
    }

    else if (Input[i].Param == PARM_EXPIRY_L)    // expiry time
    {
      ExpiryParm = Input[i].Value;
      myQueue.SetExpiry(ExpiryParm.ToInt());
    }

    else if (Input[i].Param == PARM_EXPIRY_U)      // unlimited expiry
    {
      ExpiryParm = Input[i].Value;
      myQueue.SetExpiry(MQEI_UNLIMITED);
    }

    else if (Input[i].Param == PARM_EXPIRY_N)      // negative expiry time
    {
      ExpiryParm = Input[i].Value;
      myQueue.SetExpiry(Input[i].Value.ToNegInt());
    }

    else if (Input[i].Param == PARM_SCOPE_C)
      commitScope = COMMIT;

    else if (Input[i].Param == PARM_SCOPE_R)
      commitScope = ROLLBACK;

    else if (Input[i].Param == PARM_SCOPE_SLP)
      csSleepTime = Input[i].Value.ToInt();
    else if (Input[i].Param == PARM_FILE) 
      file = Input[i].Value.C();
    else if (Input[i].Param == PARM_COMPRESS) 
      comp = Input[i].Value.ToInt();
  }  // no more parameters

  if (ecb_count > 1)
  {
    char  Buffer1[5];
    char  Buffer2[MAX_ERR_MSG_BUF + 1];
    char  Buffer3[MAX_ERR_MSG_BUF + 1];
    char  Buffer4[MAX_ERR_MSG_BUF + 1];
    char  Buffer5[MAX_ERR_MSG_BUF + 1];

    if (Put1)
    {
      sprintf(zmsg, "ZTEST MQS1 MQPUT1 %s %s %s LEN-%d %s %s %s %s %s %s %s %s",
            myQueue.GetManagerName(),
            myQueue.GetObjectName(),
            myQueue.GetManagerAlias(),
            msgLength,
            myQueue.GetPersistenceString(),
            (eyecatcher == "" ? "" : sf("DATA-%s", eyecatcher.C(), Buffer1)),
            (MsgID == "" ? "" : sf("MSGID-%s", MsgID.C(), Buffer2)),
            (CorrelID == "" ? "" : sf("CORRELID-%s", CorrelID.C(), Buffer3)),
            (ExpiryParm == "" ? "" : sf("EXPIRY-%s", ExpiryParm.C(), Buffer4)),
            (commitScope == NOCS ? "" : sf("SCOPE-%s SCOPESL-%d",
                         commitScope==COMMIT ? "COMMIT" : "ROLLBACK",
                         csSleepTime, Buffer5)),
            (file == "" ? "" : sf("FILE-%s",file.C())),
            (comp == 0 ? "" : sf("COMP-%d",comp))
            );
    }
    else
    {
      sprintf(zmsg, "ZTEST MQS1 MQPUT %s %s %s N-%d LEN-%d %s %s %s %s %s %s %s %s",
            myQueue.GetManagerName(),
            myQueue.GetObjectName(),
            myQueue.GetManagerAlias(),
            numMsgs,
            msgLength,
            myQueue.GetPersistenceString(),
            (eyecatcher == "" ? "" : sf("DATA-%s", eyecatcher.C(), Buffer1)),
            (MsgID == "" ? "" : sf("MSGID-%s", MsgID.C(), Buffer2)),
            (CorrelID == "" ? "" : sf("CORRELID-%s", CorrelID.C(), Buffer3)),
            (ExpiryParm == "" ? "" : sf("EXPIRY-%s", ExpiryParm.C(), Buffer4)),
            (commitScope == NOCS ? "" : sf("SCOPE-%s SCOPESL-%d",
                         commitScope==COMMIT ? "COMMIT" : "ROLLBACK",
                         csSleepTime, Buffer5)),
            (file == "" ? "" : sf("FILE-%s",file.C())),
            (comp == 0 ? "" : sf("COMP-%d",comp))
            );
    }

    outc(sf("Creating %d ECB%s with the following message:",
            ecb_count - 1, ecb_count > 2 ? "s" : ""));
    oute(zmsg);
    CreateECBs(ecb_count - 1, zmsg);
    time(&ltime);
    out(sf("Start putting messages at : %s", ctime(&ltime)));
  }

  myQueue.SetOpenOptions(MQOO_OUTPUT);
  int fileLength = 0;
  char * compBuffer = NULL;
  if (file == "") {
     if (((message = (char*) malloc(msgLength)) == NULL) && msgLength)
       Error("No malloc space");
     memset(message, 'Z', msgLength);
  } else {
     std::ifstream fsIn;
     fsIn.open(file.C());
     fsIn.seekg(0, std::ios::end);
     fileLength = fsIn.tellg(); 
     if (fileLength == -1) 
       Error("Could not read file");
     if (msgLength > fileLength) 
       Error(sf("File only contains %d bytes of data",fileLength));
     if (((message = (char*) malloc(fileLength)) == NULL) && fileLength)
       Error("No malloc space");
     if (comp)
       if (((compBuffer = (char*) malloc(fileLength)) == NULL) && fileLength)
         Error("No malloc space");
     fsIn.seekg(0, std::ios::beg);
     fsIn.read((char *) message, fileLength);
     if (msgLength == 0) msgLength = fileLength;
  }
  int ret;
  if (Put1)
  {
    if (fileLength == 0) {
       MessagetoPut(message, 1, msgLength);
    } 
    if (comp > 0) {
       z_stream strm;
       strm.zalloc = Z_NULL;
       strm.zfree = Z_NULL;
       strm.opaque = Z_NULL;
       ret = deflateInit(&strm, comp);
       strm.avail_in = msgLength;  
       strm.next_in = (Bytef *)message;
       strm.avail_out = fileLength;
       strm.next_out = (Bytef *)compBuffer;
       ret = deflate(&strm, Z_FINISH);
       if (ret == Z_STREAM_ERROR) {
          Error("Issue during deflate");
       }
       int deflSize = fileLength - strm.avail_out;
       int compration = (deflSize*100)/msgLength;
       out(sf("compression ratio of %d%% %d-%d",compration,msgLength,deflSize));
       myQueue.Put1(compBuffer, deflSize);
       putLength = deflSize;
    } else {
       myQueue.Put1(message, msgLength);
       putLength = msgLength; 
    }
  }
  else
  {
    char MsgIDA[25] = "";
    char CorrelIDA[25] = "";

    myQueue.Open();

    if (commitScope != NOCS)
       tx_begin();

    for (int n = 1; n <= numMsgs; n++)           // put numMsgs messages
    {
      if (numMsgs > 1)
      {
         if (MsgID != "")
         {
            sprintf(MsgIDA, "%s%d", MsgID.C(), n);
            myQueue.SetMsgID(MsgIDA);
         }
         if (CorrelID != "")
         {
            sprintf(CorrelIDA, "%s%d", CorrelID.C(), n);
            myQueue.SetCorrelID(CorrelIDA);
         }
      }

      if (fileLength == 0) {
         MessagetoPut(message, n, msgLength);
      }
      if (comp > 0) {
         z_stream strm;
         strm.zalloc = Z_NULL;
         strm.zfree = Z_NULL;
         strm.opaque = Z_NULL;
         ret = deflateInit(&strm, comp);
         strm.avail_in = msgLength;
         strm.next_in = (Bytef *)message;
         strm.avail_out = fileLength;
         strm.next_out = (Bytef *)compBuffer;
         ret = deflate(&strm, Z_FINISH);
         if (ret == Z_STREAM_ERROR) {
            Error("Issue during deflate");
         }
         int deflSize = fileLength - strm.avail_out;
         putLength = deflSize;
         myQueue.Put(compBuffer, deflSize);
      } else {
         if (eyecatcher != "")
            memcpy(message, eyecatcher.C(), eyecatcher.Length());

         myQueue.Put(message, msgLength);
         putLength = msgLength; 
      }

      // For every 200 runs, issue an defrc to avoid CTL-10
      if (n%200 == 199)
         defrc();

      if (RiplPoint == n && RiplPoint != numMsgs)  // Check RIPL Point
        Ripl();
    }

    if (csSleepTime != 0)
    {
       outc("Finished all MQPUTs. All new messages are un-commited");
       oute(sf("Sleep for %d seconds before committing/rolling back the scope",
                       csSleepTime));
       sleep(csSleepTime);
    }

    switch (commitScope)
    {
       case COMMIT:
          oute("Issuing tx_commit to commit all MQPUTs");
          tx_commit();
          break;
       case ROLLBACK:
          oute("Issuing tx_rollback to rollback all MQPUTs");
          tx_rollback();
          break;
       default:
       case NOCS:
          break;
    }

    myQueue.Close();
  }

  myQueue.Disc();

  if (RiplPoint == numMsgs)                      // Check RIPL Point
    Ripl();

  if (myQueue.GetPersistence() == MQPER_NOT_PERSISTENT)
  {
    // Display non-pers message added successfully
    out(sf("%d non-persistent message%s of size %d added to %s",
           numMsgs, numMsgs > 1 ? "s" : "",
           putLength, myQueue.GetObjectName()));
  }
  else if (myQueue.GetPersistence() == MQPER_PERSISTENT)
  {
    // Display pers message added successfully
    out(sf("%d persistent message%s of size %d added to %s",
           numMsgs, numMsgs > 1 ? "s" : "",
           putLength, myQueue.GetObjectName()));
  }
  else
  {
    // Display message ..added as Q defined
    out(sf("%d message%s of size %d added to %s",
           numMsgs, numMsgs > 1 ? "s" : "",
           putLength, myQueue.GetObjectName()));
  }
}


void QMQZ_Get(sInput *Input)
{
  struct  timeval StartTime;         // Starting Time
  struct  timeval Current;           // Current Time
  struct  timezone NoUse;            // Dummy Variable
  MQueue  myQueue;                   // Contains data for the MQ APIs
  char*   buffer;                    // message retrieved from queue
  char    zMessage[250];             // buffer for message to other ECBs
  int     numECBs = 0;               // Number of ECBs to start
  int     numMessages = 0;           // Number of msgs to MQGET
  int     numSeconds = 0;            // Length of time to MQGET
  int     messageLength = 0;         // message length
  int     messagesBrowsed = 0;
  int     messagesGotten  = 0;
  int     messagesReceived = 0;
  long    waitTime;                  // Amount of time to wait
  String  MsgID;                     // message ID for match
  String  CorrelID;                  // correlation ID for match
  String  WaitParm;                  // wait parm for other ECBs
  String  BrowseParm;                // browse parm for other ECBs
  String  WakeUpParm;                // wakeup parm for other ECBs
  enum {NOCS, COMMIT, ROLLBACK} commitScope = NOCS;
  long    csSleepTime = 0;
  char *  fileNameParm = NULL;                                                  //26653
  MQLONG Reason =  MQRC_NONE;                                                   //26653

  // Print parameter values.
  enum {RESTRICT, NONE, SOME, ALL} Printing = RESTRICT;
  // If Printing is RESTRICT, don't print if numMessages
  // or numSeconds is greater than this value.
  const int PRINT_RESTRICT = 25;
  // If Print is SOME, this is how many messages to print.
  int PrintSome = 0;

  // Browse parameter values.
  enum {NOBROWSE, READ, GET, LOCK} Browsing = NOBROWSE;
  // Number of seconds to lock the last msg browsed before unlocking it.
  // Zero indicates that no locking is to occur.
  int SleepTime = 0;

  // Wakeup action parameter values.
  enum {WK_EXIT, WK_SERRC, WK_LOCK, WK_LOCKN, WK_UNLOCK,
        WK_BROWSE, WK_BRN, WK_GET}
       WakeUp = WK_EXIT;

  myQueue.Conn(Input[1].Value.C());
  myQueue.SetObjectName(Input[2].Value.C());

  myQueue.SetGetOptions(MQGMO_ACCEPT_TRUNCATED_MSG);
  myQueue.ClearMatchOptions();

  for (int i = 3; Input[i].Param != ""; i++)
  {
    if (Input[i].Param == "Nmsg-d*")             // Number of messages
    {
      CheckNumMsgs(numMessages = Input[i].Value.ToInt());

      // If we are getting more than one message check the seq number
      if (numMessages > 1)
         myQueue.SetSeqChk('Y');
    }

    else if (Input[i].Param == "Sec-d*")         // Seconds
      numSeconds = Input[i].Value.ToInt();

    else if (Input[i].Param == PARM_LENGTH)      // LENgth (required)
      CheckMsgLength(messageLength = Input[i].Value.ToInt());

    else if (Input[i].Param == PARM_NOPRT)       // (NO)PRT
      Printing = (Input[i].Value[0] == 'N') ? NONE : ALL;

    else if (Input[i].Param == PARM_PRTN)        // PRTN-d*
    {
      Printing = SOME;
      PrintSome = Input[i].Value.ToInt();
    }

    else if (Input[i].Param == PARM_BR_R)        // BROWSE keyword
    {
      BrowseParm = Input[i].Value;
      Browsing = READ;
    }

    else if (Input[i].Param == PARM_BR_G)
    {
      BrowseParm = Input[i].Value;
      Browsing = GET;
    }

    else if (Input[i].Param == PARM_BR_L)
    {
      BrowseParm = Input[i].Value;
      Browsing = LOCK;
    }

    else if (Input[i].Param == PARM_WAKEUP_EXIT) // WakeUp Action
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_EXIT;
    }

    else if (Input[i].Param == PARM_WAKEUP_SERRC)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_SERRC;
    }

    else if (Input[i].Param == PARM_WAKEUP_LOCK)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_LOCK;
    }

    else if (Input[i].Param == PARM_WAKEUP_LOCKN)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_LOCKN;
    }

    else if (Input[i].Param == PARM_WAKEUP_UNLOCK)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_UNLOCK;
    }

    else if (Input[i].Param == PARM_WAKEUP_BROWSE)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_BROWSE;
    }

    else if (Input[i].Param == PARM_WAKEUP_BRN)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_BRN;
    }

    else if (Input[i].Param == PARM_WAKEUP_GET)
    {
      WakeUpParm = Input[i].Value;
      WakeUp = WK_GET;
    }

    else if (Input[i].Param == PARM_SLEEP)       // Sleep Period for Browse
      SleepTime = Input[i].Value.ToInt();

    else if (Input[i].Param == "ECB-d*")         // ECB
      CheckNumEcbs(numECBs = Input[i].Value.ToInt());

    else if (Input[i].Param == PARM_WAIT_L)      // wait time
    {
      WaitParm = Input[i].Value;
      waitTime = Input[i].Value.ToInt();
      myQueue.SetWaitInterval(waitTime);
      myQueue.AddGetOptions(MQGMO_WAIT);
    }

    else if (Input[i].Param == PARM_WAIT_U)      // unlimited wait
    {
      WaitParm = Input[i].Value;
      waitTime = MQWI_UNLIMITED;
      myQueue.SetWaitInterval(waitTime);
      myQueue.AddGetOptions(MQGMO_WAIT);
    }

    else if (Input[i].Param == PARM_WAIT_N)      // negative wait time
    {
      WaitParm = Input[i].Value;
      waitTime = Input[i].Value.ToNegInt();
      myQueue.SetWaitInterval(waitTime);
      myQueue.AddGetOptions(MQGMO_WAIT);
    }

    else if (Input[i].Param == PARM_SEQCHK)      // (NO)SEQchk
      myQueue.SetSeqChk(Input[i].Value[0] == 'Y');

    else if (Input[i].Param == PARM_MSGID)       // MsgID
    {
      MsgID = Input[i].Value;
      myQueue.AddMatchMsgID(MsgID.C());
    }

    else if (Input[i].Param == PARM_CORRELID)    // CorrelID
    {
      CorrelID = Input[i].Value;
      myQueue.AddMatchCorrelID(CorrelID.C());
    }

    else if (Input[i].Param == PARM_SCOPE_C)     // Commit Scope
      commitScope = COMMIT;

    else if (Input[i].Param == PARM_SCOPE_R)
      commitScope = ROLLBACK;

    else if (Input[i].Param == PARM_SCOPE_SLP)   // Sleep Time before commit/rollback
      csSleepTime = Input[i].Value.ToInt();

    else if (Input[i].Param == "Ver-d*")         // GMO Version                 //d26214
    {                                                                           //d26214
      CheckGmoVersion(Input[i].Value.ToInt());                                  //d26214
      myQueue.SetGmoVersion(Input[i].Value.ToInt());                            //d26214
    }                                                                           //d26214
    else if (Input[i].Param == PARM_FILE)         // File                       //26653
    {                                                                           //26653
      fileNameParm = Input[i].Value.C();                                        //26653
    }                                                                           //26653

  }  // no more parameters

  if (Browsing == READ && (WakeUp == WK_BRN || WakeUp == WK_BROWSE))
    myQueue.SetOpenOptions(MQOO_BROWSE);
  else if (Browsing != NOBROWSE)
    myQueue.SetOpenOptions(MQOO_INPUT_AS_Q_DEF | MQOO_BROWSE);
  else
    myQueue.SetOpenOptions(MQOO_INPUT_AS_Q_DEF);

  if (numECBs > 1)
  {
    char  Buffer1[MAX_ERR_MSG_BUF + 1];
    char  Buffer2[MAX_ERR_MSG_BUF + 1];
    char  Buffer3[MAX_ERR_MSG_BUF + 1];
    char  Buffer4[MAX_ERR_MSG_BUF + 1];
    char  Buffer5[MAX_ERR_MSG_BUF + 1];
    char  Buffer6[MAX_ERR_MSG_BUF + 1];

    sprintf(zMessage,                            // Spawn ECBs
            "ZTEST MQS1 MQGET %s %s N-%d LEN-%d %s %s %s %s %s %s %s %s",
            myQueue.GetManagerName(),
            myQueue.GetObjectName(),
            numMessages,
            messageLength,
            Printing == RESTRICT ? "" : (Printing == NONE ? "NOPRT" : "PRT"),
            WaitParm == "" ? "" : sf("WAIT-%s", WaitParm.C(), Buffer1),
            myQueue.GetSeqChk() ? "SEQ" : "NOSEQ",
            BrowseParm == "" ? "" : sf("BROWSE-%s SLEEP-%d",
                                   BrowseParm.C(), SleepTime, Buffer2),
            WakeUpParm == "" ? "" : sf("WAKEUP-%s", WakeUpParm.C(), Buffer3),
            MsgID == "" ? "" : sf("MSGID-%s", MsgID.C(), Buffer4),
            CorrelID == "" ?
              "" : sf("CORRELID-%s", CorrelID.C(), Buffer5),
            (commitScope == NOCS ? "" : sf("SCOPE-%s SCOPESL-%d",
                         commitScope==COMMIT ? "COMMIT" : "ROLLBACK",
                         csSleepTime, Buffer6)));

    outc(sf("Creating %d ECB%s with the following message:",
            numECBs - 1, numECBs > 2 ? "s" : ""));

    oute(zMessage);
    CreateECBs(numECBs - 1, zMessage);
  }

  myQueue.SetGetError(MQRC_NO_MSG_AVAILABLE);
  myQueue.Open();

  if ((buffer = (char*) malloc(messageLength + 1)) == NULL)
    Error("No malloc space");

  if (Printing == ALL || (Printing == SOME && PrintSome > 0))
    myQueue.SetGetPrint(TRUE);

  if (numMessages)
  {
    if (commitScope != NOCS)
    {
       oute("Issuing tx_begin to open a commit scope for all MQGETs");
       tx_begin();
    }

    if (Printing == RESTRICT && numMessages <= PRINT_RESTRICT)
      myQueue.SetGetPrint(TRUE);

    // if the ECB is getting more than 10 messages, don't issue printf for the time waited.
    if (numMessages > 10)
      myQueue.SetWaitPrint(FALSE);

    // add 1 to the number of messages to be browsed and get.
    // If user inputs n-3 to get the third message,
    // 4 MQGET actions are needed, browse_first -> browse_next -> browse_next -> get_under_cursor
    if (Browsing == GET)
       numMessages++;

    while (messagesReceived < numMessages)
    {
      // For every 200 messages received, issue an defrc to avoid CTL-10
      if (messagesReceived%200 == 199)
         defrc();

      memset(buffer, 0, messageLength);

      if (Browsing != NOBROWSE)
      {
        if (messagesReceived == 0)
          myQueue.AddGetOptions(MQGMO_BROWSE_FIRST);
        else
          // Remove MQGMO_BROWSE_FIRST and add MQGMO_BROWSE_NEXT.
          myQueue.AddGetOptions(MQGMO_BROWSE_NEXT, MQGMO_BROWSE_FIRST);
      }

      // If this is the nth message and LOCK was specified, lock this message.
      // If GET was specified, get the message.

      if (messagesReceived == numMessages - 1)
      {
        if (Browsing == LOCK)
          myQueue.AddGetOptions(MQGMO_LOCK);
        else if (Browsing == GET)
        {
          myQueue.SetGetOptions(MQGMO_ACCEPT_TRUNCATED_MSG |
                                MQGMO_MSG_UNDER_CURSOR);
        }
      }

      //check if we should validate the contents and size of message
      if (fileNameParm) {                                                       //26653
        Reason = myQueue.Get(buffer, messageLength, fileNameParm, NULL);        //26653
      }                                                                         //26653
      else {                                                                    //26653
         Reason = myQueue.Get(buffer, messageLength);                           //26653
      }                                                                         //26653

      if (Reason == MQRC_NO_MSG_AVAILABLE)                                      //26653
      {
        outc("MQGET ended with MQRC_NO_MSG_AVAILABLE");
        break;
      }
      else
      {
        messagesReceived++;

        // If this is a destructive get, or it's the get under cursor action for
        // browse, increment the number of get actions.
        // Otherwise, increment the number of browse actions.
        if (Browsing == NOBROWSE ||
             (messagesReceived == numMessages && Browsing == GET))
           messagesGotten++;
        else
           messagesBrowsed++;

        if (Printing == SOME && messagesReceived == PrintSome)
          myQueue.SetGetPrint(FALSE);

        if (myQueue.GetSeqChk())
          checkMsgSeq(buffer);

        // If the next one is the last message this ECB is getting before it sleeps,
        // print out the last message and print out the wakeup get action so that
        // user can know what's going on.
        if (messagesReceived == (numMessages-1))
        {
           myQueue.SetGetPrint(TRUE);
        }

        // If this is the nth message and SLEEP-s was specified,
        // wait s seconds and check for the WAKEUP action.

        if (messagesReceived == numMessages && Browsing != NOBROWSE)
        {
          long Opts = 0;

          if (SleepTime != 0)
          {
             oute(sf("MQGET sleeps for %d seconds", SleepTime));
             sleep(SleepTime);
          }

          switch (WakeUp)
          {
             case WK_EXIT:
                oute(sf("MQGET Browsed %d Messages, Got %d Messages, and exited",
                            messagesBrowsed, messagesGotten));
                myQueue.Close();
                myQueue.Disc();
                exit(0);
                break;

             case WK_SERRC:
                oute(sf("MQGET Browsed %d Messages, Got %d Messages",
                            messagesBrowsed, messagesGotten));
                myQueue.Close();
                myQueue.Disc();
                serrc_op(SERRC_EXIT,0xABCDE,"QMQZ TAKES A SERRC WITH EXIT",NULL);
                break;

             case WK_LOCK:
                Opts = MQGMO_LOCK | MQGMO_BROWSE_MSG_UNDER_CURSOR;
                break;

             case WK_LOCKN:
                Opts = MQGMO_LOCK | MQGMO_BROWSE_NEXT;
                break;

             case WK_UNLOCK:
                Opts = MQGMO_UNLOCK;
                break;

             case WK_BROWSE:
                Opts = MQGMO_BROWSE_MSG_UNDER_CURSOR;
                break;

             case WK_BRN:
                Opts = MQGMO_BROWSE_NEXT;
                break;

             case WK_GET:
                Opts = MQGMO_ACCEPT_TRUNCATED_MSG | MQGMO_MSG_UNDER_CURSOR;
                break;
          }

          // Set the WAKEUP action options and
          // perform an MQGET with that action.

          myQueue.SetGetOptions(Opts);
          Reason = myQueue.Get(buffer, messageLength);                          //26653

          if (Reason == MQRC_NONE ||
              Reason == MQRC_TRUNCATED_MSG_ACCEPTED)
          {
             switch (WakeUp)
             {
                case WK_BRN:
                case WK_LOCKN:
                   messagesBrowsed++;
                   break;
                case WK_GET:
                   messagesGotten++;
                   break;
                case WK_EXIT:
                case WK_SERRC:
                case WK_LOCK:
                case WK_UNLOCK:
                case WK_BROWSE:
                default:
                   break;
             }
          }
          else
          {
            outc(sf("MQGET-WAKEUP action ended with reason code %u, %s",
                    Reason, MQRC_String(Reason)));
          }

          // If WAKEUP action results in locking a message. Sleep again for
          // tester to verify the lock.
          if ((WakeUp == WK_LOCK || WakeUp == WK_LOCKN)
                             && SleepTime != 0)
             sleep(SleepTime);
        }
      }
    }
  }
  else
  {
    if (Printing == RESTRICT && numSeconds <= PRINT_RESTRICT)
      myQueue.SetGetPrint(TRUE);

    gettimeofday(&StartTime, &NoUse);
    gettimeofday(&Current, &NoUse);

    while (Current.tv_sec-StartTime.tv_sec < numSeconds)
    {
      memset(buffer, 0, messageLength);

      if (myQueue.Get(buffer, messageLength) == MQRC_NO_MSG_AVAILABLE)
        sleep(1);
      else
      {
        messagesReceived++;

        if (Printing == SOME && messagesReceived == PrintSome)
          myQueue.SetGetPrint(FALSE);

        if (myQueue.GetSeqChk())
          checkMsgSeq(buffer);
      }

      gettimeofday(&Current, &NoUse);
    }
  }

  if (csSleepTime != 0)
  {
     outc("Finished all MQGETs. All Gotten messages are un-commited");
     oute(sf("Sleep for %d seconds before committing/rolling back the scope",
                csSleepTime));
     sleep(csSleepTime);
  }

  switch (commitScope)
  {
     case COMMIT:
        oute("Issuing tx_commit to commit all MQGETs");
        tx_commit();
        break;

     case ROLLBACK:
        oute("Issuing tx_rollback to rollback all MQGETs");
        tx_rollback();
        break;

     default:
     case NOCS:
        break;
  }

  myQueue.Close();
  myQueue.Disc();

  if (numSeconds != 0)
    outc(sf("%d seconds have elapsed", numSeconds));

  oute(sf("MQGET Browsed %d Messages, and Got %d Messages.",
              messagesBrowsed, messagesGotten));
}

/*********/
/* MQINQ */
/*********/

void QMQZ_Inq(sInput *Input)
{
  const MQLONG ChrSize = MQ_Q_NAME_LENGTH * 2
                         + MQ_Q_MGR_NAME_LENGTH
                         + MQ_PROCESS_NAME_LENGTH
                         + MQ_TRIGGER_DATA_LENGTH
                         + MQ_CREATION_DATE_LENGTH
                         + MQ_CREATION_TIME_LENGTH;
  char   *data = (char *) malloc(ChrSize);
  char    ChrAttrs[ChrSize + 1] = {0};
  MQOD    ObjDesc = {MQOD_DEFAULT};
  MQLONG  CompCode, Reason;
  MQLONG  ObjectType;
  MQLONG  QType;
  MQLONG  IntAttrs[18];
  MQHOBJ  Hobj;
  MQHCONN Hconn;
  MQCHAR  QName[MQ_Q_NAME_LENGTH + 1] = "";

  MQCONN("", &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);

  if (Input[1].Param == "QMGR")
  {
    ObjDesc.ObjectType = MQOT_Q_MGR;
    ObjectType = MQOT_Q_MGR;
    ObjDesc.ObjectName[0] = '\0';
    strcpy(ObjDesc.ObjectQMgrName,
           ((MQP_MEM*) cinfc(CINFC_READ, CINFC_CMMMQT))->QManagerName);
  }
  else
  {
    Input[1].Value.CopyTo(ObjDesc.ObjectName);
    ObjectType = MQOT_Q;
  }

  MQOPEN(Hconn, &ObjDesc, MQOO_INQUIRE, &Hobj, &CompCode, &Reason);

  // If we were trying to open a queue and
  // it's not found, see if it's a process.

  if (Reason == MQRC_UNKNOWN_OBJECT_NAME && ObjectType == MQOT_Q)
  {
    MQLONG cc, rc;

    ObjDesc.ObjectType = MQOT_PROCESS;
    MQOPEN(Hconn, &ObjDesc, MQOO_INQUIRE, &Hobj, &cc, &rc);

    if (rc == MQRC_NONE)
    {
      CompCode = cc;
      Reason = rc;
      ObjectType = MQOT_PROCESS;
    }
  }

  CheckError("MQOPEN", Reason);

  if (ObjectType == MQOT_Q)
  {
    MQLONG Selectors[] = { MQIA_Q_TYPE, MQCA_Q_NAME };

    MQINQ(Hconn, Hobj, 2, Selectors, 1, &QType,
          MQ_Q_NAME_LENGTH, QName,
          &CompCode, &Reason);

    CheckError("MQINQ - get QType:", Reason);
    trunc(QName);
  }

  switch(ObjectType)
  {
    case MQOT_Q:
    {
      switch (QType)
      {
        case MQQT_LOCAL:
        {
          MQLONG Selectors[] = { MQIA_USAGE,
                                 MQIA_INHIBIT_PUT,
                                 MQIA_INHIBIT_GET,
                                 MQIA_MAX_MSG_LENGTH,
                                 MQIA_Q_DEPTH_HIGH_LIMIT,
                                 MQIA_Q_DEPTH_LOW_LIMIT,
                                 MQIA_SHAREABILITY,
                                 MQIA_DEF_PERSISTENCE,
                                 MQIA_CURRENT_Q_DEPTH,
                                 MQIA_MAX_Q_DEPTH,
                                 MQIA_TRIGGER_CONTROL,
                                 MQIA_TRIGGER_TYPE,
                                 MQCA_PROCESS_NAME,
                                 MQCA_TRIGGER_DATA,
                                 MQIA_Q_DEPTH_MAX_EVENT,
                                 MQIA_Q_DEPTH_HIGH_EVENT,
                                 MQIA_Q_DEPTH_LOW_EVENT,
                                 MQIA_Q_SERVICE_INTERVAL_EVENT,
                                 MQIA_Q_SERVICE_INTERVAL,
                                 MQCA_CREATION_DATE,
                                 MQCA_CREATION_TIME
                               };

          MQINQ(Hconn, Hobj, 21, Selectors, 17, IntAttrs, ChrSize,
                data, &CompCode, &Reason);

          if (CompCode == MQCC_FAILED)
            CheckError("MQINQ", Reason);

          String Chardata(data);
          char* E = "ENABLED";
          char* D = "DISABLED";

          outc(sf("Local Queue %s", QName));
          outc(sf("QType              = %d", QType));
          outc(sf("Usage              = %s (%d)",
                  IntAttrs[0] == MQUS_NORMAL       ? "NORMAL" :
                  IntAttrs[0] == MQUS_TRANSMISSION ? "XMITQ"  :
                                                     "?",
                  IntAttrs[0]));
          outc(sf("InhibitPut         = %s (%d)",
                  IntAttrs[1] == MQQA_PUT_INHIBITED ? "INHIBITED" :
                  IntAttrs[1] == MQQA_PUT_ALLOWED   ? "ALLOWED"   :
                                                      "?",
                  IntAttrs[1]));
          outc(sf("InhibitGet         = %s (%d)",
                  IntAttrs[2] == MQQA_GET_INHIBITED ? "INHIBITED" :
                  IntAttrs[2] == MQQA_GET_ALLOWED   ? "ALLOWED"   :
                                                      "?",
                  IntAttrs[2]));
          outc(sf("MaxMsgLen          = %d", IntAttrs[3]));
          if (IntAttrs[4] == MQIAV_NOT_APPLICABLE)
            outc("Q Depth High Limit = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Depth High Limit = %d", IntAttrs[4]));
          if (IntAttrs[5] == MQIAV_NOT_APPLICABLE)
            outc("Q Depth Low Limit  = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Depth Low Limit  = %d", IntAttrs[5]));
          outc(sf("Shareability       = %s (%d)",
                  IntAttrs[6] == MQQA_SHAREABLE     ? "SHAREABLE"     :
                  IntAttrs[6] == MQQA_NOT_SHAREABLE ? "NOT SHAREABLE" :
                                                      "?",
                  IntAttrs[6]));
          outc(sf("Default Persistence= %s (%d)",
                  IntAttrs[7] == MQPER_NOT_PERSISTENT ? "NOT PERSISTENT" :
                  IntAttrs[7] == MQPER_PERSISTENT     ? "PERSISTENT"     :
                                                        "?",
                  IntAttrs[7]));
          outc(sf("Current Q Depth    = %d", IntAttrs[8]));
          if (IntAttrs[9] == MQIAV_NOT_APPLICABLE)
            outc("Max Q Depth        = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Max Q Depth        = %d", IntAttrs[9]));
          outc(sf("Trigger Control    = %s (%d)",
                  IntAttrs[10] == MQTC_OFF           ? "OFF"           :
                  IntAttrs[10] == MQTC_ON            ? "ON"            :
                  IntAttrs[10] == MQTC_NOT_SPECIFIED ? "NOT SPECIFIED" :
                                                       "?",
                  IntAttrs[10]));
          outc(sf("Trigger Type       = %s (%d)",
                  IntAttrs[11] == MQTT_NONE          ? "NONE"          :
                  IntAttrs[11] == MQTT_FIRST         ? "FIRST"         :
                  IntAttrs[11] == MQTT_EVERY         ? "EVERY"         :
                  IntAttrs[11] == MQTT_NOT_SPECIFIED ? "NOT SPECIFIED" :
                                                       "?",
                  IntAttrs[11]));
          outc(sf("Process Name       = %s",
                   Chardata.Range(0, MQ_PROCESS_NAME_LENGTH)));
          outc(sf("Trigger Data       = %s",
                  Chardata.Range(MQ_PROCESS_NAME_LENGTH,
                  MQ_TRIGGER_DATA_LENGTH)));
          if (IntAttrs[12] == MQIAV_NOT_APPLICABLE)
            outc("Q Depth Max Event  = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Depth Max Event  = %s", IntAttrs[12] ? E : D));
          if (IntAttrs[13] == MQIAV_NOT_APPLICABLE)
            outc("Q Depth High Event = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Depth High Event = %s", IntAttrs[13] ? E : D));
          if (IntAttrs[14] == MQIAV_NOT_APPLICABLE)
            outc("Q Depth Low Event  = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Depth Low Event  = %s", IntAttrs[14] ? E : D));

          if (IntAttrs[15] == MQIAV_NOT_APPLICABLE)
            outc("Q Svc Int Event    = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Svc Int Event    = %s (%d)",
                  IntAttrs[15] == MQQSIE_HIGH ? "HIGH" :
                  IntAttrs[15] == MQQSIE_OK   ? "OK"   :
                  IntAttrs[15] == MQQSIE_NONE ? "NONE" :
                                                "?",
                  IntAttrs[15]));
          if (IntAttrs[16] == MQIAV_NOT_APPLICABLE)
            outc("Q Service Interval = MQIAV_NOT_APPLICABLE");
          else
            outc(sf("Q Service Interval = %d", IntAttrs[16]));
          outc(sf("Creation Date/Time = %s",
                  Chardata.Range(MQ_PROCESS_NAME_LENGTH +
                                 MQ_TRIGGER_DATA_LENGTH,
                                 MQ_CREATION_DATE_LENGTH +
                                 MQ_CREATION_TIME_LENGTH)));
          break;
        }

        case MQQT_REMOTE:
        {
          MQLONG Selectors[] = { MQIA_INHIBIT_PUT,
                                 MQCA_REMOTE_Q_MGR_NAME,
                                 MQCA_REMOTE_Q_NAME,
                                 MQCA_XMIT_Q_NAME
                               };

          char* data = (char*) malloc(ChrSize);

          MQINQ(Hconn, Hobj, 4, Selectors, 1, IntAttrs, ChrSize,
                data, &CompCode, &Reason);
          CheckError("MQINQ", Reason);

          String Chardata(data);

          outc(sf("Remote Queue %s", QName));
          outc(sf("QType                = %d", QType));
          outc(sf("InhibitPut           = %s (%d)",
                  IntAttrs[0] == MQQA_PUT_INHIBITED ? "INHIBITED" :
                  IntAttrs[0] == MQQA_PUT_ALLOWED   ? "ALLOWED"   :
                                                      "?",
                  IntAttrs[0]));
          outc(sf("Remote Queue Manager = %48s",
                  Chardata.Range(0, MQ_Q_MGR_NAME_LENGTH)));
          outc(sf("Remote Queue         = %48s",
                  Chardata.Range(MQ_Q_MGR_NAME_LENGTH, MQ_Q_NAME_LENGTH)));
          oute(sf("Xmit   Queue         = %48s",
                  Chardata.Range(MQ_Q_MGR_NAME_LENGTH + MQ_Q_NAME_LENGTH,
                  MQ_Q_NAME_LENGTH)));
          break;
        }

        case MQQT_ALIAS:
        {
          MQLONG Selectors[] = { MQIA_INHIBIT_PUT,
                                 MQIA_INHIBIT_GET,
                                 MQCA_BASE_Q_NAME
                               };

          MQINQ(Hconn, Hobj, 3, Selectors, 2, IntAttrs, ChrSize,
                ChrAttrs, &CompCode, &Reason);
          CheckError("MQINQ", Reason);

          outc(sf("Alias Queue %s", QName));
          outc(sf("QType              = %d", QType));
          outc(sf("InhibitPut         = %s (%d)",
                  IntAttrs[0] == MQQA_PUT_INHIBITED ? "INHIBITED" :
                  IntAttrs[0] == MQQA_PUT_ALLOWED   ? "ALLOWED"   :
                                                      "?",
                  IntAttrs[0]));
          outc(sf("InhibitGet         = %s (%d)",
                  IntAttrs[1] == MQQA_GET_INHIBITED ? "INHIBITED" :
                  IntAttrs[1] == MQQA_GET_ALLOWED   ? "ALLOWED"   :
                                                      "?",
                  IntAttrs[0]));
          oute(sf("Base Queue Name    = %s", ChrAttrs));
          break;
        }

        default:
          CheckError("Queue Type Not Found in QMQZ01", IntAttrs[0]);
      } // switch (QType)

      break;
    }

    case MQOT_Q_MGR:
    {
      MQLONG Selectors[] = { MQCA_DEAD_LETTER_Q_NAME,
                             MQCA_Q_MGR_NAME,
                             MQIA_CODED_CHAR_SET_ID,
                             MQIA_MAX_MSG_LENGTH,
                             MQIA_PLATFORM,
                             MQIA_MAX_HANDLES,
                             MQIA_INHIBIT_EVENT,
                             MQIA_LOCAL_EVENT,
                             MQIA_PERFORMANCE_EVENT,
                             MQIA_REMOTE_EVENT,
                             MQIA_START_STOP_EVENT,
                             MQIA_SSL_EVENT
                           };

      char* data = (char*) malloc(ChrSize);

      MQINQ(Hconn, Hobj, 12, Selectors, 10, IntAttrs,
            ChrSize, data, &CompCode, &Reason);
      CheckError("MQINQ", Reason);

      String Chardata(data);
      char* E = "ENABLED";
      char* D = "DISABLED";

      outc(sf("CCSID             = %d", IntAttrs[0]));
      outc(sf("MaxMsgLen         = %d", IntAttrs[1]));
      outc(sf("Platform          = %d", IntAttrs[2]));
      outc(sf("MaxHandles        = %d", IntAttrs[3]));
      outc(sf("Inhibit Event     = %s", IntAttrs[4] ? E : D));
      outc(sf("Local Event       = %s", IntAttrs[5] ? E : D));
      outc(sf("Performance Event = %s", IntAttrs[6] ? E : D));
      outc(sf("Remote Event      = %s", IntAttrs[7] ? E : D));
      outc(sf("Start/Stop Event  = %s", IntAttrs[8] ? E : D));
      outc(sf("SSL Event         = %s", IntAttrs[9] ? E : D));
      outc(sf("Dead Letter Queue = %s",
              Chardata.Range(0, MQ_Q_NAME_LENGTH)));
      oute(sf("Queue Manager     = %s",
              Chardata.Range(MQ_Q_NAME_LENGTH,
                             MQ_Q_MGR_NAME_LENGTH)));
      break;
    }

    case MQOT_PROCESS:
    {
      MQLONG Selectors[] = { MQCA_ALTERATION_DATE,
                             MQCA_ALTERATION_TIME,
                             MQCA_APPL_ID,
                             MQCA_ENV_DATA,
                             MQCA_PROCESS_DESC,
                             MQCA_PROCESS_NAME,
                             MQCA_USER_DATA,
                             MQIA_APPL_TYPE
                           };

      const MQLONG ChrSize = MQ_DATE_LENGTH +
                             MQ_TIME_LENGTH +
                             MQ_TPF_APPL_ID_LENGTH +
                             MQ_PROCESS_ENV_DATA_LENGTH +
                             MQ_PROCESS_DESC_LENGTH +
                             MQ_PROCESS_NAME_LENGTH +
                             MQ_PROCESS_USER_DATA_LENGTH;

      char* data = (char*) malloc(ChrSize);

      MQINQ(Hconn, Hobj, 8, Selectors, 1, IntAttrs,
            ChrSize, data, &CompCode, &Reason);
      CheckError("MQINQ", Reason);

      String Chardata(data);

      outc(sf("Application Type    = %d", IntAttrs[0]));
      outc(sf("Last Alter Date     = %s",
           Chardata.Range(0, MQ_DATE_LENGTH)));
      outc(sf("Last Alter Time     = %s",
           Chardata.Range(MQ_DATE_LENGTH,
                          MQ_TIME_LENGTH)));
      outc(sf("Application ID      = %s",
           Chardata.Range(MQ_DATE_LENGTH +
                          MQ_TIME_LENGTH,
                          MQ_TPF_APPL_ID_LENGTH)));
      outc(sf("Environment Data    = %s",
           Chardata.Range(MQ_DATE_LENGTH +
                          MQ_TIME_LENGTH +
                          MQ_TPF_APPL_ID_LENGTH,
                          MQ_PROCESS_ENV_DATA_LENGTH)));
      outc(sf("Process Description = %s",
           Chardata.Range(MQ_DATE_LENGTH +
                          MQ_TIME_LENGTH +
                          MQ_TPF_APPL_ID_LENGTH +
                          MQ_PROCESS_ENV_DATA_LENGTH,
                          MQ_PROCESS_DESC_LENGTH)));
      outc(sf("Process Name        = %s",
           Chardata.Range(MQ_DATE_LENGTH +
                          MQ_TIME_LENGTH +
                          MQ_TPF_APPL_ID_LENGTH +
                          MQ_PROCESS_ENV_DATA_LENGTH +
                          MQ_PROCESS_DESC_LENGTH,
                          MQ_PROCESS_NAME_LENGTH)));
      oute(sf("User Data           = %s",
           Chardata.Range(MQ_DATE_LENGTH +
                          MQ_TIME_LENGTH +
                          MQ_TPF_APPL_ID_LENGTH +
                          MQ_PROCESS_ENV_DATA_LENGTH +
                          MQ_PROCESS_DESC_LENGTH +
                          MQ_PROCESS_NAME_LENGTH,
                          MQ_PROCESS_USER_DATA_LENGTH)));
      break;
    }
  } // switch (ObjectType)

  if (CompCode != MQCC_OK)
  {
    oute(sf("MQINQ completed with reason code %s",
            MQRC_String(Reason)));
  }
  else
    oute("MQINQ completed successfully");

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason);
}

void QMQZ_Set(sInput *Input)
{
  MQHCONN Hconn;
  MQHOBJ  Hobj;
  MQOD    ObjDesc = {MQOD_DEFAULT};
  MQLONG  CompCode, Reason;
  MQLONG  SelectorsSet[5], IntAttrsSet[5];
  MQLONG  CharType = 0;
  MQCHAR  *inputChar = (MQCHAR *) calloc(1, MQ_TRIGGER_DATA_LENGTH);

  MQCONN("", &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);

  if (Input[2].Param == "GET")
  {
    SelectorsSet[0] = MQIA_INHIBIT_GET;

    if (Input[3].Param == "INHIBITED")
      IntAttrsSet[0] = MQQA_GET_INHIBITED;
    else
      IntAttrsSet[0] = MQQA_GET_ALLOWED;
  }

  else if (Input[2].Param == "PUT")
  {
    SelectorsSet[0] = MQIA_INHIBIT_PUT;

    if (Input[3].Param == "INHIBITED")
      IntAttrsSet[0] = MQQA_PUT_INHIBITED;
    else
      IntAttrsSet[0] = MQQA_PUT_ALLOWED;
  }

  else if (Input[2].Param == "TRIGGER")
  {
    SelectorsSet[0] = MQIA_TRIGGER_CONTROL;

    if (Input[3].Param == "ON")
      IntAttrsSet[0] = MQTC_ON;
    else
      IntAttrsSet[0] = MQTC_OFF;
  }

  else if (Input[2].Param == "TRIGTYPE")
  {
    SelectorsSet[0] = MQIA_TRIGGER_TYPE;

    if (Input[3].Param == "NONE")
      IntAttrsSet[0] = MQTT_NONE;
    else if (Input[3].Param == "FIRST")
      IntAttrsSet[0] = MQTT_FIRST;
    else if (Input[3].Param == "EVERY")
      IntAttrsSet[0] = MQTT_EVERY;
  }

  else if (Input[2].Param == "TRIGDATA")
  {
     SelectorsSet[0] = MQCA_TRIGGER_DATA;
     CharType = 1;
     Input[3].Value.CopyTo(inputChar);
  }

  Input[1].Value.CopyTo(ObjDesc.ObjectName);
  MQOPEN(Hconn, &ObjDesc, MQOO_SET, &Hobj, &CompCode, &Reason);
  CheckError("MQOPEN", Reason);

  if (CharType == 0)
  {
     MQSET(Hconn, Hobj, 1, SelectorsSet, 1, IntAttrsSet, 0, NULL,
           &CompCode, &Reason);
  }
  else
  {
     MQSET(Hconn, Hobj, 1, SelectorsSet, 0, NULL,
           MQ_TRIGGER_DATA_LENGTH, inputChar, &CompCode, &Reason);
  }

  CheckError("MQSET", Reason);

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason);

  if (CharType == 0)
  {
    out(sf("%s is now %s %s", Input[1].Value.C(),
                              Input[2].Param.C(),
                              Input[3].Param.C()));
  }
  else
  {
    out(sf("%s is now %s %s", Input[1].Value.C(),
                              Input[2].Param.C(),
                              Input[3].Value.C()));
  }
}


void QMQZ_Display(sInput *Input)
{
  if (Input[1].Param == "TABle")
  {
    // get pointer to the MQ table
    MQP_MEM* mqt = (MQP_MEM*)cinfc_fast_ss(CINFC_CMMMQT, ecbptr()->ce1dbi);

    if (mqt == NULL) Error("MQ Table does not exist to do display");

    outc("Core Addresses for MQ Tables:");
    outc(sf("QMGR MQP_MEM Table       - %p", mqt));
    outc(sf("Queue Definition Table   - %p", mqt->pQHashTable));
    outc(sf("Channel Definition Table - %p", mqt->pCHashTable));
    outc(sf("Process Definition Table - %p", mqt->pPHashTable));
    outc(sf("Queue Trace Table        - %p", mqt->TraceTables.pTraceTableQ));
    outc(sf("Channel Trace Table      - %p", mqt->TraceTables.pTraceTableC));
    outc(sf("Communication Trace Table- %p", mqt->TraceTables.pTraceTableM));
    outc(sf("Dynamic Routing Table    - %p", mqt->pDHashTable));                //d26841
    oute(sf("MQM Memory Table         - %p", cinfc_fast(CINFC_CMMMQM)));        //d26841
  }

  else if (Input[1].Param == "MEMory")
  {
    if (Input[3].Param == "")
      PrintData((void*) HexToInt((char*)Input[2].Value.C()), 256);
    else
      PrintData((void*) HexToInt((char*)Input[2].Value.C()),
                HexToInt(Input[3].Value.C()));

    oute("End of display");
  }

  else if (Input[1].Param == "CHANnels")
  {
/*    char buffer[MQ_CHANNEL_NAME_LENGTH+1] = "";
    int rc; */

    oute("This function is being disabled");
/*  outc("Channel names in channel definition table:");

    forAllChEntries(hashKey, pEntry, rc)
    {
      memcpy(buffer,
             pEntry->Status.Status.Channel.CD.ChannelName,
             MQ_CHANNEL_NAME_LENGTH);
      outc(buffer);
    }

    oute("End of list");
*/
  }

  else
  {
    cQDEF* pQDef = NULL;
    char QName[MQ_Q_NAME_LENGTH + 1];                                           //f17629
    Input[1].Value.CopyTo(QName);                                               //f17629

    if (locateQDef(QName, &pQDef, FALSE) == 0)                                  //f17629
    {
      if (Input[2].Param == "Msg-d*")
      {
        int messageNum = Input[2].Value.ToInt();
        union { MQ_ENTRY_M* pMQEntryM;  MQ_ENTRY_64* pMQEntry64; void* pMQEntryV;};

        if (((cQLOCAL*)pQDef)->isQMQM())                                        //d26841
        {
           MQ_ENTRY_64* pPMEntry = (MQ_ENTRY_64*) ((cQLOCAL*)pQDef)->getFrontMsgListHead();
           MQ_ENTRY_64* pNPMEntry = (MQ_ENTRY_64*)((cQLOCAL*)pQDef)->getFrontNPMMsgListHead64();

           if (pPMEntry == NULL && pNPMEntry == NULL)                           //d26841
              Error("Null message pointer (Message not in front memory chain)");//d26841

           do                                                                   //d26841
           {                                                                    //d268841
              // Determine which list is the current message
              if (pPMEntry && pNPMEntry)                                        //d26841
              {                                                                 //d26841
                 if (pPMEntry->MessageID < pNPMEntry->MessageID)                //d26841
                 {                                                              //d26841
                    pMQEntry64 = pPMEntry;                                      //d26841
                    pPMEntry = pPMEntry->NextMessage_ptr;                       //d26841
                 }                                                              //d26841
                 else                                                           //d26841
                 {                                                              //d26841
                    pMQEntry64 = pNPMEntry;                                     //d26841
                    pNPMEntry = pNPMEntry->NextMessage_ptr;                     //d26841
                 }                                                              //d26841
              }                                                                 //d26841
              else if (pPMEntry)                                                //d26841
              {                                                                 //d26841
                 pMQEntry64 = pPMEntry;                                         //d26841
                 pPMEntry = pPMEntry->NextMessage_ptr;                          //d26841
              }                                                                 //d26841
              else                                                              //d26841
              {                                                                 //d26841
                 pMQEntry64 = pNPMEntry;                                        //d26841
                 pNPMEntry = pNPMEntry->NextMessage_ptr;                        //d26841
              }                                                                 //d26841

             if (pMQEntryV == NULL)                                             //d26841
                Error("QMQZ ERROR: NULL message pointer");                      //d26841
           } while (--messageNum > 0);                                          //d26841
        }                                                                       //d26841
        else                                                                    //d26841
        {                                                                       //d26841
           if ((pMQEntryV = ((cQLOCAL*)pQDef)->getFrontMsgListHead()) == NULL)  //d26841
           {                                                                    //d26841
             Error("NULL message pointer "                                      //d26841
                   "(Message not in front memory chain)");                      //d26841
           }                                                                    //d26841

           while (--messageNum > 0)                                             //d26841
           {                                                                    //d26841
             pMQEntryM = pMQEntryM->NextMessage_ptr;                            //d26841

             if (pMQEntryM == NULL)                                             //d26841
                Error("QMQZ ERROR: NULL message pointer");                      //d26841
           }                                                                    //d26841
        }                                                                       //d26841

        outc(sf("MESSAGE %d ON QUEUE %s:",                                      //d26841
                Input[2].Value.ToInt(), Input[1].Value.C()));                   //d26841

        DisplayMemoryMessage(pMQEntryV);                                        //d26841
      }                                                                         //d26841
      else out(sf("Queue %s is located at %p",
                  Input[1].Value.C(), (long) pQDef));
    }
    else out(sf("Queue definition for %s not found in core",
                Input[1].Value.C()));
  }
}


void DisplayMemoryMessage(void * pMessage)                                      //d26841
{
  char  buffer[100];
  union { MQ_ENTRY_M* pMQEntryM;  MQ_ENTRY_64* pMQEntry64; void* pMQEntryV;};   //d26841
  bool is64bit = is64bit(pMessage);                                             //d26841
  MSG_DESC_UNION* mdUnion;                                                      //d26841
  int headerLen = 0;                                                            //d26841

  pMQEntryV = pMessage;                                                         //d26841

  if (!is64bit)                                                                 //d26841
     mdUnion = &pMQEntryM->mdUnion;                                             //d26841
  else                                                                          //d26841
  {                                                                             //d26841
     mdUnion = (MSG_DESC_UNION*) &pMQEntry64->messageData[0];                   //d26841
     if (memcmp(mdUnion->StrucId, MQXQH_STRUC_ID, sizeof(mdUnion->StrucId))==0) //d26841
        headerLen = MQXQH_LENGTH_1;                                             //d26841
     else if (memcmp(mdUnion->StrucId, MQMD_STRUC_ID, sizeof(mdUnion->StrucId))==0)
     {                                                                          //d26841
        if (mdUnion->MsgDescriptorL.Version == MQMD_VERSION_1)                  //d26841
           headerLen = MQMD_LENGTH_1;                                           //d26841
        else                                                                    //d26841
           headerLen = MQMD_LENGTH_2;                                           //d26841
     }                                                                          //d26841
  }                                                                             //d26841

  sprintf(buffer,"  Message Address          - %p",
          pMessage);
  outc(buffer);

  sprintf(buffer,"  Message data pointer     - %p",
          !is64bit ? (void*) pMQEntryM->EntryCommonData.Msg_ptr :               //d26841
          (void*) pMQEntry64->Msg_ptr);                                         //d26841
  outc(buffer);

  sprintf(buffer,"  Actual Start of Data     - %p",
          !is64bit ? &pMQEntryM->messageData[0] :                               //d26841
          &pMQEntry64->messageData[0] + headerLen);                             //d26841
  outc(buffer);

  sprintf(buffer,"  Local Q Pointer          - %p",
          !is64bit ? pMQEntryM->EntryCommonData.pQLocal :                       //d26841
          pMQEntry64->pQLocal);                                                 //d26841
  outc(buffer);

  sprintf(buffer,"  Next Message Ptr         - %p",
          !is64bit ? (void *) pMQEntryM->NextMessage_ptr :                      //d26841
          (void *) pMQEntry64->NextMessage_ptr);                                //d26841
  outc(buffer);

  sprintf(buffer,"  Previous message Ptr     - %p",
          !is64bit ? (void*) pMQEntryM->PrevMessage_ptr :                       //d26841
          (void*) pMQEntry64->PrevMessage_ptr);                                 //d26841
  outc(buffer);

  sprintf(buffer,"  Next Segment of Msg      - %p",
          !is64bit ? (void*) pMQEntryM->messageHeader.NextSegmentOfMsg :        //d26841
          (void*) pMQEntry64->messageHeader.NextSegmentOfMsg);                  //d26841
  outc(buffer);

  sprintf(buffer,"  Size of this segment     - %d",
          !is64bit ? pMQEntryM->messageHeader.SizeofThisSegment :               //d26841
          pMQEntry64->messageHeader.SizeofThisSegment);                         //d26841
  outc(buffer);

  sprintf(buffer,"  Total Message Size       - %d",
          !is64bit ? pMQEntryM->EntryCommonData.totalMsgSize :                  //d26841
          pMQEntry64->totalMsgSize - headerLen);                                //d26841
  outc(buffer);

  sprintf(buffer,"  QName                    - %.48s",
          !is64bit ? pMQEntryM->EntryCommonData.QName :                         //d26841
          pMQEntry64->QName);                                                   //d26841
  outc(buffer);

  sprintf(buffer,"  Non-persistence          - %d",
          !is64bit ? pMQEntryM->EntryCommonData.NPM :                           //d26841
          pMQEntry64->NPM);                                                     //d26841
  outc(buffer);

  sprintf(buffer,"  Uncommitted              - %d",
          !is64bit ? pMQEntryM->EntryCommonData.Uncommitted :                   //d26841
          pMQEntry64->Uncommitted);                                             //d26841
  outc(buffer);

  sprintf(buffer,"  Gotten                   - %d",
          !is64bit ? pMQEntryM->EntryCommonData.Gotten :                        //d26841
          pMQEntry64->Gotten);                                                  //d26841
  outc(buffer);

  sprintf(buffer,"  Rolled Back              - %d",
          !is64bit ? pMQEntryM->EntryCommonData.Rollback :                      //d26841
          pMQEntry64->Rollback);                                                //d26841
  outc(buffer);

  if (!is64bit)                                                                 //d26841
  {                                                                             //d26841
     sprintf(buffer,"  Rollback pending         - %d",
             pMQEntryM->RollbackPending);                                       //d26841
     outc(buffer);
  }                                                                             //d26841

  sprintf(buffer,"  Batch Number             - %d",
          !is64bit ? pMQEntryM->EntryCommonData.Batchnumber :                   //d26841
          pMQEntry64->Batchnumber);                                             //d26841
  outc(buffer);

/*sprintf(buffer,"  Structure ID             - %4s",
          pMessage->EntryCommonData.structID);
  outc(buffer);

  sprintf(buffer,"  Instance ID              - %d",
          pMessage->EntryCommonData.InstanceID);
  outc(buffer);
*/
  sprintf(buffer,"  Internal Message ID      - %lX",                            //d26841
          !is64bit ? pMQEntryM->EntryCommonData.MessageID :                     //d26841
          pMQEntry64->MessageID);                                               //d26841
  outc(buffer);
/*
  sprintf(buffer,"  Root TID                 - %.8X %.8X %.8X",
          pMessage->EntryCommonData.RootTID);
  outc(buffer);

  sprintf(buffer,"  Record Version           - %d",
          pMessage->Version);
  outc(buffer);

  sprintf(buffer,"  Sort Key Priority        - %d",
          pMessage->SortKey.Priority);
  outc(buffer);

  sprintf(buffer,"  Sort Key Time Stamp      - %d",
          pMessage->SortKey.TimeStamp);
  outc(buffer);
*/

  if (memcmp(mdUnion->StrucId, MQXQH_STRUC_ID, sizeof(mdUnion->StrucId))!=0 &&  //d26841
      memcmp(mdUnion->StrucId, MQMD_STRUC_ID, sizeof(mdUnion->StrucId))!=0)     //d26841
  {
    sprintf(buffer,"  MD Union ID              - %.4s",
            mdUnion->StrucId);                                                  //d26841
    outc(buffer);
  }
  else if (memcmp(mdUnion->StrucId, MQXQH_STRUC_ID, sizeof(mdUnion->StrucId))==0)
  {
    sprintf(buffer,"  XQH Destination Queue    - %.48s",
            mdUnion->MsgDescriptorR.RemoteQName);                               //d26841
    outc(buffer);

    sprintf(buffer,"  XQH Destination QMGR     - %.48s",
            mdUnion->MsgDescriptorR.RemoteQMgrName);                            //d26841
    outc(buffer);

    sprintf(buffer,"  MD1 Persistence          - %d",
            mdUnion->MsgDescriptorR.MsgDesc.Persistence);                       //d26841
    outc(buffer);

    sprintf(buffer,"  MD1 Character Set ID     - %d",
            mdUnion->MsgDescriptorR.MsgDesc.CodedCharSetId);                    //d26841
    outc(buffer);

    sprintf(buffer,"  MD1 Expiry               - %d",                           //d26841
            mdUnion->MsgDescriptorR.MsgDesc.Expiry);                            //d26841
    outc(buffer);                                                               //d26841

  }
  else if (memcmp(mdUnion->StrucId, MQMD_STRUC_ID, sizeof(mdUnion->StrucId))==0)//d26841
  {
    sprintf(buffer, "  MD Persistence           - %d",
            mdUnion->MsgDescriptorL.Persistence);                               //d26841
    outc(buffer);

    sprintf(buffer, "  MD Character Set ID      - %d",
            mdUnion->MsgDescriptorL.CodedCharSetId);                            //d26841
    outc(buffer);

    sprintf(buffer,"  MD Expiry                - %d",                           //d26841
            mdUnion->MsgDescriptorL.Expiry);                                    //d26841
    outc(buffer);                                                               //d26841

    sprintf(buffer, "  MsgID    - %24.24s",
            mdUnion->MsgDescriptorL.MsgId);                                     //d26841
    outc(buffer);

    sprintf(buffer, "  CorrelID - %24.24s",
            mdUnion->MsgDescriptorL.CorrelId);                                  //d26841
    outc(buffer);
  }

  outc(sf("Message data in this segment (%d bytes):",
       !is64bit ? pMQEntryM->messageHeader.SizeofThisSegment :                  //d26841
       pMQEntry64->messageHeader.SizeofThisSegment-headerLen));                 //d26841
  PrintData(!is64bit ? &pMQEntryM->messageData[0] :                             //d26841
       &pMQEntry64->messageData[0] + headerLen,                                 //d26841
       !is64bit ? pMQEntryM->messageHeader.SizeofThisSegment :                  //d26841
       MIN(pMQEntry64->messageHeader.SizeofThisSegment-headerLen,150),          //d26841
       !is64bit ? true : false);                                                //d26841
       /* max 150 bytes */                                                      //d26841
  oute("End of display");
}


void PrintData(void* pData, int bytesToPrint, boolean ShowAddr)
{
  char* data = (char*) pData;
  char buffer[100] = "";
  char zeros[16] = "";
  int zerolines = 0;
  int returnlength = 0;

  if (data == NULL)
    Error("PrintData: NULL Data pointer");

  while (bytesToPrint >= 16)
  {
    for (zerolines = 0; bytesToPrint > 31; zerolines++)
    {
      if (memcmp(data, zeros, 16))
        break;

      data += 16;
      bytesToPrint -= 16;
    }

    if (zerolines > 0)
    {
      outc(sf("            (%d lines (%d bytes) of zeros not displayed)",
              zerolines, zerolines * 16));
    }

    returnlength = sprintf(buffer,
                           ShowAddr ? "  %p- " : "            ",
                           data);

    returnlength += sprintf(buffer + returnlength,
                            "%.8X %.8X %.8X %.8X  "
                            "%c%c%c%c%c%c%c%c %c%c%c%c%c%c%c%c",
                            ((int*) data)[0], ((int*) data)[1],
                            ((int*) data)[2], ((int*) data)[3],
                            data[0],  data[1],  data[2], data[3],
                            data[4],  data[5],  data[6], data[7],
                            data[8],  data[9],  data[10], data[11],
                            data[12], data[13], data[14], data[15]);

    for (int i = 0; i < returnlength; i++)
    {
      if (!OKChar(buffer[i]))
        buffer[i] = '.';
    }

    outc(buffer);
    data += 16;
    bytesToPrint -= 16;
  }

  char temp[16] = {0};

  if (bytesToPrint > 0)
  {
    returnlength = 0;
    memcpy(temp, data, bytesToPrint);

    returnlength = sprintf(buffer,
                           ShowAddr ? "  %p- " : "            ",
                           data);

    returnlength += sprintf(buffer + returnlength,
                            "%.8X %.8X %.8X %.8X  "
                            "%c%c%c%c%c%c%c%c %c%c%c%c%c%c%c%c",
                            ((int*) temp)[0], ((int*) temp)[1],
                            ((int*) temp)[2], ((int*) temp)[3],
                            temp[0],  temp[1],  temp[2], temp[3],
                            temp[4],  temp[5],  temp[6], temp[7],
                            temp[8],  temp[9],  temp[10], temp[11],
                            temp[12], temp[13], temp[14], temp[15]);

    memset(buffer + 12 + 2 * bytesToPrint + bytesToPrint / 4, ' ',
           2 * (16 - bytesToPrint) + (16 - bytesToPrint) / 4);

    if (bytesToPrint <= 8)
    {
      memset(buffer + 49 + bytesToPrint, ' ', 16 - bytesToPrint);
      memset(buffer + 58, ' ', 8);
    }
    else
      memset(buffer + 50 + bytesToPrint, ' ', 16 - bytesToPrint);

    for (int i = 0; i < returnlength; i++)
    {
      if (!OKChar(buffer[i]))
        buffer[i] = '.';
    }

    outc(buffer);
  }
  else
    outc("");
}


int OKChar(char inchar)     // see if it is a normal typed character, in EBCDIC
{
  if ((inchar ==  64) ||
      (inchar >=  74 && inchar <=  80) ||
      (inchar >=  90 && inchar <=  97) ||
      (inchar >= 107 && inchar <= 111) ||
      (inchar >= 121 && inchar <= 127) ||
      (inchar >= 129 && inchar <= 137) ||
      (inchar >= 145 && inchar <= 153) ||
      (inchar >= 161 && inchar <= 169) ||
      (inchar >= 186 && inchar <= 187) ||
      (inchar >= 192 && inchar <= 201) ||
      (inchar >= 192 && inchar <= 201) ||
      (inchar >= 208 && inchar <= 217) ||
      (inchar >= 224 && inchar <= 233) ||
      (inchar >= 240 && inchar <= 249))
  {
    return 1;
  }
  else
    return 0;
}


void QMQZ_ZMQSC(sInput *Input)
{
  int i = 0, hi, low;
  char msg[100] = "";

  while (Input[i].Param != "Low-d*" &&
         Input[i].Param != "")
    i++;

  low = Input[i].Value.ToInt();
  hi = Input[i + 1].Value.ToInt();

  Input[i].Value = "";
  Input[i+1].Value = "";

  for (int j = 0; j < i; j++)
    Input[j].Value.Convert('_', '-');

  // ZMQSC DEF QR-qrname RQMNAME-rqmname RNAME-rname XMITQ-xmitq
  if ((Input[3].Value[1] == 'r') || (Input[3].Value[1] == 'R'))
  {
    for (i = low; i <= hi; i++)
    {
      sprintf(msg, "%s %s %s%03d %s%03d %s %s%03d %s %s",
              Input[0].Value.C(), Input[1].Value.C(),
              Input[2].Value.C(), i,
              Input[3].Value.C(), i,
              Input[4].Value.C(), Input[5].Value.C(), i,
              Input[6].Value.C(), Input[7].Value.C());

      CreateECBs(1, msg);
    }
  }
  // ZMQSC DEF CHL-chlname CHLTYPE-chltype CONNAME-conname XMITQ-xmitq
  else if ((Input[5].Value[0] == 'x') || (Input[5].Value[0] == 'X'))
  {
    for (i = low; i <= hi; i++)
    {
      sprintf(msg, "%s %s %s%03d %s %s %s%03d %s %s",
              Input[0].Value.C(), Input[1].Value.C(),
              Input[2].Value.C(), i, Input[3].Value.C(),
              Input[4].Value.C(), Input[5].Value.C(), i,
              Input[6].Value.C(), Input[7].Value.C());
      CreateECBs(1, msg);
    }
  }
  else
  {
    for (i = low; i <= hi; i++)
    {
      sprintf(msg, "%s %s %s%03d %s %s %s %s %s",
              Input[0].Value.C(), Input[1].Value.C(),
              Input[2].Value.C(), i, Input[3].Value.C(),
              Input[4].Value.C(), Input[5].Value.C(),
              Input[6].Value.C(), Input[7].Value.C());
      CreateECBs(1, msg);
    }
  }
}


void CheckRiplPoint(int point, int numMsgs)
{
  if (point <= 0)
    Error("RIPL Point must be positive");
  else if (point > numMsgs)
    Error("RIPL Point must be < or = to the number of messages");
}


void CheckRate(int commRate, int numMsgs)
{
  if (commRate > numMsgs)
  {
    Error("Commit/Rollback rate cannot be greater"
          " than number of messages");
  }
  else if (commRate < 1)
    Error("Commit/Rollback rate must be positive");
}


void CheckNumMsgs(int nummsgs)
{
  if ((nummsgs <= 0) || (nummsgs >= MAXMSG_COUNT))
  {
    Error(sf("Number of messages must be between 1 and %d",
             MAXMSG_COUNT));
  }
}


void CheckMsgLength(MQLONG Length)
{
  if (Length < 0 || Length > MAXMSG_LENGTH)
    Error("QMQZ01: Invalid Message Length");
}


void CheckGmoVersion(int ver)                                                   //d26214
{                                                                               //d26214
  if ((ver <= 0) || (ver > MAX_GMO_VERSION))                                    //d26214
  {                                                                             //d26214
    Error(sf("GMO Version must be between 1 and %d", MAX_GMO_VERSION));         //d26214
  }                                                                             //d26214
}


char* MessagetoPut(char *QMsg, int i, int msgLength)
{
  char header[25];

  // Add integer value to the message
  if (i > 0)
     sprintf(header, MSG_DATA_PREFIX "%d.<", i);
  else
     strcpy(header, MSG_DATA_PREFIX);

  for (int j = 0; j < msgLength && header[j] != '\0'; j++)
    QMsg[j] = header[j];

  return QMsg;
}


void CheckNumEcbs(int numEcbs)
{
  if (numEcbs <= 0)
    Error("Number of ECBs must be positive");
  else if (numEcbs > MAXECB)
    Error(sf("Number of ECBs must be less than MAXECB = %d",MAXECB));
}


void Ripl()
{
  out("*** QMQZ IPL Initiated ***");
//sleep(100);                              // time for a hard IPL
  while (1) out("Systems going down...");  // catastrophic
  CreateECBs(1, "ZRIPL BP");               // regular IPL
  out("IPL skipped for placement testing, exiting ECB...");
  exit(0);
}


void CreateECBs(int ecb_count, char *zmsg, boolean wait)
                                                 // create ecb_count ECBs
{
  mi0mi* block_ptr;                              // pointer to core block
  swisc_create_input swiscParameters;            // parameters for swisc_create
  struct tpf_cresc_input tci={0};                    // if cresc is used

  swiscParameters.istream = SWISC_IS_BALANCE;    // Goes to least busy I-Stream
  swiscParameters.cpu_list = SWISC_LIST_READY;   // Launch Immediately
  swiscParameters.bypass = SWISC_BYPASS_NO;      // Use available I-Streams
  swiscParameters.program = "CVAA";              // Send to console interperter
  swiscParameters.ebw_data = NULL;
  swiscParameters.ebw_data_length = 0;
  swiscParameters.data_level = D2;               // passing a core block

  for (int i = 1; i <= ecb_count; i++)           // make ecb_count ECBs
  {
    if (wait)
    {
        tci.program = "QMQZ";
        tci.istream = TPF_CRESC_IS_SAME;
        tci.timeout = 30;
        tci.wait    = TPF_CRESC_WAIT_YES;
        tci.data_length = sizeof(NOPARSE);
        tci.data = zmsg;
        free(tpf_cresc(&tci));
    }
    else
    {
        // get a core block for D2 to pass to new ECB...
        block_ptr = (struct mi0mi *) getcc(D2, GETCC_TYPE, L1);
        block_ptr->mi0cct = strlen(zmsg) + 4;        // set message length
        block_ptr->mi0adr = ecbptr()->ebrout;
        strcpy(block_ptr->mi0acc, zmsg);             // put functional message on the core block
        swisc_create(&swiscParameters);              // Launch new ECB
    }
  }
}


long HexToInt(char* datain)
{
  int mult = 1;
  long toreturn = 0;
  int index = strlen(datain);

  while (--index >= 0)
  {
    switch (datain[index])
    {
      case '0':                                    break;
      case '1':            toreturn += mult * 1;   break;
      case '2':            toreturn += mult * 2;   break;
      case '3':            toreturn += mult * 3;   break;
      case '4':            toreturn += mult * 4;   break;
      case '5':            toreturn += mult * 5;   break;
      case '6':            toreturn += mult * 6;   break;
      case '7':            toreturn += mult * 7;   break;
      case '8':            toreturn += mult * 8;   break;
      case '9':            toreturn += mult * 9;   break;
      case 'a': case 'A':  toreturn += mult * 10;  break;
      case 'b': case 'B':  toreturn += mult * 11;  break;
      case 'c': case 'C':  toreturn += mult * 12;  break;
      case 'd': case 'D':  toreturn += mult * 13;  break;
      case 'e': case 'E':  toreturn += mult * 14;  break;
      case 'f': case 'F':  toreturn += mult * 15;  break;
      default : Error("HexToInt: Bad Input");
    }

    mult *= 16;
  }

  return toreturn;
}

/**********************************************************************/
/* static functions                                                   */
/**********************************************************************/

static void checkMsgSeq(char* Buffer)
{
  static int  MsgNbr = 0;
  static BOOL ShortMsgWarning = FALSE;
  String Message(Buffer);

  char data[5];

  int ThisMsgNbr;

  // The sequence can't be checked if the message is too short.
  // Look for a '<' to ensure that it's long enough.  Put adds
  // that character after the sequence number.

  if (strchr(Message.C(), '<') == NULL)
  {
    if (!ShortMsgWarning)
    {
      out("Message too short for sequence check.");
      ShortMsgWarning = TRUE;
    }

    return;
  }

  sscanf(Message.C(), "%4c" MSG_SEQ_PFX "%d", data, &ThisMsgNbr);

  // If this is the first message, just save the number.
  // Otherwise, check its number against the previous one.

  if (MsgNbr == 0)
    MsgNbr = ThisMsgNbr;
  else
  {
    if (ThisMsgNbr != ++MsgNbr)
    {
      out(sf("message sequence error - expecting %d, got %d",
             MsgNbr, ThisMsgNbr));
      MsgNbr = ThisMsgNbr;
    }
  }

  return;
}
