/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2018                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  d26352  20180810  Compiler compatibility updates                 */
/*                                                                   */
/*********************************************************************/
/*  The following subroutines are used to regression test how the MQ */
/*  API calls handle certain error conditions.  This is useful to    */
/*  test the error path/handling of the APIs themselves.             */
/*********************************************************************/

#include <qmqz.h>

void QMQZ_ErrorTest(sInput *Input) {                                 // Jump to appropriate function
  MQCHAR QName[MQ_Q_NAME_LENGTH];                                    // Queue Name
  MQCHAR QMgrName[MQ_Q_NAME_LENGTH];                                 // Queue Manager Name
  int    CaseNum = 0;
  int    i = 0;

  static sErrorTable ErrorTable[] = {
    { "MQCONN"   , error_test_conn    },
    { "MQDISC"   , error_test_disc    },
    { "MQOPEN"   , error_test_open    },
    { "MQCLOSE"  , error_test_close   },
    { "MQPUT"    , error_test_put     },
    { "MQGET"    , error_test_get     },
    { "MQPUT1"   , error_test_put1    },
    { "MQINQ"    , error_test_inq     },
    { "MQSET"    , error_test_set     } };

  Input[2].Value.CopyTo(QMgrName);
  Input[3].Value.CopyTo(QName);
  if (Input[4].Param == "d*") {                                      // specific case to run
    CaseNum = Input[4].Value.ToInt();
    if (CaseNum <= 0) Error("Test case not valid"); }

  while (Input[1].Param != ErrorTable[i].Command) i++;               // search table

  ErrorTable[i].Function(QMgrName, QName, CaseNum);                  // Call function
  oute("End of error testing"); }



void PrintCodes(char *api, MQLONG Reason, MQLONG expected) {
  Trace(api);
  if (expected == Reason)
    outc(sf("  Success:  %s Reason %d, %s",
            api, Reason, MQRC_String(Reason)));
  else {
    outc(sf("  Error:    %s Reason %d, %s",
         api, Reason, MQRC_String(Reason)));
    outc(sf("  Expected: %s Reason %d, %s",
         api, expected, MQRC_String(expected))); } }




void error_test_conn(char *QMgr, char *QName, unsigned int CaseNum) {
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int     i;                      // loop counter

  static sETable ETable[] = {
    {"NULL Hconn pointer"            , MQRC_HCONN_ERROR             },    // 1
    {"NULL Name pointer"             , MQRC_Q_MGR_NAME_ERROR        } };  // 2

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQCONN");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_conn(QMgr, QName, i);
    return; }

  outc(sf("MQCONN Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  switch (CaseNum) {
    case 1:  MQCONN(QMgr, NULL, &CompCode, &Reason);         break;
    case 2:  MQCONN(NULL, &Hconn, &CompCode, &Reason);       }

  PrintCodes("MQCONN", Reason, ETable[CaseNum-1].EReason);
}

void error_test_disc(char *QMgr, char *QName, unsigned int CaseNum) {
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQHCONN BadHconn = MQHC_UNUSABLE_HCONN;  // Connection Handle
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int i;

  static sETable ETable[] = {
    {"NULL Hconn pointer"            , MQRC_HCONN_ERROR             },    // 1
    {"Bad Hconn"                     , MQRC_HCONN_ERROR             } };  // 2

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQDISC");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_disc(QMgr, QName, i);
    return; }

  outc(sf("MQDISC Case %d: %s", CaseNum, ETable[CaseNum-1].Text));
  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);

  switch (CaseNum) {
    case 1:  MQDISC(NULL, &CompCode, &Reason);            break;
    case 2:  MQDISC(&BadHconn, &CompCode, &Reason);       }

  PrintCodes("MQDISC", Reason, ETable[CaseNum-1].EReason); }




void error_test_open(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQHCONN BadHconn = MQHC_UNUSABLE_HCONN;  // Connection Handle
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  O_options = MQOO_OUTPUT;         // MQOPEN options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int i;

  static sETable ETable[] = {
    {"Bad Hconn"                   , MQRC_HCONN_ERROR               },    // 1
    {"NULL object descriptor"      , MQRC_OD_ERROR                  },    // 2
    {"Unsupported object type"     , MQRC_OBJECT_TYPE_ERROR         },    // 3
    {"Unsupported open option"     , MQRC_OPTIONS_ERROR             },    // 4
    {"QM with an object name"      , MQRC_NAME_NOT_VALID_FOR_TYPE   },    // 5
    {"Remote queue for input"      , MQRC_OPTION_NOT_VALID_FOR_TYPE },    // 6
    {"QM object without a QM name" , MQRC_UNKNOWN_OBJECT_Q_MGR      },    // 7
    {"Inactive queue manager"      , MQRC_Q_MGR_NOT_ACTIVE          },    // 8
    {"Undefined queue manager"     , MQRC_Q_MGR_NOT_AVAILABLE       } };  // 9

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQOPEN");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_open(QMgr, QName, i);
    return; }

  outc(sf("MQOPEN Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  switch (CaseNum) {
    case 8:  outc("Queue manager must not be active");
             MQOPEN(4,&od,O_options,&Hobj,&CompCode,&Reason);
             PrintCodes("MQOPEN", Reason, ETable[CaseNum-1].EReason);
             return;
    case 9:  outc("Queue manager must not be defined");
             MQOPEN(4,&od,O_options,&Hobj,&CompCode,&Reason);
             PrintCodes("MQOPEN", Reason, ETable[CaseNum-1].EReason);
             return; }

  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);

  switch (CaseNum) {
    case 1:  MQOPEN(BadHconn,&od,O_options,&Hobj,&CompCode,&Reason);
             break;
    case 2:  MQOPEN(Hconn,NULL,O_options,&Hobj,&CompCode,&Reason);
             break;
    case 3:  od.ObjectType = MQOT_PROCESS;
             MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
             break;
    case 4:  MQOPEN(Hconn,&od,MQOO_BROWSE,&Hobj,&CompCode,&Reason);
             break;
    case 5:  od.ObjectType = MQOT_Q_MGR;
             MQOPEN(Hconn,&od,MQOO_INQUIRE,&Hobj,&CompCode,&Reason);
             break;
    case 6:  outc("Remote Queue RQ1 must be defined");
             strncpy(od.ObjectName, "RQ1", MQ_Q_NAME_LENGTH);
             MQOPEN(Hconn,&od,MQOO_INPUT_AS_Q_DEF,
                    &Hobj,&CompCode,&Reason);
             break;
    case 7:  od.ObjectType = MQOT_Q_MGR;
             od.ObjectName[0] = '\0';
             strcpy(od.ObjectQMgrName, "Bad Manager Name");
             MQOPEN(Hconn,&od,MQOO_INQUIRE,&Hobj,&CompCode,&Reason);
             break; }

  PrintCodes("MQOPEN", Reason, ETable[CaseNum-1].EReason);

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }



void error_test_close(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  O_options = MQOO_OUTPUT;         // MQOPEN options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int     i;                      // loop counter

  static sETable ETable[] = {
    {"NULL Hobj"                     , MQRC_HOBJ_ERROR              },    // 1
    {"Bad Hconn"                     , MQRC_HCONN_ERROR             },    // 2
    {"Inactive queue manager"        , MQRC_Q_MGR_NOT_ACTIVE        },    // 3
    {"No queue manager defined"      , MQRC_Q_MGR_NOT_AVAILABLE     } };  // 4

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQCLOSE");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_close(QMgr, QName, i);
    return; }

  outc(sf("MQCLOSE Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  switch (CaseNum) {
    case 1:  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
             CheckError("MQCONN", Reason);
             MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
             CheckError("MQOPEN", Reason);
             MQCLOSE(Hconn, NULL, 0, &CompCode, &Reason);         break;
    case 2:  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
             CheckError("MQCONN", Reason);
             MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
             CheckError("MQOPEN", Reason);
             MQCLOSE(-1, &Hobj, 0, &CompCode, &Reason);           break;
    case 3:  outc("Queue manager must not be active");
             MQCLOSE(4, &Hobj, 0, &CompCode, &Reason);
             PrintCodes("MQCLOSE", Reason, ETable[CaseNum-1].EReason);
             return;
    case 4:  outc("Queue manager must not be defined");
             MQCLOSE(4, &Hobj, 0, &CompCode, &Reason);
             PrintCodes("MQCLOSE", Reason, ETable[CaseNum-1].EReason);
             return; }

  PrintCodes("MQCLOSE", Reason, ETable[CaseNum-1].EReason);

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }


void error_test_put(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQMD    md = {MQMD_DEFAULT};             // Message Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQPMO   pmo = {MQPMO_DEFAULT};           // Put Msg Options
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int i;
  char msgBuffer[] = "MQ Message";

  static sETable ETable[] = {
    {"Negative BufferLength"            , MQRC_BUFFER_LENGTH_ERROR  },    // 1
    {"NULL message"                     , MQRC_BUFFER_ERROR         },    // 2
    {"Invalid Hconn"                    , MQRC_HCONN_ERROR          },    // 3
    {"MQHO_UNUSABLE_HOBJ Hobj"          , MQRC_HOBJ_ERROR           },    // 4
    {"NULL MsgDesc"                     , MQRC_MD_ERROR             },    // 5
    {"Request message with no ReplyToQ" , MQRC_MISSING_REPLY_TO_Q   },    // 6
    {"Queue only opened for input"      , MQRC_NOT_OPEN_FOR_OUTPUT  },    // 7
    {"Unsupported PMO"                  , MQRC_OPTIONS_ERROR        },    // 8
    {"Unsupported persistence"          , MQRC_PERSISTENCE_ERROR    },    // 9
    {"NULL PutMsgOptions"               , MQRC_PMO_ERROR            },    // 10
    {"Inactive queue manager"           , MQRC_Q_MGR_NOT_ACTIVE     },    // 11
    {"No queue manager defined"         , MQRC_Q_MGR_NOT_AVAILABLE  } };  // 12

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQPUT");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_put(QMgr, QName, i);
    return; }

  outc(sf("MQPUT Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  switch (CaseNum) {
    case 7:  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
             CheckError("MQCONN", Reason);
             MQOPEN(Hconn, &od, MQOO_INPUT_AS_Q_DEF,
                    &Hobj, &CompCode, &Reason);
             CheckError("MQOPEN", Reason);
             MQPUT(Hconn, Hobj, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);
             PrintCodes("MQPUT", Reason, ETable[CaseNum-1].EReason);
             return;
    case 12: outc("Queue manager must not be active");
             MQPUT(4,0,&md,&pmo,11,                                             //d26352
                   msgBuffer, &CompCode, &Reason);
             PrintCodes("MQPUT", Reason, ETable[CaseNum-1].EReason);
             return;
    case 13: outc("Queue manager must not be defined");
             MQPUT(4,0,&md,&pmo,11,                                             //d26352
                   msgBuffer, &CompCode, &Reason);
             PrintCodes("MQPUT", Reason, ETable[CaseNum-1].EReason);
             return; }

  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);
  MQOPEN(Hconn, &od, MQOO_OUTPUT, &Hobj, &CompCode, &Reason);
  CheckError("MQOPEN", Reason);

  switch (CaseNum) {
    case 1:  MQPUT(Hconn, Hobj, &md, &pmo, -1,
                   msgBuffer, &CompCode, &Reason);    break;
    case 2:  MQPUT(Hconn, Hobj, &md, &pmo, 10,
                   NULL, &CompCode, &Reason);            break;
    case 3:  MQPUT(-1, Hobj, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 4:  MQPUT(Hconn, MQHO_UNUSABLE_HOBJ, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 5:  MQPUT(Hconn, Hobj, NULL, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 6:  md.MsgType = MQMT_REQUEST;
             MQPUT(Hconn, Hobj, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 8:  pmo.Options = 9999;
             MQPUT(Hconn, Hobj, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 9:  md.Persistence = MQRC_PERSISTENCE_ERROR;
             MQPUT(Hconn, Hobj, &md, &pmo, 11,
                   msgBuffer, &CompCode, &Reason);    break;
    case 10: MQPUT(Hconn, Hobj, &md, NULL, 11,
                   msgBuffer, &CompCode, &Reason);    }

  PrintCodes("MQPUT", Reason, ETable[CaseNum-1].EReason);

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);
  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }



void error_test_get(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQMD    md = {MQMD_DEFAULT};             // Message Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQGMO   gmo = {MQGMO_DEFAULT};           // Get Msg Options
  gmo.Version  = MQGMO_VERSION_2;
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  O_options;                       // MQOPEN options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  MQLONG  msgLen;                          // message length
  char    buffer[20];                      // get buffer
  unsigned int i;

  static sETable ETable[] = {
    {"Negative BufferLength"            , MQRC_BUFFER_LENGTH_ERROR  },    // 1
    {"NULL message"                     , MQRC_BUFFER_ERROR         },    // 2
    {"Invalid Hconn"                    , MQRC_HCONN_ERROR          },    // 3
    {"MQHO_UNUSABLE_HOBJ Hobj"          , MQRC_HOBJ_ERROR           },    // 4
    {"NULL MsgDesc"                     , MQRC_MD_ERROR             },    // 5
    {"NULL DataLength"                  , MQRC_DATA_LENGTH_ERROR    },    // 6
    {"Queue only opened for output"     , MQRC_NOT_OPEN_FOR_INPUT   },    // 7
    {"Unsupported options"              , MQRC_OPTIONS_ERROR        },    // 8
    {"NULL GetMsgOpts"                  , MQRC_GMO_ERROR            },    // 9
    {"Inactive queue manager"           , MQRC_Q_MGR_NOT_ACTIVE     },    // 10
    {"No queue manager defined"         , MQRC_Q_MGR_NOT_AVAILABLE  },    // 11
    {"Not accepting truncated messages" , MQRC_TRUNCATED_MSG_FAILED } };  // 12

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQGET");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_get(QMgr, QName, i);
    return; }

  outc(sf("MQGET Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  switch (CaseNum) {
    case 7:  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
             CheckError("MQCONN", Reason);
             MQOPEN(Hconn, &od, MQOO_OUTPUT, &Hobj, &CompCode, &Reason);
             CheckError("MQOPEN", Reason);
             MQGET(Hconn, Hobj, &md, &gmo, 20,
                   buffer, &msgLen, &CompCode, &Reason);
             PrintCodes("MQGET", Reason, ETable[CaseNum-1].EReason);
             return;
    case 10: outc("Queue manager must not be active");
             MQGET(4, 0, &md, &gmo, 20,                                         //d26352
                   buffer, &msgLen, &CompCode, &Reason);
             PrintCodes("MQGET", Reason, ETable[CaseNum-1].EReason);
             return;
    case 11: outc("Queue manager must not be defined");
             MQGET(4, 0, &md, &gmo, 20,                                         //d26352
                   buffer, &msgLen, &CompCode, &Reason);
             PrintCodes("MQGET", Reason, ETable[CaseNum-1].EReason);
             return; }

  O_options = MQOO_INPUT_AS_Q_DEF + MQOO_OUTPUT;
  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);
  MQOPEN(Hconn, &od, O_options, &Hobj, &CompCode, &Reason);
  CheckError("MQOPEN", Reason);

  switch (CaseNum) {
    case 1:  MQGET(Hconn, Hobj, &md, &gmo, -1,
                   buffer, &msgLen, &CompCode, &Reason);  break;
    case 2:  MQGET(Hconn, Hobj, &md, &gmo, 20,
                   NULL, &msgLen, &CompCode, &Reason);    break;
    case 3:  MQGET(-1, Hobj, &md, &gmo, 20,
                   buffer, &msgLen ,&CompCode ,&Reason);  break;
    case 4:  MQGET(Hconn, MQHO_UNUSABLE_HOBJ, &md, &gmo, 20,
                   buffer, &msgLen, &CompCode, &Reason);  break;
    case 5:  MQGET(Hconn, Hobj, NULL, &gmo, 20,
                   buffer, &msgLen, &CompCode, &Reason);  break;
    case 6:  MQGET(Hconn, Hobj, &md, &gmo, 20,
                   buffer, NULL, &CompCode, &Reason);  break;
    case 8:  gmo.Options = MQGMO_WAIT;
             MQGET(Hconn, Hobj, &md, &gmo, 20,
                   buffer, &msgLen, &CompCode, &Reason);  break;
    case 9:  MQGET(Hconn, Hobj, &md, NULL, 20,
                   buffer, &msgLen, &CompCode, &Reason);  break;
    case 12: outc("There must be a message on the queue, with "\
                  "datalength > 1");
             MQGET(Hconn, Hobj, &md, &gmo, 1,
                   buffer, &msgLen, &CompCode, &Reason);  }

  PrintCodes("MQGET", Reason, ETable[CaseNum-1].EReason);

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);
  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }



void error_test_put1(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQMD    md = {MQMD_DEFAULT};             // Message Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQPMO   pmo = {MQPMO_DEFAULT};           // Put Msg Options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  unsigned int i;
  char msgBuffer[] = "MQ Message";

  static sETable ETable[] = {
    {"Negative Buffer Length"        , MQRC_BUFFER_LENGTH_ERROR     },    // 1
    {"NULL Buffer"                   , MQRC_BUFFER_ERROR            },    // 2
    {"invalid Hconn"                 , MQRC_HCONN_ERROR             },    // 3
    {"NULL MsgDesc"                  , MQRC_MD_ERROR                },    // 4
    {"request without reply to queue", MQRC_MISSING_REPLY_TO_Q      },    // 5
    {"unsupported options"           , MQRC_OPTIONS_ERROR           },    // 6
    {"unsupported persistence"       , MQRC_PERSISTENCE_ERROR       },    // 7
    {"NULL PutMsgOpts"               , MQRC_PMO_ERROR               },    // 8
    {"Inactive Queue Manager"        , MQRC_Q_MGR_NOT_ACTIVE        },    // 9
    {"undefined Queue Manager"       , MQRC_Q_MGR_NOT_AVAILABLE     },    // 10
    {"NULL Object Descriptor"        , MQRC_OD_ERROR                },    // 11
    {"unsupported type of object"    , MQRC_OBJECT_TYPE_ERROR       },    // 12
    {"unsupported option"            , MQRC_OPTIONS_ERROR           } };  // 13

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQPUT1");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_put1(QMgr, QName, i);
    return; }

  outc(sf("MQPUT1 Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  switch (CaseNum) {
    case 9:  outc("Queue namager must not be active");
             MQPUT1(4, &od, &md, &pmo, -1, msgBuffer,
                    &CompCode, &Reason);
             PrintCodes("MQPUT1", Reason, ETable[CaseNum-1].EReason);
             return;
    case 10: outc("Queue namager must not be defined");
             MQPUT1(4, &od, &md, &pmo, -1, msgBuffer,
                    &CompCode, &Reason);
             PrintCodes("MQPUT1", Reason, ETable[CaseNum-1].EReason);
             return; }

  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);
  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  switch (CaseNum) {
    case 1:  MQPUT1(Hconn, &od, &md, &pmo, -1, msgBuffer,
                    &CompCode, &Reason);           break;
    case 2:  MQPUT1(Hconn, &od, &md, &pmo, 11, NULL,
                    &CompCode, &Reason);           break;
    case 3:  MQPUT1(-1, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 4:  MQPUT1(Hconn, &od, NULL, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 5:  md.MsgType = MQMT_REQUEST;
             MQPUT1(Hconn, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 6:  pmo.Options = MQOO_ALTERNATE_USER_AUTHORITY;
             MQPUT1(Hconn, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 7:  md.Persistence = MQPER_PERSISTENCE_AS_Q_DEF;
             MQPUT1(Hconn, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 8:  MQPUT1(Hconn, &od, &md, NULL, 11, msgBuffer,
                   &CompCode, &Reason);            break;
    case 11: MQPUT1(Hconn, NULL, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 12: od.ObjectType = 9999;
             MQPUT1(Hconn, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           break;
    case 13: pmo.Options = MQOO_BROWSE;
             MQPUT1(Hconn, &od, &md, &pmo, 11, msgBuffer,
                    &CompCode, &Reason);           }

  PrintCodes("MQPUT1", Reason, ETable[CaseNum-1].EReason);

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }



void error_test_inq(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  O_options = MQOO_INQUIRE;        // MQOPEN options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  MQLONG  Selectors[2]={MQIA_Q_TYPE, 0};   // MQINQ Selectors                   //d26352
  MQLONG  IntAttrs[] = {9999};             // Integer returns
  char    ChrAttrs[50] = {0};              // String returns
  unsigned int i;

  static sETable ETable[] = {
    {"Invalid Hconn"                , MQRC_HCONN_ERROR              },    // 1
    {"Unusable Hobj"                , MQRC_HOBJ_ERROR               },    // 2
    {"Queue not opened for inquiry" , MQRC_NOT_OPEN_FOR_INQUIRE     },    // 3
    {"Negative CharAttrLength"      , MQRC_CHAR_ATTR_LENGTH_ERROR   },    // 4
    {"NULL CharAttrs"               , MQRC_CHAR_ATTRS_ERROR         },    // 5
    {"Negative IntAttrCount"        , MQRC_INT_ATTR_COUNT_ERROR     },    // 6
    {"NULL IntAttrs"                , MQRC_INT_ATTRS_ARRAY_ERROR    },    // 7
    {"SelectorCount > 256"          , MQRC_SELECTOR_LIMIT_EXCEEDED  },    // 8
    {"Negative SelectorCount"       , MQRC_SELECTOR_COUNT_ERROR     },    // 9
    {"Unsupported Selector"         , MQRC_SELECTOR_ERROR           },    // 10
    {"CharAttr buffer too small"    , MQRC_CHAR_ATTRS_TOO_SHORT     },    // 11
    {"Integer array too small"      , MQRC_INT_ATTR_COUNT_TOO_SMALL },    // 12
    {"Selector not valid for object", MQRC_SELECTOR_NOT_FOR_TYPE    },    // 13
    {"Inactive Queue Manager"       , MQRC_Q_MGR_NOT_ACTIVE         },    // 14
    {"Undefined Queue Manager"      , MQRC_Q_MGR_NOT_AVAILABLE      },    // 15
    {"QMGR using a Queue Selector"  , MQRC_SELECTOR_ERROR           } };  // 16

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQINQ");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_inq(QMgr, QName, i);
    return; }

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  outc(sf("MQINQ Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  switch (CaseNum) {
    case 3:  O_options = MQOO_OUTPUT;
             break;
    case 14: outc("Queue manager must not be active");
             MQINQ(4, 0, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,                //d26352
                   &CompCode, &Reason);
             PrintCodes("MQINQ", Reason, ETable[CaseNum-1].EReason);
             return;
    case 15: outc("Queue manager must not be defined");
             MQINQ(4, 0, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,                //d26352
                   &CompCode, &Reason);
             PrintCodes("MQINQ", Reason, ETable[CaseNum-1].EReason);
             return;
    case 16: od.ObjectType = MQOT_Q_MGR;
             od.ObjectName[0] = '\0';
             strcpy(od.ObjectQMgrName, ((MQP_MEM *)cinfc(CINFC_READ,
                    CINFC_CMMMQT)) -> QManagerName); }

  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);
  MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
  CheckError("MQOPEN", Reason);

  switch (CaseNum) {
    case 1:  MQINQ(-1, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 2:  MQINQ(Hconn, MQHO_UNUSABLE_HOBJ, 1, Selectors, 1, IntAttrs,
                   0, ChrAttrs, &CompCode, &Reason);          break;
    case 3:  MQINQ(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);
    case 4:  MQINQ(Hconn, Hobj, 1, Selectors, 1, IntAttrs, -1, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 5:  Selectors[0] = MQCA_CREATION_DATE;
             MQINQ(Hconn, Hobj, 1, Selectors, 0, IntAttrs, 1, NULL,
                   &CompCode, &Reason);                       break;
    case 6:  MQINQ(Hconn, Hobj, 1, Selectors, -1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 7:  MQINQ(Hconn, Hobj, 1, Selectors, 1, NULL, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 8:  MQINQ(Hconn, Hobj, 257,Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 9:  MQINQ(Hconn, Hobj, -1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 10: Selectors[0] = 9999;
             MQINQ(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 11: Selectors[0] = MQCA_CREATION_DATE;
             MQINQ(Hconn, Hobj, 1, Selectors, 0, IntAttrs, 1, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 12: Selectors[1] = MQIA_INHIBIT_PUT;
             MQINQ(Hconn, Hobj, 2, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 13: outc("(Selector is for a remote queue)");
             Selectors[0] = MQCA_REMOTE_Q_MGR_NAME;
             MQINQ(Hconn, Hobj, 1, Selectors, 0, IntAttrs, 50, ChrAttrs,
                   &CompCode, &Reason);                       break;
    case 16: MQINQ(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);                       }

  PrintCodes("MQINQ", Reason, ETable[CaseNum-1].EReason);

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);
  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }




void error_test_set(char *QMgr, char *QName, unsigned int CaseNum) {
  MQOD    od = {MQOD_DEFAULT};             // Object Descriptor
  MQHCONN Hconn = MQHC_UNUSABLE_HCONN;     // Connection Handle
  MQHOBJ  Hobj = MQHO_UNUSABLE_HOBJ;       // Object Handle
  MQLONG  O_options = MQOO_SET;            // MQOPEN options
  MQLONG  CompCode;                        // completion code
  MQLONG  Reason;                          // reason code
  MQLONG  Selectors[]={MQIA_INHIBIT_PUT};  // MQSET Selectors
  MQLONG  IntAttrs[] = {9999};             // Integer returns
  char    ChrAttrs[2] = {0};               // String returns
  unsigned int i;

  static sETable ETable[] = {
    {"Invalid Hconn"                 , MQRC_HCONN_ERROR             },    // 1
    {"Unusable Hobj"                 , MQRC_HOBJ_ERROR              },    // 2
    {"Queue not opened for setting"  , MQRC_NOT_OPEN_FOR_SET        },    // 3
    {"Negative CharAttrLength"       , MQRC_CHAR_ATTR_LENGTH_ERROR  },    // 4
    {"Bad MQIA_INHIBIT_PUT selector" , MQRC_INHIBIT_VALUE_ERROR     },    // 5
    {"Negative IntAttrCount"         , MQRC_INT_ATTR_COUNT_ERROR    },    // 6
    {"NULL IntAttrs"                 , MQRC_INT_ATTRS_ARRAY_ERROR   },    // 7
    {"SelectorCount > 256"           , MQRC_SELECTOR_LIMIT_EXCEEDED },    // 8
    {"Negative SelectorCount"        , MQRC_SELECTOR_COUNT_ERROR    },    // 9
    {"Unsupported Selector"          , MQRC_SELECTOR_ERROR          },    // 10
    {"Inactive queue manager"        , MQRC_Q_MGR_NOT_ACTIVE        },    // 11
    {"Undefined Queue Manager"       , MQRC_Q_MGR_NOT_AVAILABLE     } };  // 12

  if (CaseNum > sizeof(ETable)/sizeof(sETable))
    Error("Test case not found for MQSET");
  if (CaseNum == 0) {
    for (i = 1; i <= sizeof(ETable)/sizeof(sETable); i++)
      error_test_set(QMgr, QName, i);
    return; }

  strncpy(od.ObjectName, QName, MQ_Q_NAME_LENGTH);

  outc(sf("MQSET Case %d: %s", CaseNum, ETable[CaseNum-1].Text));

  switch (CaseNum) {
    case  3: O_options = MQOO_OUTPUT;
             MQCONN(QMgr, &Hconn, &CompCode, &Reason);
             CheckError("MQCONN", Reason);
             MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
             CheckError("MQOPEN", Reason);
             MQSET(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);
             PrintCodes("MQSET", Reason, ETable[CaseNum-1].EReason);
             return;
    case 11: outc("Queue manager must not be active");
             MQSET(4, 0, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,                //d26352
                   &CompCode, &Reason);
             PrintCodes("MQSET", Reason, ETable[CaseNum-1].EReason);
             return;
    case 12: outc("Queue manager must not be defined");
             MQSET(4, 0, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,                //d26352
                 &CompCode, &Reason);
             PrintCodes("MQSET", Reason, ETable[CaseNum-1].EReason);
             return; }

  MQCONN(QMgr, &Hconn, &CompCode, &Reason);
  CheckError("MQCONN", Reason);
  MQOPEN(Hconn,&od,O_options,&Hobj,&CompCode,&Reason);
  CheckError("MQOPEN", Reason);

  switch (CaseNum) {
    case 1:  MQSET(-1, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);           break;
    case 2:  MQSET(Hconn, MQHO_UNUSABLE_HOBJ, 1, Selectors, 1, IntAttrs,
                   0, ChrAttrs, &CompCode, &Reason); break;
    case 4:  MQSET(Hconn, Hobj, 1, Selectors, 1, IntAttrs, -1, ChrAttrs,
                   &CompCode, &Reason);           break;
    case 5:  IntAttrs[0] = 9999;
             MQSET(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, NULL,
                   &CompCode, &Reason);           break;
    case 6:  MQSET(Hconn, Hobj, 1, Selectors, -1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);           break;
    case 7:  MQSET(Hconn, Hobj, 1, Selectors, 1, NULL, 0, ChrAttrs,
                   &CompCode, &Reason);           break;
    case 8:  MQSET(Hconn, Hobj, 257, Selectors, 1, IntAttrs, 0,
                   ChrAttrs, &CompCode, &Reason); break;
    case 9:  MQSET(Hconn, Hobj, -1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);           break;
    case 10: Selectors[0] = 9999;
             MQSET(Hconn, Hobj, 1, Selectors, 1, IntAttrs, 0, ChrAttrs,
                   &CompCode, &Reason);           }

  PrintCodes("MQSET", Reason, ETable[CaseNum-1].EReason);

  MQCLOSE(Hconn, &Hobj, 0, &CompCode, &Reason);
  CheckError("MQCLOSE", Reason);
  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("MQDISC", Reason); }


