/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      (C) Copyright IBM Corp. 2006                                 */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*                                                                   */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Segment Name:  MQ_Request                                         */
/*                                                                   */
/* Abstract:        ZTEST MQS1 REQUEST...                            */
/*                                                                   */
/* Description:  This driver may be used to send a request type      */
/*               message to a destination queue manager, and         */
/*               queue.  The user must also specify the reply-to     */
/*               queue, to which the original message will be        */
/*               echoed.                                             */
/*                                                                   */
/*               Five parameters are expected as input as part       */
/*               of the IPRSE pointer which is passed from QMQZ      */
/*               the MQ driver parser.  First, the queue manager     */
/*               to send the messages to.  Second, the queue, known  */
/*               to the aforementioned queue manager, to put the     */
/*               messages to.  Third, the reply-to queue to which    */
/*               the request's replies should be returned.  Fourth,  */
/*               the number of messages to send.  And finally,       */
/*               the size of the messages to send.                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE REQUIREMENTS:                                           */
/*                                                                   */
/* Dependencies:  None.                                              */
/*                                                                   */
/* Restrictions:  MQ Server must be running on your TPF system.      */
/*                                                                   */
/*-------------------------------------------------------------------*/
/*                                                                   */
/* For each Entry Point:                                             */
/*                                                                   */
/* Input:                                                            */
/*                                                                   */
/*  Parameters:    pointer to IPRSE structure, listing the           */
/*                 parsed input parameters                           */
/*                                                                   */
/*  Function return value:                                           */
/*  Normal:  None.                                                   */
/*  Error:   Error message printed out to PRC.                       */
/*                                                                   */
/*********************************************************************/
#include <qmqz.h>


void QMQZ_Request(sInput *Input) {
  int            defrcount, getloop, i;
  int            message_count, message_size;
  int            ecb_count = 1;
  char           queue_manager[MQ_Q_NAME_LENGTH];
  char           queue_name[MQ_Q_NAME_LENGTH];
  char           queue_reply[MQ_Q_NAME_LENGTH];
  char           zmsg[100];
  char          *buffer;                                           // message buffer to send
  MQOD           Request_obj = {MQOD_DEFAULT};
  MQOD           Reply_obj = {MQOD_DEFAULT};
  MQMD           md = {MQMD_DEFAULT};                              // message descriptor
  MQPMO          pmo = {MQPMO_DEFAULT};
  MQGMO          gmo = {MQGMO_DEFAULT};
  gmo.Version  = MQGMO_VERSION_2;
  MQHOBJ         Hobj_reply;                                       // object handle
  MQHOBJ         Hobj_request;                                     // object handle
  MQLONG         CompCode;                                         // MQ API completion code
  MQLONG         Reason;                                           // MQ API reason code
  MQLONG         replylen;
  MQHCONN        Hconn;                                            // connection handle

  // first thing to do is pick up all the parameters that were input by the user
  Input[1].Value.CopyTo(queue_manager);
  Input[2].Value.CopyTo(queue_name);
  Input[3].Value.CopyTo(queue_reply);
  message_count = Input[4].Value.ToInt();
  if (message_count > MAXMSG_COUNT)
    Error(sf("NMSG field is greater than max = %d", MAXMSG_LENGTH));
  message_size = Input[5].Value.ToInt();
  if (message_size > MAXMSG_LENGTH)
    Error(sf("LENGTH field is greater than max = %d", MAXMSG_LENGTH));
  if (Input[6].Param == "ECB-d*") {                                 // Spawn ECBs
    ecb_count = Input[6].Value.ToInt();
    CheckNumEcbs(ecb_count);
    Input++; }
  if (Input[6].Param == "(NO)PERsis") {
    if (Input[6].Value[0] == 'N')
      md.Persistence = MQPER_NOT_PERSISTENT;
    Input++; }
  else md.Persistence = MQPER_PERSISTENT;

  // initialize the zmsg with the functional message that was input, but setting ECB to 1
  if (md.Persistence == MQPER_PERSISTENT)
    sprintf(zmsg,"ZTEST MQS1 REQUEST %s %s %s N-%d LEN-%d ECB-1 PER",
      queue_manager,queue_name,queue_reply,message_count,message_size);
  else
    sprintf(zmsg,"ZTEST MQS1 REQUEST %s %s %s N-%d LEN-%d ECB-1 NOPER",
      queue_manager,queue_name,queue_reply,message_count,message_size);
  // create as many extra ECBs as we need to complete req.
  CreateECBs(ecb_count - 1, zmsg);

  // OK, now we have all the parameters, lets start doing some MQ
  if (memcmp(&(ecbptr()->ebw000), "VIA CREEC", 9) != 0)
    out("QMQZ03 : MQ REQUEST driver started");

  MQCONN(queue_manager, &Hconn, &CompCode, &Reason);
  CheckError("QMQZ03: MQCONN", Reason);

  strncpy(Request_obj.ObjectName, queue_name, sizeof(queue_name));  // Prep for open
  MQOPEN(Hconn, &Request_obj, MQOO_OUTPUT, &Hobj_request,
         &CompCode, &Reason);
  CheckError("QMQZ03: MQOPEN", Reason);

  // Now we need to open the reply-to queue for receiving reply msgs
  strncpy(Reply_obj.ObjectName, queue_reply, sizeof(queue_reply));
  MQOPEN(Hconn, &Reply_obj, MQOO_INPUT_AS_Q_DEF, &Hobj_reply,
          &CompCode, &Reason);
  CheckError("QMQZ03: MQOPEN on reply queue", Reason);

  // save reply-to queue name, might be dynamic, but we probably wont allow this
  strncpy(queue_reply, Reply_obj.ObjectName, MQ_Q_NAME_LENGTH);

  // setup buffer with data for message...
  if ((buffer = (char *)malloc(message_size+1)) == NULL)
    Error("No malloc space left!");

  // now we need to put a message to the request queue, one at a time, and get
  // the reply off, until we have done this the desired number of times (message_count)
  for (i = 0; i < message_count; i++) {
    memset(buffer, 'J', message_size);
    buffer[message_size] = '\0';
    memcpy(buffer, "QMQZ-", 5);
    md.MsgType = MQMT_REQUEST;                                       // message is a request
    memcpy(md.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);
    md.CodedCharSetId = CCSID_EBCDIC;
    strncpy(md.ReplyToQ, queue_reply, MQ_Q_NAME_LENGTH);
    memset(md.ReplyToQMgr, ' ', MQ_Q_MGR_NAME_LENGTH);

    MQPUT(Hconn, Hobj_request, &md, &pmo, message_size,
          buffer, &CompCode, &Reason);
    CheckError("MQPUT", Reason);

    getloop = 1;
    defrcount = 0;
    while (getloop) {
      // setup message descriptor for getting message from reply-to queue
      gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
      memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
      memcpy(md.CorrelId, MQCI_NONE, sizeof(md.CorrelId));

      MQGET(Hconn, Hobj_reply, &md, &gmo, message_size,
            (MQBYTE *)buffer, &replylen, &CompCode, &Reason);
      Trace("MQGET");
      switch (Reason) {
        case MQRC_NONE:                   getloop = 0; break;
        case MQRC_TRUNCATED_MSG_ACCEPTED: getloop = 0; break;
        case MQRC_NO_MSG_AVAILABLE:
          sleep(1);
          defrcount++;
          if (defrcount >= MAXDELAY) {                               // Try MQGET until time is up
            oute("QMQZ03 : No replies received...");
            getloop = 0; }
          break;
        default: Error("MQGET"); }

      // only accept REPLY type messages
      if (md.MsgType != MQMT_REPLY && Reason != MQRC_NO_MSG_AVAILABLE) {
        defrcount = 0;
        getloop = 1; }
    }  // end WHILE loop

    // if the reply message isn't received in time, break out from the for loop too
    if ((getloop == 0) && (Reason == MQRC_NO_MSG_AVAILABLE))
       break;

  }  // end of FOR loop

  MQCLOSE(Hconn, &Hobj_request, 0, &CompCode, &Reason);              // close the two queues...
  CheckError("QMQZ03: MQCLOSE", Reason);

  // only try to close reply queue if different from request queue
  if (Hobj_request != Hobj_reply) {
    MQCLOSE(Hconn,&Hobj_reply, 0, &CompCode, &Reason);
    CheckError("QMQZ03: MQCLOSE", Reason); }

  MQDISC(&Hconn, &CompCode, &Reason);
  CheckError("QMQZ03: MQDISC", Reason);

  outc(sf("QMQZ03 : MQ REQUEST driver ended. %d request messages sent.", i));
  oute("QMQZ03 : And the replied messages are received properly."); }


/*********************************************************************/
/*                                                                   */
/* Segment Name:  MQ_Reply                                           */
/*                                                                   */
/* Abstract:        This driver may be used to send reply messages   */
/*                  to MQ request messages.                          */
/*                                                                   */
/*********************************************************************/
void QMQZ_Reply(sInput *Input) {
  int            message_count, message_size, ecb_count;
  int            i, defrcount;                            // loop index
  int            getloop = 1;
  char           zmsg[100];
  char           queue_name[MQ_Q_NAME_LENGTH];
  char           queue_manager[MQ_Q_NAME_LENGTH];
  char           *buffer, *convq, *convqm;                //message buffer to send
  MQOD           Output_obj = {MQOD_DEFAULT};
  MQOD           Input_obj = {MQOD_DEFAULT};
  MQMD           model_md = {MQMD_DEFAULT};                        // message descriptor
  MQMD           md;
  MQPMO          pmo = {MQPMO_DEFAULT};
  MQGMO          gmo = {MQGMO_DEFAULT};
  gmo.Version  = MQGMO_VERSION_2;
  MQLONG         CompCode, Reason;                                 // completion codes
  MQLONG         replylen;
  MQHOBJ         Hobj_input;                                       // object handle
  MQHCONN        Hconn;                                            // connection handle

  // first thing to do is pick up all the parameters that were input by the user
  Input[1].Value.CopyTo(queue_manager);
  Input[2].Value.CopyTo(queue_name);
  message_count = Input[3].Value.ToInt();
  if (message_count > MAXMSG_COUNT)
    Error(sf("QMQZ03 : NMSG field is greater than max = %d",
             MAXMSG_COUNT));
  message_size = Input[4].Value.ToInt();
  if (message_size > MAXMSG_LENGTH)
    Error(sf("QMQZ03 : LENGTH field is greater than max = %d",
             MAXMSG_LENGTH));

  if (Input[5].Param == "ECB-d*") {                                 // Spawn ECBs
    ecb_count = Input[5].Value.ToInt();
    Input++;
  }
  else
    ecb_count = 1;

  if (ecb_count > MAXECB || ecb_count <= 0)
    Error(sf("QMQZ03 : ECB field must be greater than "\
                    "0 and less than max = %d",MAXECB));
  // initialize the zmsg with the functional message that was input, but setting ECB to 1
  sprintf(zmsg,"ZTEST MQS1 REPLY %s %s N-%d LEN-%d ECB-1",
      queue_manager,queue_name,message_count,message_size);
  // create as many extra ECBs as we need to complete req.
  CreateECBs(ecb_count - 1, zmsg);

  // OK, now we have all the parameters, lets start doing some MQ
  if (memcmp(&(ecbptr()->ebw000), "VIA CREEC", 9) != 0)
    out(sf("QMQZ03 : MQ REPLY driver started for %d ECB(s)",
            ecb_count));

  MQCONN(queue_manager, &Hconn, &CompCode, &Reason);
  CheckError("QMQZ03: MQCONN", Reason);

  strncpy(Input_obj.ObjectName, queue_name, sizeof(queue_name));     // Prep for open
  MQOPEN(Hconn, &Input_obj, MQOO_INPUT_AS_Q_DEF, &Hobj_input,
         &CompCode, &Reason);
  CheckError("QMQZ03: MQOPEN", Reason);

  if (!(buffer = (char *)calloc(1, message_size)))
    Error("QMQZ03 : Not enough malloc space for message buffer");

  // now we need to get a message from the input queue, one at a
  // time, and put the reply to the reply-to queue, until the queue is empty
  defrcount = 0;
  for (i = 0; i < message_count; i++) {
    // setup message descriptor for getting message from the input queue
    gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG;
    md = model_md;
    MQGET(Hconn, Hobj_input, &md, &gmo, message_size,
          (MQBYTE *)buffer, &replylen, &CompCode, &Reason);
    Trace("MQGET");
    switch (Reason) {
      case MQRC_NONE:
        defrcount = 0;
        break;
      case MQRC_NO_MSG_AVAILABLE: sleep(1);  defrcount++;
        if (defrcount >= MAXDELAY) {                                 // Try until time is up
          out(sf("%d requests were received before time expired", i));
          getloop = 0; }
        else
          i--;
        break;
      default:
        if (CompCode == MQCC_FAILED)
          Error("MQGET");
        defrcount = 0;
        break; }

    if (getloop == 0)
    // we are told to break out of the for loop
      break;

    // no message is available, go back to the top of the loop to do MQGET
    if (defrcount > 0) continue;

    // only try to reply to request messages, else, toss them...
    if (md.MsgType != MQMT_REQUEST && Reason != MQRC_NO_MSG_AVAILABLE){
      i--;
      out("QMQZ03 : Not a Request message...Toss it...");
      continue; }

    // setup the message descriptor
    md.MsgType = MQMT_REPLY;
    md.Report = MQRO_NONE;
    memcpy(md.Format, MQFMT_STRING, MQ_FORMAT_LENGTH);
    if (replylen > message_size) replylen = message_size;

    // copy reply-to queue name to the object descriptor
    if (md.CodedCharSetId != CCSID_EBCDIC) {
      buffer[replylen] = '\0';
      buffer = CodepageConvert(CCSID_ASCII, CCSID_EBCDIC, buffer);
      if (buffer == NULL)
        Error("QMQZ03 buffer is null"); }

    convq = md.ReplyToQ;
    convqm = md.ReplyToQMgr;

    md.CodedCharSetId = CCSID_EBCDIC;
    strncpy(Output_obj.ObjectName, convq, MQ_Q_NAME_LENGTH);
    strncpy(Output_obj.ObjectQMgrName, convqm, MQ_Q_NAME_LENGTH);

    MQPUT1(Hconn, &Output_obj, &md, &pmo, replylen, buffer,
           &CompCode, &Reason);
    CheckError("QMQZ03: MQPUT1", Reason); }   // end of for message_count loop

    MQCLOSE(Hconn, &Hobj_input, 0, &CompCode, &Reason);
    CheckError("QMQZ03: MQCLOSE", Reason);

    MQDISC(&Hconn, &CompCode, &Reason);
    CheckError("QMQZ03: MQDISC", Reason);

    out(sf("QMQZ03 : MQ REPLY driver ended. %d request messages are replied", i)); }


/*********************************************************************/
/*                                                                   */
/* SUBROUTINE:                                                       */
/*                                                                   */
/* Name:            CodepageConvert()                                */
/*                                                                   */
/* Abstract:        Convert character string from one codepage       */
/*                  to another.                                      */
/*                                                                   */
/*********************************************************************/
char *CodepageConvert(u_short conv_from,u_short conv_to,char *in_data) {
    size_t   outleft = MAXMSG_LENGTH;
    size_t   datalength;
    char    *outbuf_ptr, *outbuf;
    iconv_t  cd;

    if ((cd = iconv_open(ccsid_conv(&conv_to)->csname,
              ccsid_conv(&conv_from)->csname)) == (iconv_t)(-1)) {   // get a conversion descriptor for iconv()
      out("CodepageConvert: error in iconv_open, "\
          "Returning original data");
      return in_data; }

    datalength = (size_t)strlen(in_data);

    if ((outbuf = (char *)calloc(1, datalength+1)) == NULL)
      Error("No malloc space available");

    outbuf_ptr = outbuf;
    if (iconv(cd,&in_data,&datalength,&outbuf_ptr,&outleft) == (size_t) -1)   // convert the data
      Error("CodepageConvert: error in iconv");

    iconv_close (cd);                                                // close the conversion descriptor
    return outbuf; }






