/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2006, 2023                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or dislosure restricted by GSA ADP Schedule Contract         */
/*      with IBM Corp.                                               */
/*********************************************************************/
/* This file is provided on an "as is" basis.  There are no          */
/* warranties, express or implied, including the implied warranties  */
/* of merchantability and fitness for a particular purpose.          */
/* IBM has no obligation to provide service, defect correction, or   */
/* any maintenance for the file.  IBM has no obligation to supply    */
/* any updates or enhancements for the file to you even if such are  */
/* or later become available.                                        */
/*********************************************************************/
/* Change History                                                    */
/*  DEFECT  DATE      DESCRIPTION                                    */
/*  d26352  20180810  Compiler compatibility updates                 */
/*  d26841  20230927  IBM MQ support for 64-bit                      */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* FUNCTION NAME.....MQS1                                            */
/* FUNCTION TYPE.....TPF                                             */
/* SEGMENT NAME....  QMQZ05                                          */
/*                                                                   */
/* DESCRIPTION:                                                      */
/*   QMQZ05 Contains functions to gather information from the MQ     */
/*   checkpointing                                                   */
/*                                                                   */
/*********************************************************************/

#include <qmqz.h>

#undef  MQ_REC_ID
#define MQ_REC_ID "\xFC\x42"

void QMQZ_CheckPoint(sInput *Input)  {
  unsigned int k;                                                    // dummy variable for face_facs
  int FacsError = 0;                                                 // error variable
  int OrdNumber = 0;                                                 // ControlRec=0, AvailRec=1

  if (Input[1].Param == "NUMRecords")
    OrdNumber = 1;                                                   // CPNumRecords() Wants the AvailRec

  if (levtest(D8))
     relcc(D8);                                                      // All the CP_functions need a free data level
  FacsError = face_facs(OrdNumber, MQ_CHKPT_REC_TYPE, 0, D8, &k);    // Store Control Record file address on D8
  CheckError("face_facs", FacsError);

  if (Input[1].Param == "Messages")
    CPNumMessages();
  else if (Input[1].Param == "NUMQueues")
    CPNumQueues();
  else if (Input[1].Param == "NUMChannels")
    CPNumChannels();
  else if (Input[1].Param == "NUMRecords")
    CPNumRecords();
  else if (Input[1].Param == "ALLQueuenames")
    CPFindAllQueues();
  else if (Input[1].Param == "ALLChannelnames")
    CPFindAllChannels();
  else if (Input[1].Param == "FIND")
    CPFindName(Input[2].Value.C()); }                                // send name to find





void CPNumMessages()  {                                              // Count all checkpointed messages
  unsigned int i;                                                    // loop counter
  int TotalMessages = 0;                                             // return value
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  outc("Msgs Before Sweep / After Sweep / Queue Name");
  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level
  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++)                            // All entries in this Control Record
     if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_queue)    // Entry contains a queue
      CountMessages(ControlRec->recordEntry[i].queue,&TotalMessages);// Sum messages in all queues
    NextControlRec(&ControlRec); }                                   // go to next record
  oute(sf("There are %d Messages Checkpointed",TotalMessages)); }






void CPNumRecords()  {                                               // Count number of used records
  MQAvailRecord *AvailRec;                                           // Record of available records
  int InUse = 0;                                                     // return value
  int Recycled = 0;                                                  // other return value
  int i;                                                             // loop counter

  AvailRec = (MQAvailRecord *)finwc(D8);                             // Core block already set up

  for (i=0; i < AvailRec->num_entries; i++)  {
    if (AvailRec->AvailArray[i] == MQ_AVAIL_INUSE)                   // Record in use
      InUse++;                                                       // Increment counter
    else if (AvailRec->AvailArray[i] == MQ_AVAIL_RECYCLE)            // Record being recycled
      Recycled++;  }

  // Repeat for each available record in chain
  while (AvailRec->header.std_header.stdfch) {                                  //d26352
    unsigned int address = AvailRec->header.std_header.stdfch;
    if (levtest(D8)) relcc(D8);                                      // Release D8 for Incoming record
    AvailRec = (MQAvailRecord *)                                     // Get next available record
                  find_record(D8, &address, MQ_REC_ID, '\0', NOHOLD);
    if (AvailRec == NULL) Error("QMQZ05:  find_record() Error");
    for (i=0; i < AvailRec->num_entries; i++)  {                     // Count records in use
      if (AvailRec->AvailArray[i] == MQ_AVAIL_INUSE)
        InUse++;
      else if (AvailRec->AvailArray[i] == MQ_AVAIL_RECYCLE)
        Recycled++; } }

  out(sf("%d Records are in use", InUse));
  if (Recycled)
    out(sf("And %d are being recycled",Recycled)); }




void CPNumQueues()  {                                                // Number of checkpointed queues
  unsigned int i;                                                    // loop counter
  int TotalQueues = 0;                                               // return value
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level

  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++)                            // All entries in this Control Record
      if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_queue)   // Entry contains a queue
       TotalQueues++;                                                // Increment counter
    NextControlRec(&ControlRec); }                                   // go to next record
  out(sf("There are %d Queues Checkpointed",TotalQueues)); }



void CPNumChannels()  {                                              // Number of checkpointed channels
  unsigned int i;                                                    // loop counter
  int TotalChannels = 0;                                             // return value
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level
  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++)                            // All entries in this Control Record
      if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_channel)
        TotalChannels++;
    NextControlRec(&ControlRec); }                                   // go to next record
  out(sf("There are %d Channels Checkpointed",TotalChannels)); }



void CPFindAllQueues()  {                                            // outputs all queues checkpointed
  unsigned int i;                                                    // loop counter
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level
  outc("The Following Queues are Checkpointed: ");
  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++)                            // All entries in this Control Record
      if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_queue)   // Entry contains a queue
       outc(ControlRec->recordEntry[i].queue.QueueName);             // Output Queue Name
    NextControlRec(&ControlRec); }                                   // go to next record
  oute("End of list"); }



void CPFindAllChannels()  {                                          // outputs all channels checkpointed
  unsigned int i;                                                    // loop counter
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level
  outc("The Following Channels are Checkpointed: ");
  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++)                            // All entries in this Control Record
      if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_channel) // Entry contains a queue
       outc(ControlRec->recordEntry[i].channel.ChannelName);         // Output Channel Name
    NextControlRec(&ControlRec); }                                   // go to next record
  oute("End of list"); }



void CPFindName(char Name[MQ_Q_NAME_LENGTH])  {
  unsigned int i;                                                    // loop counter
  int found = 0;                                                     // return value
  char output[250];                                                  // output string
  char N[MQ_Q_NAME_LENGTH];                                          // Name to find
  char CN[MQ_CHANNEL_NAME_LENGTH];                                   // Channel name to find
  MQCntrlRecord *ControlRec;                                         // MQ Checkpointing Control Record

  ControlRec = (MQCntrlRecord *)finwc(D8);                           // Load Control Record off data level
  blankPad(N, Name, MQ_Q_NAME_LENGTH);                               // N will be padded with spaces to 48 characters
  blankPad(CN, Name, MQ_CHANNEL_NAME_LENGTH);                        // CN will be padded with spaces to 20 characters

  while (ControlRec != NULL) {
    for (i=0; i<=MQ_MAX_CHKPT_ENTRY; i++) {                          // All entries in this Control Record
      if (ControlRec->recordEntry[i].objecttype == CHKPTYPE_queue) { // Entry contains a queue
        snprintf(output, MQ_Q_NAME_LENGTH+1, "%.*s", (int) MQ_Q_NAME_LENGTH,    //d26841
                ControlRec->recordEntry[i].queue.QueueName);
        if (!strncmp(N, output, MQ_Q_NAME_LENGTH)) {                 // Determine if names match
          out(sf("Queue %s Found in Checkpoint",Name));                         //d26841
          found++; } }                                               // queue found
      else if
      (ControlRec->recordEntry[i].objecttype == CHKPTYPE_channel) {  // See if name is in channel
        snprintf(output, MQ_CHANNEL_NAME_LENGTH+1, "%.*s", (int) MQ_CHANNEL_NAME_LENGTH,
                ControlRec->recordEntry[i].channel.ChannelName);
        if (!strncmp(Name, output, MQ_CHANNEL_NAME_LENGTH)) {        // Determine if names match
          out(sf("Channel %s Found in Checkpoint",Name));                       //d26841
          found++; } } }                                             // Channel found
    NextControlRec(&ControlRec); }                                   // go to next record
  if (!found) {
    out(sf("%s Not Found in Checkpoint",Name)); } }                             //d26841



void NextControlRec(MQCntrlRecord **pCurrRec)  {                     // set up next record using D8
  MQCntrlRecord* CurrRec = *pCurrRec;                                // current record
  unsigned int address = CurrRec->header.std_header.stdfch;          // File Address = next in chain
  if (address == 0) {                                                           //d26352
    *pCurrRec = NULL;
    return; }
  if (levtest(D8)) relcc(D8);                                        // Release D8 for Incoming record
  CurrRec = (MQCntrlRecord *)
               find_record(D8, &address, MQ_REC_ID, '\0', NOHOLD);   // Load record into memory
  if (CurrRec == NULL)
    Error("QMQZ05:  find_record() Error");
  pCurrRec = &CurrRec; }



void CountMessages(MQChkptQEntry Q, int *total)  {                   // Returns # messages in checkpointed queue
  int tot1=0, tot2=0;
  if (Q.fileaddr_1) CountMsgList(Q.fileaddr_1, &tot1);               // Messages in first chain
  if (Q.fileaddr_2) CountMsgList(Q.fileaddr_2, &tot2);               // Messages in second chain
  outc(sf("%4d %4d   %s", tot1, tot2, Q.QueueName));
  *total += tot1+tot2; }





void CountMsgList(unsigned int FileAddr, int *tot)  {
  unsigned int i;                                                    // loop counter
  MQChkptRecord *CurrentRec;                                         // current record

  if (levtest(D9)) relcc(D9);                                        // Find space for incoming record
  CurrentRec = (MQChkptRecord *)                                     // Load new record into memory
                find_record(D9, &FileAddr, MQ_REC_ID, '\0', NOHOLD);
  if (CurrentRec == NULL) Error("QMQZ05:  find_record() Error");
  for (i=0; i < MQ_SWB_PER_FILE; i++)  {                             // Go through SWBs
    if ((CurrentRec->SWBentry[i].segment == CKPSWB_FIC) ||           // Count all messages first in chain
        (CurrentRec->SWBentry[i].segment == CKPSWB_OIC))             // And only in chain
      (*tot)++;  }
  // count rest of message records for queue
  if (CurrentRec->header.std_header.stdfch)                                     //d26352
    CountMsgList(CurrentRec->header.std_header.stdfch, tot);
}                                                                               //d26352

